EnableExplicit

CompilerIf #PB_Compiler_OS = #PB_OS_Windows 
	
	ImportC "msvcrt.lib"
		CompilerIf #PB_Compiler_Unicode
			strtok(*string,*seps) As "_wcstok"
		CompilerElse
			strtok(*string,*seps) As "_strtok"
		CompilerEndIf
	EndImport 
	
CompilerElse 
	
	ImportC ""
		system(cmd.p-utf8)
	EndImport
	
CompilerEndIf

Enumeration #PB_Event_FirstCustomValue
	#EventBeginProcessing
	#EventProcessingFinished
	#EventNewLogMessage
EndEnumeration

Global ShitHappend

Global NewList FileToCopy.s()

Global NewList IncludeBinaryFile.s()

Procedure WaitForFile(Filename.s)
	
	Protected Counter = 0
	
	Repeat
		If FileSize(Filename) > 5
			Break	
		EndIf
		Delay(200)
		Counter + 1
		If Counter > 100
			Break
		EndIf
	ForEver
	
EndProcedure

Procedure.s LoadTextFile(TextFilename.s)
	
	Protected FF
	Protected ReturnValue.s
	Protected StringFormat
	
	WaitForFile(TextFilename)
	
	If FileSize(TextFilename) <= 0 : ProcedureReturn "" : EndIf
	FF = ReadFile(#PB_Any, TextFilename)
	If FF = 0 : ProcedureReturn "" : EndIf
	StringFormat = ReadStringFormat(FF)
	ReturnValue = ReadString(FF, #PB_File_IgnoreEOL | StringFormat)
	CloseFile(FF)
	
	ProcedureReturn ReturnValue
	
EndProcedure

Procedure SaveTextFile(TextFileContent.s, TextFileName.s, WriteStringFormat = #PB_Ascii)
	
	Protected FF
	FF = CreateFile(#PB_Any, TextFileName)
	If FF 
		WriteString(FF, TextFileContent, WriteStringFormat)
		CloseFile(FF)
	EndIf
	
EndProcedure

Procedure.s GetExePath()
	Protected ExePath.s = GetPathPart(ProgramFilename())
	If LCase(ExePath) = LCase(GetTemporaryDirectory()) : ExePath = GetCurrentDirectory() : EndIf
	If Right(ExePath, 1) <> "/" : ExePath + "/" : EndIf
	ProcedureReturn ExePath
EndProcedure

Procedure ListFiles(EntryPath.s)
	
	Protected UsedDirectory
	Protected EntryType
	Protected EntryName.s
	
	If Right(EntryPath, 1) <> "/" : EntryPath + "/" : EndIf
	
	UsedDirectory = ExamineDirectory(#PB_Any, EntryPath, "*.*")
	
	If UsedDirectory
		
		While NextDirectoryEntry(UsedDirectory)
			
			EntryType = DirectoryEntryType(UsedDirectory)
			EntryName = DirectoryEntryName(UsedDirectory)
			
			If EntryName = "." Or EntryName = "..": Continue: EndIf
			
			If EntryType = #PB_DirectoryEntry_File
				AddElement(FileToCopy())
				FileToCopy() = EntryPath + EntryName
			EndIf
			
			If EntryType = #PB_DirectoryEntry_Directory
				ListFiles(EntryPath + EntryName)
			EndIf
			
			
		Wend
		
		FinishDirectory(UsedDirectory)
		
	EndIf
	
EndProcedure

Procedure StartsWith(String.s, StartString.s)
	If Left(String, StringByteLength(StartString)) = StartString
		ProcedureReturn #True
	Else
		ProcedureReturn #False
	EndIf
EndProcedure

Procedure EndsWith(String.s, EndString.s)
	If Right(String, StringByteLength(EndString)) = EndString
		ProcedureReturn #True
	Else
		ProcedureReturn #False
	EndIf
EndProcedure

; Procedure.s LTrimWhiteSpace(String.s)
; 	
; 	While Left(String, 1) = " " Or Left(String, 1) = #TAB$
; 		String = LTrim(String, " ")
; 		String = LTrim(String, #TAB$)
; 	Wend
; 	
; 	ProcedureReturn String
; 	
; EndProcedure
; 
; Procedure.s RemoveWhiteSpace(String.s)
; 	String=RemoveString(String, Chr(32))
; 	String=RemoveString(String, #TAB$)
; 	ProcedureReturn String
; EndProcedure

#WHITESPACE$ = " " + #TAB$ + #CRLF$

Procedure.s LTrimChars (source$, charlist$ = #WHITESPACE$)
	; http://www.purebasic.fr/english/viewtopic.php?p=411500#p411500
	; removes from source$ all leading characters which are contained in charlist$
	Protected p, last=Len(source$)
	
	p = 1
	While p <= last And FindString(charlist$, Mid(source$,p,1)) <> 0
		p + 1
	Wend
	
	ProcedureReturn Mid(source$, p)
EndProcedure

Procedure.s RTrimChars (source$, charlist$ = #WHITESPACE$)
	; http://www.purebasic.fr/english/viewtopic.php?p=411500#p411500
	; removes from source$ all trailing characters which are contained in charlist$
	Protected p
	
	p = Len(source$)
	While p >= 1 And FindString(charlist$, Mid(source$,p,1)) <> 0
		p - 1
	Wend
	
	ProcedureReturn Left(source$, p)
	
EndProcedure

Macro TrimChars (_source_, _charlist_=#WHITESPACE$)
	; http://www.purebasic.fr/english/viewtopic.php?p=411500#p411500
	LTrimChars(RTrimChars(_source_, _charlist_), _charlist_)
EndMacro

Procedure.s GetAppDataPath(Firma.s, Programm.s)
	
	; by rolaf (02.04.2010)
	; http://www.purebasic.fr/german/viewtopic.php?p=269821#p269821
	
	Protected DataPath.s
	Protected FolderID
	
	CompilerIf #PB_Compiler_OS = #PB_OS_Windows
		
		DataPath = Space(#MAX_PATH)
		SHGetSpecialFolderLocation_(0, #CSIDL_APPDATA, @FolderID)
		SHGetPathFromIDList_(FolderID, @DataPath)
		
		If Right(DataPath, 1) <> "/"
			DataPath + "/"
		EndIf
		
		DataPath + Firma + "/"
		If FileSize(DataPath) <> - 2
			CreateDirectory(DataPath)
		EndIf
		
		DataPath + Programm + "/"
		If FileSize(DataPath) <> - 2
			CreateDirectory(DataPath)
		EndIf
		
		ProcedureReturn DataPath
		
	CompilerElseIf PB_Compiler_OS = #PB_OS_Linux
		
		DataPath = GetHomeDirectory() + ".PB2Web/
		
		If FileSize(DataPath) <> - 2
			CreateDirectory(DataPath)
		EndIf
		
		ProcedureReturn DataPath
		
	CompilerElseIf PB_Compiler_OS = #PB_OS_MacOS
		
		DataPath = GetHomeDirectory() + "Library/Application Support/PB2Web/"
		
		If FileSize(DataPath) <> - 2
			CreateDirectory(DataPath)
		EndIf
		
		ProcedureReturn DataPath
		
	CompilerEndIf
	
EndProcedure

Procedure CreateDirectoryEx(DirPath.s)
	
	;http://forums.purebasic.com/german/viewtopic.php?f=8&t=27633
	
	CompilerSelect #PB_Compiler_OS
			
		CompilerCase #PB_OS_Windows
			RunProgram(GetEnvironmentVariable("COMSPEC"), "/c mkdir " + #DQUOTE$ + DirPath + #DQUOTE$, "", #PB_Program_Wait | #PB_Program_Hide)
			If FileSize(DirPath) = -2
				ProcedureReturn #True
			EndIf
			
		CompilerDefault
			If Not system("mkdir -p " + #DQUOTE$ + DirPath + #DQUOTE$)
				ProcedureReturn #True
			EndIf
			
	CompilerEndSelect
	
EndProcedure

Procedure Split(Array StringArray.s(1), StringToSplit.s, Separator.s = " ")
	; http://www.purebasic.fr/english/viewtopic.php?p=409005#p409005	
	Protected c = CountString(StringToSplit, Separator)
  Protected i, l = StringByteLength(Separator.s)
  Protected *p1.Character = @StringToSplit
  Protected *p2.Character = @Separator
  Protected *p = *p1
  ReDim StringArray(c)
  While i < c
    While *p1\c <> *p2\c
      *p1 + SizeOf(Character)
    Wend
    If CompareMemory(*p1, *p2, l)
      CompilerIf #PB_Compiler_Unicode
        StringArray(i) = PeekS(*p, (*p1 - *p) >> 1)
      CompilerElse
        StringArray(i) = PeekS(*p, *p1 - *p)
      CompilerEndIf
      *p1 + l
      *p = *p1
    EndIf
    i + 1
  Wend
  StringArray(c) = PeekS(*p)
  ProcedureReturn c
EndProcedure

Procedure.s Join(Array StringArray.s(1), Separator.s = "")
	; http://www.purebasic.fr/english/viewtopic.php?p=409005#p409005	
  Protected r.s, i, l, c = ArraySize(StringArray())
  While i <= c
    l + Len(StringArray(i))
    i + 1  
  Wend
  r = Space(l + Len(Separator) * c)
  i = 1
  l = @r
  CopyMemoryString(@StringArray(0), @l)
  While i <= c
    CopyMemoryString(@Separator)
    CopyMemoryString(@StringArray(i))
    i + 1  
  Wend
  ProcedureReturn r
EndProcedure

Procedure.i IsNumeric(value.s)
	;http://www.purebasic.fr/german/viewtopic.php?p=328021#p328021
	Static regex.i = 0
	If regex = 0
		regex = CreateRegularExpression(#PB_Any, "^\s*(\+|-)?(\$[[:xdigit:]]+|%[01]+|(\d+|\d*\.\d+)([eE](\+|-)?\d+)?)\s*$", #PB_RegularExpression_MultiLine)
	EndIf
	ProcedureReturn MatchRegularExpression(regex, value)
EndProcedure

Global SourceFolder.s = GetExePath() + "system/"

; IDE Options = PureBasic 5.41 LTS Beta 1 (Windows - x86)
; CursorPosition = 155
; FirstLine = 32
; Folding = AAw
; EnableXP