EnableExplicit

Structure sProfile
	
  Name.s
  ApplicationName.s
  IndexPage.s
  
  ServerPath.s
  ServerAddress.s
  
  AspFilename.s
  AspAddress.s
  
  AspxFilename.s
  AspxAddress.s
  
  CgiFilename.s
  CgiAddress.s
  
  PhpFilename.s
  PhpAddress.s
  
  PythonFilename.s
  PythonAddress.s
  
  NodeJsFilename.s
  NodeJsAddress.s
  
  Theme.s
  Encoding.s
  
  FtpHost.s
  FtpDirectory.s
  FtpUser.s
  FtpPass.s
  
  AdditionalScriptsAndStylesheets.s
  
  AfterConvertingDoNothing.i
  AfterConvertingStartDefaultBrowser.i
  AfterConvertingStartSpecificBrowser.i
  AfterConvertingSpecificBrowser.s
  
EndStructure

Global NewList Profile.sProfile()

Procedure LoadProfiles(IniPath.s)
  
  ClearList(Profile())
  
  If OpenPreferences(IniPath)
    
    ExaminePreferenceGroups()
    
    While NextPreferenceGroup()
      
      If Left(PreferenceGroupName(), Len("profile:")) = "profile:"
        
        AddElement(Profile())
        Profile()\Name = ReplaceString(PreferenceGroupName(), "profile:", "")
        
        ExaminePreferenceKeys()
        
        While NextPreferenceKey()
        	
          Select PreferenceKeyName()
              
          	Case "indexpage"
          		Profile()\IndexPage = Trim(PreferenceKeyValue())
          		If Profile()\IndexPage = ""
          			Profile()\IndexPage = "index.html"
          		EndIf
          		
            Case "serverpath"      : Profile()\ServerPath      = Trim(PreferenceKeyValue())
            Case "serveraddress"   : Profile()\ServerAddress   = Trim(PreferenceKeyValue())
            	
            Case "aspfilename"  	 : Profile()\AspFilename  	 = Trim(PreferenceKeyValue())
            Case "aspaddress"   	 : Profile()\AspAddress	 		 = Trim(PreferenceKeyValue())
            	
            Case "aspxfilename"  	 : Profile()\AspxFilename  	 = Trim(PreferenceKeyValue())
            Case "aspxaddress"   	 : Profile()\AspxAddress     = Trim(PreferenceKeyValue())
            	
            Case "cgifilename"     : Profile()\CgiFilename     = Trim(PreferenceKeyValue())
            Case "cgiaddress"      : Profile()\CgiAddress      = Trim(PreferenceKeyValue())
            	
            Case "nodejsfilename"  : Profile()\NodeJsFilename  = Trim(PreferenceKeyValue())
            Case "nodejsaddress"   : Profile()\NodeJsAddress	 = Trim(PreferenceKeyValue())
            	
            Case "phpfilename"  	 : Profile()\PhpFilename  	 = Trim(PreferenceKeyValue())
            Case "phpaddress"   	 : Profile()\PhpAddress	 		 = Trim(PreferenceKeyValue())
            	
            Case "pythonfilename"	 : Profile()\PythonFilename  = Trim(PreferenceKeyValue())
            Case "pythonaddress" 	 : Profile()\PythonAddress	 = Trim(PreferenceKeyValue())
            	
            Case "applicationname" : Profile()\ApplicationName = Trim(PreferenceKeyValue())
            Case "theme"           : Profile()\Theme           = Trim(PreferenceKeyValue())
            Case "encoding"        : Profile()\Encoding        = Trim(PreferenceKeyValue())
            	
            Case "additionalscriptsandstylesheets"
            	
            	Profile()\AdditionalScriptsAndStylesheets = Trim(PreferenceKeyValue())
            	Profile()\AdditionalScriptsAndStylesheets = ReplaceString(Profile()\AdditionalScriptsAndStylesheets, "[#CRLF$]", #CRLF$)
            	
            Case "AfterConvertingDoNothing"            : Profile()\AfterConvertingDoNothing            = Val(PreferenceKeyValue())
            Case "AfterConvertingStartDefaultBrowser"  : Profile()\AfterConvertingStartDefaultBrowser  = Val(PreferenceKeyValue())
            Case "AfterConvertingStartSpecificBrowser" : Profile()\AfterConvertingStartSpecificBrowser = Val(PreferenceKeyValue())
            Case "AfterConvertingSpecificBrowser"      : Profile()\AfterConvertingSpecificBrowser      =     PreferenceKeyValue()
            	
            Case "ftphost"         : Profile()\FtpHost         = PreferenceKeyValue()
            Case "ftpdirectory"    : Profile()\FtpDirectory    = PreferenceKeyValue()
            Case "ftpuser"         : Profile()\FtpUser         = PreferenceKeyValue()
            Case "ftppass"         : Profile()\FtpPass         = PreferenceKeyValue()
              
          EndSelect
          
        Wend
        
        If Profile()\Theme = ""
        	Profile()\Theme = "smoothness"
        EndIf
        
        If Profile()\AfterConvertingDoNothing = 0 And
           Profile()\AfterConvertingStartDefaultBrowser = 0 And
           Profile()\AfterConvertingStartSpecificBrowser = 0
        	
					Profile()\AfterConvertingStartDefaultBrowser = 1        	
        	
        EndIf

        
      EndIf
      
    Wend
    
    ClosePreferences()
    
  EndIf  
  
EndProcedure

Procedure SaveProfiles(IniPath.s)
  
  If CreatePreferences(IniPath, #PB_Preference_GroupSeparator)
    
    ForEach Profile()
      
      PreferenceGroup("profile:" + Profile()\Name)
      
      WritePreferenceString("applicationname", Profile()\ApplicationName)
      WritePreferenceString("indexpage",       Profile()\IndexPage)
      WritePreferenceString("serverpath",      Profile()\ServerPath)
      WritePreferenceString("serveraddress",   Profile()\ServerAddress)
      
      WritePreferenceString("aspfilename",     Profile()\AspFilename)
      WritePreferenceString("aspaddress",      Profile()\AspAddress)
      
      WritePreferenceString("aspxfilename",    Profile()\AspxFilename)
      WritePreferenceString("aspxaddress",     Profile()\AspxAddress)
      
      WritePreferenceString("cgifilename",     Profile()\CgiFilename)
      WritePreferenceString("cgiaddress",      Profile()\CgiAddress)
      
      WritePreferenceString("nodejsfilename",  Profile()\NodeJsFilename)
      WritePreferenceString("nodejsaddress",   Profile()\NodeJsAddress)
      
      WritePreferenceString("phpfilename",     Profile()\PhpFilename)
      WritePreferenceString("phpaddress",      Profile()\PhpAddress)
      
      WritePreferenceString("pythonfilename",  Profile()\PythonFilename)
      WritePreferenceString("pythonaddress",   Profile()\PythonAddress)
      
      WritePreferenceString("encoding",        Profile()\Encoding)
      WritePreferenceString("theme",           Profile()\Theme)
      
      WritePreferenceString("additionalscriptsandstylesheets", ReplaceString(Profile()\AdditionalScriptsAndStylesheets, #CRLF$, "[#CRLF$]"))
      
      WritePreferenceInteger("AfterConvertingDoNothing",            Profile()\AfterConvertingDoNothing)
      WritePreferenceInteger("AfterConvertingStartDefaultBrowser",  Profile()\AfterConvertingStartDefaultBrowser)
      WritePreferenceInteger("AfterConvertingStartSpecificBrowser", Profile()\AfterConvertingStartSpecificBrowser)
      WritePreferenceString ("AfterConvertingSpecificBrowser",      Profile()\AfterConvertingSpecificBrowser)
      
      WritePreferenceString("ftphost",         Profile()\FtpHost)
      WritePreferenceString("ftpdirectory",    Profile()\FtpDirectory)
      WritePreferenceString("ftpuser",         Profile()\FtpUser)
      WritePreferenceString("ftppass",         Profile()\FtpPass)
      
    Next
    
    ClosePreferences()
    
  Else
    
    MessageRequester("Ups!", "Couldn't save the Profile!")
    
  EndIf  
  
EndProcedure
; IDE Options = PureBasic 5.41 LTS Beta 2 (Windows - x86)
; CursorPosition = 183
; FirstLine = 8
; Folding = 9
; EnableXP