<?php

/**
 * cDatabase short summary.
 *
 * cDatabase description.
 *
 * @version 1.0
 * @author Peter Tbben
 * 
 */

require_once("cCommon.php");
require_once("cConstants.php");

class DbObject {
  public $Handler;
  public $Statement;
	public $CurrentRow;
}

class cDatabase {

	static $Databases = [];
	static $LastDatabaseError = "";
	
	static function AffectedDatabaseRows() {}
	
	static function CheckDatabaseNull() {}
	
	// CloseDatabase(#Database)
	static function CloseDatabase($Database) {
		
		$DBO = cDatabase::$Databases[$Database];
		
		$DBO->Handler = null;
		$DBO->Statement = null;
		
		unset(cDatabase::$Databases[$Database]);
		
	}
	
	static function DatabaseColumnIndex() {}
	
	// Text$ = DatabaseColumnName(#Database, Column) 
	static function DatabaseColumnName($Database, $Column) {
		
		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->Statement) {
				return $DBO->Statement->getColumnMeta($Column)['name'];
			}
		}
		
		return "";			
		
	}
	
	static function DatabaseColumnSize($Database, $Column) {
		
	}
	
	// Result = DatabaseColumnType(#Database, Column) 
	static function DatabaseColumnType($Database, $Column) {
	
		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->Statement) {
				
				$pdoType = $DBO->Statement->getColumnMeta($Column)['pdo_type'];
				
				$nativeType = $DBO->Statement->getColumnMeta($Column)['native_type'];
				
				
				switch ( $nativeType ) {
					
					case 1:
						
						break;
						
				}
				
				
				/*
				
				#PB_Database_Long  : Numeric format (a Long (.l) in PureBasic)
				#PB_Database_String: String format (a String (.s) in PureBasic)
				#PB_Database_Float : Numeric float format (a Float (.f) in PureBasic)
				#PB_Database_Double: Numeric double format (a Double (.d) in PureBasic)
				#PB_Database_Quad  : Numeric quad format (a Quad (.q) in PureBasic)
				#PB_Database_Blob  : Blob format				 
				
				*/
				
				
			}
		}
		
		return "";		
	
	}
	
	static function DatabaseColumns() {}
	
	static function DatabaseDriverDescription() {}
	
	static function DatabaseDriverName() {}
	
	// Error$ = DatabaseError()
	static function DatabaseError() {
		return cDatabase::$LastDatabaseError;
	}
	
	static function DatabaseID() {}
	
	// Result = DatabaseQuery(#Database, Request$ [, Flags])
	static function DatabaseQuery($Database, $Request, $Flags = cConstants::PB_Database_StaticCursor ) {
		
		// The flags to use. It can be one of the following value: 
		//   #PB_Database_StaticCursor : performs the query to access the result in a sequential manner. It's not possible to rewind 
		//                               with PreviousDatabaseRow() or FirstDatabaseRow() on some drivers, but it is the faster way to get the data (default).
		//   #PB_Database_DynamicCursor: performs the query to access the result in a random manner using PreviousDatabaseRow() or FirstDatabaseRow(). 
		//                               It can be slower, or even unsupported on some drivers.
		// 
		// 		
		
		cDatabase::$LastDatabaseError = "";
		
		$DBO = cDatabase::$Databases[$Database];
		
		if ($Flags === cConstants::PB_Database_StaticCursor) {
			
			$DBO->Statement = $DBO->Handler->prepare($Request, array( PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
			
		} else { // $Flags === cConstants::PB_Database_DynamicCursor
			
			$DBO->Statement = $DBO->Handler->prepare($Request, array(PDO::ATTR_CURSOR => PDO::CURSOR_SCROLL)); // doesn't work with SQLite?
			
		}
		
		if (!$DBO->Statement) {
			cDatabase::$LastDatabaseError = $DBO->Handler->errorInfo()[2];
			return false;
		} else {
			try {
				$DBO->Statement->execute();
			}
			catch(PDOException $e) {
				cDatabase::$LastDatabaseError = $e->getMessage();
				return false;
			}
		}
		
		return true;
		
	}
	
	// Result = DatabaseUpdate(#Database, Request$)
	static function DatabaseUpdate($Database, $Request) {
		
		cDatabase::$LastDatabaseError = "";
		
		$DBO = cDatabase::$Databases[$Database];
		
		$DBO->Statement = $DBO->Handler->prepare($Request);
		
		if (!$DBO->Statement) {
			cDatabase::$LastDatabaseError = $DBO->Handler->errorInfo()[2];
			return false;
		} else {
			try {
				$DBO->Statement->execute();
			}
			catch(PDOException $e) {
				cDatabase::$LastDatabaseError = $e->getMessage();
				return false;
			}
		}
		
		return true;
		
	}
	
	static function ExamineDatabaseDrivers() {}
	
	// FinishDatabaseQuery(#Database)
	static function FinishDatabaseQuery($Database) {
	
		$DBO = cDatabase::$Databases[$Database];

		$DBO->Statement = null;
		$DBO->CurrentRow = null;
		
	}
	
	// Result = FirstDatabaseRow(#Database)
	static function FirstDatabaseRow($Database) {

		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->Statement) {
				$DBO->CurrentRow = $DBO->Statement->fetch(PDO::FETCH_BOTH, PDO::FETCH_ORI_FIRST);
				if ($DBO->CurrentRow) {
					return true;
				}
			}
		}
		
		return false;		
	
	}
		
	static function GetDatabaseFieldContent($Database, $Column) {
		
		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->CurrentRow) {
				return $DBO->CurrentRow[$Column];
			}
		}
		
		return "";		
		
	}
	
	static function GetDatabaseBlob() {}
	
	// Result.d = GetDatabaseDouble(#Database, Column)
	static function GetDatabaseDouble($Database, $Column) {
		return cDatabase::GetDatabaseFieldContent($Database, $Column);
	}
	
	// Result.f = GetDatabaseFloat(#Database, Column)
	static function GetDatabaseFloat($Database, $Column) {
		return cDatabase::GetDatabaseFieldContent($Database, $Column);
	}
	
	// Result = GetDatabaseLong(#Database, Column)
	static function GetDatabaseLong($Database, $Column) {
		return cDatabase::GetDatabaseFieldContent($Database, $Column);
	}
	
	// Result.q = GetDatabaseQuad(#Database, Column)
	static function GetDatabaseQuad($Database, $Column) {
		return cDatabase::GetDatabaseFieldContent($Database, $Column);
	}
	
	// Text$ = GetDatabaseString(#Database, Column)
	static function GetDatabaseString($Database, $Column) {
		return cDatabase::GetDatabaseFieldContent($Database, $Column);
	}
	
	static function IsDatabase() {}
	
	static function NextDatabaseDriver() {}
	
	// Result = NextDatabaseRow(#Database)
	static function NextDatabaseRow($Database) {
		
		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->Statement) {
				$DBO->CurrentRow = $DBO->Statement->fetch(PDO::FETCH_BOTH, PDO::FETCH_ORI_NEXT);
				if ($DBO->CurrentRow) {
					return true;
				}
			}
		}
		
		return false;
		
	}
	
	// Result = OpenDatabase(#Database, DatabaseName$, User$, Password$ [, Plugin]) 
	
	static function OpenDatabase($Database, $DatabaseName, $User, $Password, $Plugin) {
		
		if ($Database == cConstants::PB_Any) {
			$Database = cCommon::GUID();
		}
		
		if (cDatabase::$Databases[$Database]) {
			cDatabase::CloseDatabase($Database);
		}
		
		$DBO = new DbObject();
		
		$DBO->Handler = new PDO('sqlite:' . $DatabaseName, $User, $Password);
		
		cDatabase::$Databases[$Database] = $DBO;
		
		if ($Database === 0) {
			return true;
		}
		
		return $Database;		
		
	}
	
	static function OpenDatabaseRequester() {}
	
	// Result = PreviousDatabaseRow(#Database)
	static function PreviousDatabaseRow($Database) {
	
		$DBO = cDatabase::$Databases[$Database];

		if ($DBO) {
			if ($DBO->Statement) {
				$DBO->CurrentRow = $DBO->Statement->fetch(PDO::FETCH_BOTH, PDO::FETCH_ORI_PRIOR);
				if ($DBO->CurrentRow) {
					return true;
				}
			}
		}
		
		return false;	
	
	}
	
	static function SetDatabaseBlob() {}
	
	static function UseODBCDatabase() {}
	
	static function UsePostgreSQLDatabase() {}
	
	static function UseSQLiteDatabase() {}
	
}
