XIncludeFile "inc.lexer.pbi"

EnableExplicit

Structure sParams
	Name.s
	IsMap.i
	IsLinkedList.i
	IsOptional.i
EndStructure

Structure sProcedure
	
	Name.s
	Line.i
	List Params.sParams()
	
EndStructure

Global NewList Procedures.sProcedure()

; Targets

#Target_JS     = 1
#Target_NodeJS = 2
#Target_PHP    = 3
#Target_CGI    = 4
#Target_Python = 5
#Target_ASP    = 6
#Target_ASPX   = 7

Global TargetSource

Declare.s PreProcess_DataSection(Output.s)

Declare.s PreProcess_IncludeBinary(Output.s)

Structure sIsInUse
	BlockUI.i
	ColorRequester.i
	CryptoJs.i
	Database.i
	DateTimePicker.i
	FileSaver.i
	GoogleChart.i
	GoogleMap.i
	JQueryCookie.i
	KeyboardShortcut.i
	Menu.i
	PDF.i
	Phaser.i
	Pixi.i
	Preference.i
	SaveImage.i
	Splitter.i
	TouchSwipe.i
	TreeGadget.i
	TriState.i
	w2ui.i
	XML.i
	TrackBarGadget.i
	Cordova.i
	FontAwesome.i
	Bootstrap.i
	TinyMce.i
	Websocket.i
	Mobile.i
EndStructure

Global IsInUse.sIsInUse

#PBSC_Constant = 10
#PBSC_Operator = 11

Structure sCompilerInfo
	
	IndexPage.s
	
  TargetFolder.s
  TargetFolderUser.s
  ServerAddress.s
  ServerAddressUser.s
  
  TargetFolderJS.s
  TargetFolderJSUser.s
  ServerAddressJS.s
  ServerAddressJSUser.s
  
  AspxFilename.s
  AspxAddress.s
  AspxFilenameUser.s
  AspxAddressUser.s
  AspFilename.s
  AspAddress.s
  AspFilenameUser.s
  AspAddressUser.s
  CgiFilename.s
  CgiAddress.s
  CgiFilenameUser.s
  CgiAddressUser.s
  NodeJsFilename.s
  NodeJsAddress.s
  NodeJsFilenameUser.s
  NodeJsAddressUser.s
  PhpFilename.s
  PhpAddress.s
  PhpFilenameUser.s
  PhpAddressUser.s
  PythonFilename.s
  PythonAddress.s
  PythonFilenameUser.s
  PythonAddressUser.s
  UseFlask.s
  UseFlaskUser.s
  FlaskPort.s
  FlaskPortUser.s
  
  ApplicationName.s
  ApplicationNameUser.s
  
  Theme.s
  Encoding.s
  
  FtpHost.s
  FtpUser.s
  FtpPass.s
  
  AdditionalScriptsAndStylesheets.s
  DoNotCopy.i
  ProfileName.s
  
  AfterConvertingDoNothing.i
  AfterConvertingStartDefaultBrowser.i
  AfterConvertingStartSpecificBrowser.i
  AfterConvertingSpecificBrowser.s  
  
EndStructure

Global PbCompiler.s

Global CompilerInfo.sCompilerInfo

Global SourceFile.s

XIncludeFile "functions.pbi"

Enumeration
	#TokenTypeX_Comma
	#TokenTypeX_Dot
	#TokenTypeX_Parenthesis_Open
	#TokenTypeX_Parenthesis_Close
EndEnumeration
		
Structure Token
  Token.s
  Type.i
  TypeX.i
  Line.i
EndStructure

Global NewList Token.Token()

Structure sOptionalParams
  v1.s
  v2.s
  v3.s
EndStructure

Procedure AddLog(LogMessage.s, IsError = 0)
	
	CompilerIf #PB_Compiler_IsIncludeFile
		
		If IsGadget(#frmMain_Log)
			
			If IsError
				LogMessage = "Error: " + LogMessage
			EndIf
			
			Protected *msg
			*msg = AllocateMemory(StringByteLength(LogMessage)+SizeOf(Character))
			PokeS(*msg, LogMessage)
			PostEvent(#PB_Event_Gadget, #frmMain, #frmMain_Log, #AddLog, *msg)
			
		EndIf
		
	CompilerElse
		
		PrintN(LogMessage)
		; Debug LogMessage
	
	CompilerEndIf
	
	If IsError = 1
		ShitHappend = #True
	EndIf
	
EndProcedure

Procedure PrevToken(List TT.Token(), Match.s)
  Protected ReturnValue
  PreviousElement(TT())
  ReturnValue = Bool(LCase(TT()\Token) = LCase(Match))
  NextElement(TT())
  ProcedureReturn ReturnValue
EndProcedure

Procedure ShowCheckNextToken()
  NextElement(Token())
  ;Debug "Next Token: " + Token()\Token
  PreviousElement(Token())
EndProcedure

Procedure CheckNextToken(List TT.Token(), Match.s)
  Protected ReturnValue
  If NextElement(TT())
	  ReturnValue = Bool(TT()\Token = Match)
	  PreviousElement(TT())
  EndIf
  ProcedureReturn ReturnValue
EndProcedure

Procedure.s NextToken(List TT.Token())
  Protected ReturnValue.s
  If NextElement(TT())
	  ReturnValue = TT()\Token
	  PreviousElement(TT())
  EndIf
  ProcedureReturn ReturnValue
EndProcedure

Procedure RemoveTokenIfEmpty(List TT.Token())
  
  If CheckNextToken(TT(), "")
    NextElement(TT())
    DeleteElement(TT())
  EndIf
  
EndProcedure

Procedure.s GetPrevToken()
  Protected ReturnValue.s
  PreviousElement(Token())
  ReturnValue=Token()\Token
  NextElement(Token())
  ProcedureReturn ReturnValue
EndProcedure

Procedure.s GetTokenTypeAsString(TokenType)
  
  Protected ReturnValue.s
  
  Select TokenType
    Case #PBSC_Comment 
      ReturnValue = "#PBSC_Comment"
    Case #PBSC_Identifier 
      ReturnValue = "#PBSC_Identifier"
    Case #PBSC_NewLine 
      ReturnValue = "#PBSC_NewLine"
    Case #PBSC_Number 
      ReturnValue = "#PBSC_Number"
    Case #PBSC_Other 
      ReturnValue = "#PBSC_Other"
    Case #PBSC_String 
      ReturnValue = "#PBSC_String"
    Default 
      CallDebugger
      ReturnValue = "???"
  EndSelect
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure RemoveTypeDeclarations(List TT.Token())
	
  ; remove Type-Declarations
  Protected inDot
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_NewLine
        inDot = #False
      Case #PBSC_Other
        Select TT()\Token
          Case "." ; , "\"
            inDot = #True
        EndSelect
      Case #PBSC_Identifier 
      	Select LCase(TT()\Token)
      			
        	Case "b", "a", "c", "w", "u", "l", "i", "f", "q", "d", "s", "word"
        		
            If inDot
            	
            	Protected TypenDeklaration.s = LCase(TT()\Token)
            	Protected PostFix.s = ""
            	
            	; TODO PostFix fr Variableninitialisierung
            	
            	PreviousElement(TT())
            	PreviousElement(TT())
            	PreviousElement(TT())
            	Select LCase(TT()\Token)
            		Case "protected", "define"
            			Select TypenDeklaration
            				Case "s"
            					PostFix = " = " + Chr(34) + Chr(34)
            			EndSelect
            	EndSelect
            	NextElement(TT())
            	NextElement(TT())
            	NextElement(TT())
            	
            	;
            	
              PreviousElement(TT()) ; Punkt entfernen
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
              NextElement(TT()) ; Typendeklartion entfernen
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              inDot = #False
              
            EndIf
          Default
            
        EndSelect
    EndSelect
  Next	
  
EndProcedure

Procedure RemoveEmptyToken(List TT.Token())
	Protected *Old_Element = @Token()
	ForEach TT()
		If TT()\Token=""
			DeleteElement(TT())
		EndIf
	Next
	ChangeCurrentElement(TT(), *Old_Element)
EndProcedure

Procedure ProcessVariables()
  
  NewMap Variable.s()
  
  ForEach Token()
  	
  	Select Token()\Type
  			
  		Case #PBSC_Identifier
  			
        Select LCase(Token()\Token)
            
          Case "procedure"
            
            Repeat
              
            	If Token()\Type = #PBSC_Newline : Break : EndIf
            	If Token()\Type = #PBSC_Comment : Break : EndIf
            	
            	If Token()\Type = #PBSC_Identifier
            		Select Token()\Token
            			Case "", "(", ")", "=", ","
            			Default
            				;Debug Token()\Token
            		EndSelect
            	EndIf
            	
            	NextElement(Token())
              
            ForEver
            
          Case "endprocedure"
            
        EndSelect
        
    EndSelect
    
  Next	
  
EndProcedure

Procedure ProcessVariablesPHP(List TT.Token())
  
  NewMap Variable.s()
  
  Protected Inline
  Protected InProcedure
  
  ForEach TT()
  	
  	Select TT()\Type
  			
  		Case #PBSC_NewLine
  			
  			InProcedure = #False
  			
  			If CheckNextToken(TT(), "!")
  				Inline = #True
  			Else
  				Inline = #False
  			EndIf
  			
  		Case #PBSC_Identifier
  			
  			Select LCase(TT()\Token)
					Case "procedure", "proceduredll", "procedurecdll"
  					InProcedure = #True	
  			EndSelect
  			
  			If Not Inline	And Not InProcedure
  				
  				If TT()\Token = "\"
  					TT()\Token = "."
  				EndIf
  				
  				If CheckNextToken(TT(), "=")
  					TT()\Token = "$" + TT()\Token
  				EndIf
  				
  			EndIf
  			
  	EndSelect
  	
  Next	
  
EndProcedure

Procedure ProcessIdentifier2Classnames(List TT.Token())
  
  ; ProcedureReturn 
  
	NewMap Library.s()
	
	Protected FF
	Protected Line.s
	
	FF = ReadFile(#PB_Any, GetExePath() + "libraries.txt")
	
	If FF
		While Not Eof(FF)
			Line = ReadString(FF)
			If FindString(Line, ".")
				Library(StringField(Line, 2, ".")) = StringField(Line, 1, ".")
			EndIf
		Wend
		CloseFile(FF)
	EndIf
	
  ForEach TT()
  	
  	Select TT()\Type
  			
    	Case #PBSC_Identifier
    		
    		If FindMapElement(Library(), TT()\Token) And CheckNextToken(TT(), "(")
    			
      		TT()\Token = "c" + Library(TT()\Token) + "." + TT()\Token
      		
      	ElseIf FindMapElement(Library(), TT()\Token)
      		
      		Protected TempToken.s
      		
      		Select LCase(TT()\Token)
      				
      			; Read, Restore, End
      			
      			Case "read"
      				
      				TempToken = "c" + Library(TT()\Token) + "." + TT()\Token + "()"
      				TT()\Token = ""
      				
							NextElement(TT())
							NextElement(TT())
							NextElement(TT())
							
							TempToken = TT()\Token + " = " + TempToken
      					
      				TT()\Token = TempToken 
      				
      			Case "restore"
      				
      				TempToken = "c" + Library(TT()\Token) + "." + TT()\Token + "("
      				TT()\Token = ""
      				
      				Repeat
      					
      					NextElement(TT())
      					
      					If TT()\Type = #PBSC_Newline : Break : EndIf
      					If TT()\Type = #PBSC_Comment : Break : EndIf
                  
      					If TT()\Token  <> ""
      						TempToken + "P2W_Label_" + TT()\Token 
      						TT()\Token = ""
      					EndIf
      					
      				ForEver
      				
      				PreviousElement(TT())
      				TT()\Token = TempToken + ")"
      				
      			Case "end"
      				
      				TT()\Token = "c" + Library(TT()\Token) + "." + TT()\Token + "()"
      				
      		EndSelect
      		
        EndIf
    EndSelect
  Next
  
  FreeMap(Library())
  
  ; Konstanten [!!!]
  
  NewMap Constant.s()
  
	FF = ReadFile(#PB_Any, GetExePath() + "constants.txt")
	
	If FF
		While Not Eof(FF)
			Line = ReadString(FF)
			Constant("#" + UCase(Line)) = UCase(Line)
		Wend
		CloseFile(FF)
	EndIf  
	
	ForEach TT()
		Select TT()\Type
			Case #PBSC_Constant
				If FindMapElement(Constant(), UCase(TT()\Token))
					TT()\Token = "cConstants." + Mid(TT()\Token, 2)
				EndIf
		EndSelect
	Next
  
  FreeMap(Constant())
  
  ; #####################
  
  ForEach TT()
  	
		Select TT()\Type
				
	  	Case #PBSC_Identifier
	  		
	    	Select LCase(TT()\Token)
	    			
	    		Case "enableexplicit", "disableexplicit"
	    			TT()\Token = ""
	    			
	    		Case "enablegadgetsautobind"
	    			
	    			TT()\Token = "$.fn.GadgetsEvents="
	    			
	    			Repeat
	    				NextElement(TT())
	    				If TT()\Type = #PBSC_Newline : Break : EndIf
	    				If TT()\Type = #PBSC_Comment : Break : EndIf	    				
	    				If TT()\Token = ")" : TT()\Token="" : EndIf
	    				If TT()\Token = "(" : TT()\Token="" : EndIf
	    			ForEver
	    			
	    		Case "disablegadgetsautobind"
	    			
	    			TT()\Token = "$.fn.GadgetsEvents=null"
	    			
	    			Repeat
	    				NextElement(TT())
	    				If TT()\Type = #PBSC_Newline : Break : EndIf
	    				If TT()\Type = #PBSC_Comment : Break : EndIf	    				
	    				If TT()\Token = ")" : TT()\Token="" : EndIf
	    				If TT()\Token = "(" : TT()\Token="" : EndIf
	    			ForEver
	    			
	    	EndSelect
	    	
	  EndSelect
	  
	Next
  
EndProcedure

Procedure ProcessIdentifier2ClassnamesPHP(List TT.Token())
  
	NewMap Library.s()
	
	Protected FF
	Protected Line.s
	
	FF = ReadFile(#PB_Any, GetExePath() + "libraries.txt")
	
	If FF
		While Not Eof(FF)
			Line = ReadString(FF)
			If FindString(Line, ".")
				Library(StringField(Line, 2, ".")) = StringField(Line, 1, ".")
			EndIf
		Wend
		CloseFile(FF)
	EndIf
	
  ForEach TT()
  	
  	Select TT()\Type
  			
    	Case #PBSC_Identifier
    		
    		If FindMapElement(Library(), TT()\Token) And CheckNextToken(TT(), "(")
    			
      		TT()\Token = "c" + Library(TT()\Token) + "::" + TT()\Token
      		
      	ElseIf FindMapElement(Library(), TT()\Token)
      		
      		Protected TempToken.s
      		
      		Select LCase(TT()\Token)
      				
      			; Read, Restore, End
      			
      			Case "read"
      			Case "restore"
      			Case "end"
      				
      		EndSelect
      		
        EndIf
    EndSelect
  Next
  
  FreeMap(Library())
  
  ; Konstanten [!!!]
  
  NewMap Constant.s()
  
	FF = ReadFile(#PB_Any, GetExePath() + "constants.txt")
	
	If FF
		While Not Eof(FF)
			Line = ReadString(FF)
			Constant("#" + UCase(Line)) = UCase(Line)
		Wend
		CloseFile(FF)
	EndIf  
	
	ForEach TT()
		Select TT()\Type
			Case #PBSC_Constant
				If FindMapElement(Constant(), UCase(TT()\Token))
					TT()\Token = "cConstants::" + Mid(TT()\Token, 2)
				EndIf
		EndSelect
	Next
  
  FreeMap(Constant())
  
  ; #####################
  
  ForEach TT()
  	
		Select TT()\Type
				
	  	Case #PBSC_Identifier
	  		
	    	Select LCase(TT()\Token)
	    			
	    		Case "enableexplicit", "disableexplicit"
	    			TT()\Token = ""
	    			
	    	EndSelect
	    	
	  EndSelect
	  
	Next
  
EndProcedure

Procedure ProcessIf(List TT.Token())
  
  AddLog("Processing: If - Else - EndIf...")
  
  ; ----
  ; If - Else - EndIf
  ; ----
  
  Protected inIf
  Protected NextToken.s
  
  ForEach TT()
    
    Select TT()\Type
        
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
          Case "if"
            inIf = #True
            TT()\Token = "if ("
          Case "elseif"
            inIf = #True
            TT()\Token = "} else if ("
          Case "else"
            ; inIf = #True
            TT()\Token = "} else {"
          Case "endif"
            inIf = #False 
            TT()\Token = "}"
            
        EndSelect
        
      Case #PBSC_Other
        
        If inIf
          
          Select TT()\Token
              
            Case "="
            	
            	If CheckNextToken(TT(), ">")
            		
            		TT()\Token = ">="
            		NextElement(TT())
            		TT()\Token = ""
            		
            	ElseIf CheckNextToken(TT(), "<")	
            		
            		TT()\Token = "<="
            		NextElement(TT())
            		TT()\Token = ""
            		
            	Else
            		
            		TT()\Token = "=="
            		
            	EndIf
              
            Case "<>"
            	
              TT()\Token = "!="
              
            Default
            	
            	; Debug TT()\Token
            	
          EndSelect
          
        EndIf
        
      Case #PBSC_NewLine, #PBSC_Comment
      	
      	; if(!(DoNotUpdatePosition) // ; avoid stutter) {
      	; if(!(DoNotUpdatePosition)) { // ; avoid stutter 
      	
        If inIf
          
          PreviousElement(TT())
          TT()\Token + ") {"
          NextElement(TT())
          
          inIf = #False
          
        EndIf
        
    EndSelect
    
  Next
  
EndProcedure

Procedure ProcessStructures_TS()
  
  AddLog("Processing: Structures...")
  
  NewMap StructureMap.s()
  
  ; preprocessing
  
  ; get all structure-names
  
  ForEach Token()
  	If LCase(Token()\Token) = "structure"
  		NextElement(Token())
  		StructureMap(Token()\Token) = Token()\Token
  	EndIf
  Next
  
  ; rename and lowercase all structures
  
  ; "Structure Person" -> "Structure p2w_structure_person"
  
  ForEach Token()
  	If Token()\Token = "."
  		NextElement(Token())
  		If FindMapElement(StructureMap(), Token()\Token)
  			Token()\Token = "p2w_structure_" + LCase(Token()\Token)
  		EndIf
  	ElseIf LCase(Token()\Token) = "structure"
  		NextElement(Token())
  		Token()\Token = "p2w_structure_" + LCase(Token()\Token)
  	ElseIf LCase(Token()\Token) = "extends"
  		NextElement(Token())
  		Token()\Token = "p2w_structure_" + LCase(Token()\Token)
  	EndIf
  Next
  
  ClearMap(StructureMap())  
  
  ; --------------   
  
  Protected inStructure
  
  Protected ReturnValue = #True
  
  ; Structure Person
  ;   FirstName.s
  ;   Name.s
  ; EndStructure  
  
  ; ->
  
  ; class Person {
  ;   FirstName:string;
  ;   Name.string;
  ; }
  
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case "structure"
            
            inStructure = #True
            
            Token()\Token = "class"
            
            NextElement(Token())
            
            NextElement(Token())
            
            If LCase(Token()\Token)="extends"
            	
            	Token()\Token = "extends"
            	
            	NextElement(Token())
            	NextElement(Token())
            	
            EndIf
            
            PreviousElement(Token())
            
            Token()\Token + " {"
            
          Case "endstructure"  
            
            inStructure = #False
            
            Token()\Token = "}"
            
          Default
            
          	If inStructure
          		
          		Token()\Token + ":any"
              
            EndIf
            
        EndSelect
        
    EndSelect
    
  Next
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessStructures(List TT.Token())
  
  AddLog("Processing: Structures...")
  
  ; --------------   
  
  Protected StructureLine.s
    
  Protected ReturnValue = #True
  
  ; Structure Person
  ;   FirstName.s
  ;   Name.s
  ; EndStructure  
  
  ; ->
  
  ; old:  
  ; var Person = (function () {
  ;     function Person() {
  ;         this.FirstName = "";
  ;         this.Name = "";
  ;     }
  ;     return Person;
  ; })();   
  
  ; new
  ; var Person = {
  ;   FirstName : null,
  ;   Name : null
  ; };   
  
  Protected CurrentStructureType.s
  Protected FirstStructure = -1
  
  ForEach TT()
  	If LCase(TT()\Token) = "structureunion"
  		AddLog("")
  		AddLog("Line " + Str(TT()\Line) + ": StructureUnion is not supported (yet), sorry!")
  		AddLog("", #True)
  		ProcedureReturn #False
  	EndIf
  Next
  
  ForEach TT()
  	
    Select TT()\Type
    		
      Case #PBSC_Identifier
        
        Select LCase(TT()\Token)
        		
          Case "structure"
          	
          	If FirstStructure = -1
          		FirstStructure = ListIndex(TT())
          	EndIf
          	
            TT()\Token = ""
            
            StructureLine + "var "
            
            NextElement(TT()) ; StructureName
            
            StructureLine + TT()\Token + " = { "
            
            TT()\Token = ""
            
            Repeat
            	
            	Select TT()\Type
            			
            		Case #PBSC_NewLine
            			
            		Case #PBSC_Comment
            			
            		Default
            			
            			Select LCase(TT()\Token)
            					
            				Case "endstructure"
            					StructureLine + "#};"
            					TT()\Token = ""
            					Break
            					
            				Case "map"
            					
            					; Structure MyStructure2
											;   Map myMap.MyStructure1()
											; EndStructure              					
            					
            					TT()\Token = "" ; Map
            					
            					NextElement(TT()) ; myMap
            					
            					StructureLine + TT()\Token
            					
            					TT()\Token = "" 
            					
            					NextElement(TT()) ; "." ?
            					
            					If Token()\Token="."
            						
            						StructureLine + ":"
            						
            						TT()\Token = ""	
            						
            						NextElement(TT())
            						StructureLine + "new cMap(" + TT()\Token + "),"
            						Token()\Token = ""
            						
            						NextElement(TT()) ; "("
            						Token()\Token = ""
            						
            						NextElement(TT()) ; ")"
            						Token()\Token = ""
            						
            					Else
            						
            						StructureLine + ":"
            						NextElement(TT())
            						Token()\Token = ""
            						
            						StructureLine + "new cMap(),"
            						NextElement(TT())
            						Token()\Token = ""
            						
            						NextElement(TT())
            						Token()\Token = ""            						
            						
            					EndIf
            					
            				Case "list"
            					
            					; Structure MyStructure2
											;   List myList.MyStructure1()
											; EndStructure              					
            					
            					TT()\Token = "" ; List
            					
            					NextElement(TT()) ; myList
            					
            					StructureLine + TT()\Token
            					
            					TT()\Token = "" 
            					
            					NextElement(TT()) ; "." ?
            					
            					If Token()\Token="." ; another structure
            						
            						StructureLine + ":"
            						
            						TT()\Token = ""	
            						
            						NextElement(TT())
            						
            						StructureLine + "new cLinkedList(),"
            						Token()\Token = ""
            						
            						NextElement(TT()) ; "("
            						Token()\Token = ""
            						
            						NextElement(TT()) ; ")"
            						Token()\Token = ""
            						
            					Else ; standard type (s, b, i, l, ...)
            						
            						StructureLine + ":"
            						NextElement(TT())
            						Token()\Token = ""
            						
            						StructureLine + "new cLinkedList(),"
            						NextElement(TT())
            						Token()\Token = ""
            						
            						NextElement(TT())
            						Token()\Token = ""
            						
            					EndIf
            					
            				Case "."
            					
            					PreviousElement(TT())
            					StructureLine + TT()\Token + ":"
            					TT()\Token = ""
            					
            					NextElement(TT())
            					TT()\Token = ""
            					
            					NextElement(TT())
            					StructureLine + "jQuery.extend(true, {}, " + TT()\Token + "),"
            					TT()\Token = ""
            					
            				Default
            					
            					If TT()\Token = "{"
            						Repeat
            							If TT()\Token = "}" : TT()\Token = "" : Break : EndIf
	            						TT()\Token = ""
		            					NextElement(TT())
	           						ForEver
            					EndIf
            						
            					If TT()\Token = "["
            						Repeat
            							If TT()\Token = "]" : TT()\Token = "" : Break : EndIf
	            						TT()\Token = ""
		            					NextElement(TT())
	           						ForEver
	           					EndIf
	           					
	           					If TT()\Token <> ""
            						If Not CheckNextToken(TT(), ".")
            							StructureLine + TT()\Token + " : null,"
            							TT()\Token = ""
            						EndIf
            					EndIf
            					
            			EndSelect
            			
            	EndSelect
            	
         			NextElement(TT())

            ForEver
            
            StructureLine = ReplaceString(StructureLine, "#}", "}")
            StructureLine = ReplaceString(StructureLine, ",}", "}")
            StructureLine + #LF$
            
          Default
          	
        EndSelect
        
    EndSelect
    
  Next
  
  If FirstStructure <> -1
	  SelectElement(TT(), FirstStructure)
	  TT()\Token = StructureLine
  EndIf
  
  RemoveEmptyToken(TT())
    
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessStructures_old()
  
  AddLog("Processing: Structures...")
  
  NewMap StructureMap.s()
  
  ; preprocessing
  
  ; get all structure-names
  
  ForEach Token()
  	If LCase(Token()\Token) = "structure"
  		NextElement(Token())
  		StructureMap(Token()\Token) = Token()\Token
  	EndIf
  Next
  
  ; rename and lowercase all structures
  
  ; "Structure Person" -> "Structure p2w_structure_person"
  
  ForEach Token()
  	If Token()\Token = "." Or Token()\Token = ","
  		NextElement(Token())
  		If FindMapElement(StructureMap(), Token()\Token)
  			Token()\Token = "p2w_structure_" + LCase(Token()\Token)
  		EndIf
  	ElseIf LCase(Token()\Token) = "structure"
  		NextElement(Token())
  		Token()\Token = "p2w_structure_" + LCase(Token()\Token)
  	EndIf
  Next
  
  ClearMap(StructureMap())  
  
  ; --------------   
  
  Protected inStructure
  
  Protected StructureLine.s
  Protected CurrentStructure.s
  
  Protected StructureItemName.s = ""
  Protected StructureItemType.s = ""
  
  Protected ReturnValue = #True
  
  ; Structure Person
  ;   FirstName.s
  ;   Name.s
  ; EndStructure  
  
  ; ->
  
  ; var Person = (function () {
  ;     function Person() {
  ;         this.FirstName = "";
  ;         this.Name = "";
  ;     }
  ;     return Person;
  ; })();   
  
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case "structure"
            
            inStructure = #True
            
            Token()\Token = ""
            Token()\Type  = #PBSC_Other
            
            NextElement(Token())
            
            StructureMap(Token()\Token) = Token()\Token
            
            CurrentStructure = Token()\Token
            
            StructureLine = "var " + Token()\Token + " = (function () { " + #LF$ + "function " + Token()\Token + "() { " + #LF$
            
            Token()\Token = ""
            Token()\Type  = #PBSC_Other
            
          Case "endstructure"  
            
            inStructure = #False
            
            StructureLine + "} return " + CurrentStructure + ";})();"
            
            Token()\Token = StructureLine
            Token()\Type = #PBSC_Other
            
            StructureLine = ""
            
          Default
            
            If inStructure
              
              If Not FindMapElement(StructureMap(), Token()\Token)
                
                StructureItemName = Token()\Token
                
                If LCase(StructureItemName) = "list"
                  
                  AddLog("")
                  AddLog("Line " + Str(Token()\Line) + ": LinkedLists in Structures are not supported (yet), sorry!")
                  AddLog("", #True)
                  
                  ReturnValue = #False
                  
                  Break 
                  
                Else
                  
                  Token()\Token = ""
                  Token()\Type  = #PBSC_Other
                  
                  NextElement(Token())
                  Token()\Token = ""
                  Token()\Type  = #PBSC_Other
                  
                  NextElement(Token())
                  
                EndIf
                
              EndIf
              
              If StructureItemName <> ""
                
                If FindMapElement(StructureMap(), Token()\Token)
                  StructureItemType = " = new " + Token()\Token
                  Token()\Token = ""
                  Token()\Type  = #PBSC_Other
                Else
                	
                	If CheckNextToken(Token(), "[") ; static array?
                		
                		NextElement(Token())
                		Token()\Token = ""
                		Token()\Type  = #PBSC_Other
                		
                		NextElement(Token())
                		
                		StructureItemType = " = new Array(" + Token()\Token + ")"
                		
                		Token()\Token = ""
                		Token()\Type  = #PBSC_Other
                		
                		NextElement(Token())
                		Token()\Token = ""
                		Token()\Type  = #PBSC_Other
                		
                	Else
                		
                		StructureItemType = " = null"
                		Token()\Token = ""
                		Token()\Type  = #PBSC_Other
                		
                	EndIf                	
                	
                EndIf
                
                StructureLine + "this." + StructureItemName + StructureItemType + ";" + #LF$
                
                StructureItemName = ""
                StructureItemType = ""
                
              EndIf
              
            EndIf
            
        EndSelect
        
    EndSelect
    
  Next
  
;   ForEach Token()
;   	If Token()\Token = "."
;   		NextElement(Token())
;   		If FindString(Token()\Token, "p2w_structure_")
; 	   		PreviousElement(Token())
; 	   		PreviousElement(Token())
; 	   		Debug "Stuctvar: " + Token()\Token
; 	  		NextElement(Token())
; 	  		NextElement(Token())
;   		EndIf
;   	ElseIf LCase(Token()\Token) = "structure"
;   		NextElement(Token())
;   		; Token()\Token = "p2w_structure_" + LCase(Token()\Token)
;   	EndIf
;   Next  
  
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessWhileWend(List TT.Token())
  
  AddLog("Processing: While - Wend...")
  
  Protected WhileTerm.s
  Protected currentToken
  Protected NextToken.s
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
            
          Case "while"
            
            WhileTerm = "while ("
            
            currentToken = ListIndex(TT())
            
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            
            Repeat
              
              If TT()\Type = #PBSC_Newline : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
              Select TT()\Token
                  
                Case "="
                  
                  NextElement(TT())
                  NextToken = TT()\Token
                  
                  If NextToken = ">" Or NextToken = "<"
                    TT()\Token = ""
                    PreviousElement(TT())
                    TT()\Token = NextToken + "="
                    NextElement(TT())
                  EndIf
                  
                  PreviousElement(TT())
                  
                  If NextToken <> ">" And NextToken <> "<"
                    TT()\Token = "=="
                  EndIf
                  
                Case "<>"
                  TT()\Token = "!="							
                  
              EndSelect
              
              WhileTerm + TT()\Token
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              NextElement(TT())
              
            ForEver						
            
            WhileTerm + ") {"
            
            SelectElement(TT(), currentToken)
            
            TT()\Token = WhileTerm
            
          Case "wend"
            TT()\Token = "}"
            
        EndSelect
    EndSelect
  Next
  
EndProcedure

Procedure ProcessRepeatUntil(List TT.Token())
  
  AddLog("Processing: Repeat - Until...")

  Protected Expression.s
  Protected NextToken.s
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
            
          Case "repeat"
            
            TT()\Token = "do {"
            
          Case "until"
            
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            
            NextElement(TT())
            
            Repeat
              
              If TT()\Type = #PBSC_Newline : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
              Select TT()\Token
                  
                Case "="
                  
                  NextElement(TT())
                  NextToken = TT()\Token
                  
                  If NextToken = ">" Or NextToken = "<"
                    TT()\Token = ""
                    PreviousElement(TT())
                    TT()\Token = NextToken + "="
                    NextElement(TT())
                  EndIf
                  
                  PreviousElement(TT())
                  
                  If NextToken <> ">" And NextToken <> "<"
                    TT()\Token = "=="
                  EndIf
                  
                Case "<>"
                  TT()\Token = "!="							
                  
              EndSelect
              
              Expression + TT()\Token
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
              NextElement(TT())
              
            ForEver						
            
            PreviousElement(TT())
            
            TT()\Token = " } while (!( " + Expression + " ));"
            
            Expression = ""
            
          Case "forever"
            
            TT()\Token = " } while ( true );"
            
        EndSelect
    EndSelect
  Next
  
EndProcedure

Procedure ProcessForNext(List TT.Token())
  
  AddLog("Processing: For - Next...")

  ; ----
  ; For -- Next
  ; ----
  
  Protected inFor
  Protected ForVar.s
  Protected ForStart.s
  Protected ForEnd.s
  Protected ForStep.s
  
  ForEach TT()
    
    Select TT()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(TT()\Token)
            
          Case "for"
            
            inFor = #True
            
            TT()\Token = " for ( "
            
            ; nchste element: variable
            NextElement(TT())
            ForVar = TT()\Token
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            
            ; nchste element: =
            NextElement(TT())
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            
            ; nchste element(e): startwert
            
            NextElement(TT())
            ForStart = TT()\Token
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            
            Repeat
              
              If LCase(TT()\Token) = "to"
                TT()\Token = ""
                TT()\Type = #PBSC_Other
                Break 
              EndIf
              
              ForStart + TT()\Token
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              NextElement(TT())
              
            ForEver
            
            ; nchste element: endwert
            
            ForEnd = ForVar + "#groesserkleinergleich#"
            
            Repeat
              
              If TT()\Type = #PBSC_NewLine Or LCase(TT()\Token) = "step"
                Break 
              EndIf
              
              If TT()\Type = #PBSC_Comment
                Break 
              EndIf
              
              ForEnd + TT()\Token
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              NextElement(TT())
              
            ForEver
            
            ForStep = ""
            
            If LCase(TT()\Token) = "step"
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              NextElement(TT())
              
              Repeat
                
                If TT()\Type = #PBSC_Newline : Break : EndIf
                If TT()\Type = #PBSC_Comment : Break : EndIf
                
                ForStep + TT()\Token
                TT()\Token = ""
                TT()\Type = #PBSC_Other
                NextElement(TT())
                
              ForEver
              
            EndIf
            
            PreviousElement(TT())
            
            TT()\Token = ForVar + "=" + ForStart + ";" + ForEnd + "; " 
            
            If ForStep = ""
            	TT()\Token + ForVar + "++) {"
            	TT()\Token = ReplaceString(TT()\Token, "#groesserkleinergleich#", "<=")
            Else
            	
            	If Left(ForStep, 1) = "-"
              	TT()\Token + ForVar + "=" + ForVar + ForStep + ") {"
	            	TT()\Token = ReplaceString(TT()\Token, "#groesserkleinergleich#", ">=")
            	Else
              	TT()\Token + ForVar + "=" + ForVar + "+" + ForStep + ") {"
  	          	TT()\Token = ReplaceString(TT()\Token, "#groesserkleinergleich#", "<=")
            	EndIf
            	
            EndIf
            
          Case "next"
            
            Repeat
              
              If TT()\Type = #PBSC_Newline : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              NextElement(TT())
              
            ForEver
            
            PreviousElement(TT())
            inFor = #False
            TT()\Token = "}"
            
        EndSelect
        
    EndSelect
    
  Next
  
EndProcedure

Procedure ProcessComments(List TT.Token())
  
  AddLog("Processing: Comments...")

  Protected inRaw
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
          Case LCase("P2W_Converter_Begin_Raw"), LCase("P2W_EnableJS"), LCase("EnableJS")
            inRaw = #True
          Case LCase("P2W_Converter_End_Raw"), LCase("P2W_DisableJS"), LCase("DisableJS")
            inRaw = #False            
        EndSelect
      Case #PBSC_Comment
        If Not inRaw
          TT()\Token = "// " + TT()\Token
        EndIf
    EndSelect
  Next
  
EndProcedure

Procedure ProcessCommentsPHP(List TT.Token())
  
  AddLog("Processing: Comments...")
  
  Protected Inline
  
  ForEach TT()
  	
  	Select TT()\Type
  			
  		Case #PBSC_NewLine
  			
  			If CheckNextToken(TT(), "!")
  				Inline = #True
  			Else
  				Inline = #False
  			EndIf
  			
  		Case #PBSC_Comment
  			If Not Inline
  				TT()\Token = " // " + TT()\Token
  			EndIf
      	
  	EndSelect
  	
  Next
  
EndProcedure

Procedure GetProcedureInformations()
	
	ProcedureReturn
	
	Protected IsMap
	Protected IsLinkedList
	Protected IsOptional
	
	ForEach Token()
		Select Token()\Type
			Case #PBSC_Identifier
				Select LCase(Token()\Token)
						
					Case "procedure"
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Token <> "" 
								
								AddElement(Procedures())
								Procedures()\Name = Token()\Token
								Procedures()\Line = Token()\Line
								
								Break
								
							EndIf
							
						ForEver
						
						; ---
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Type = #PBSC_Newline : Break : EndIf
							If Token()\Type = #PBSC_Comment : Break : EndIf
							
							If Token()\Token = "(" : Continue : EndIf
							If Token()\Token = ")" : Continue : EndIf
							If Token()\Token = ""  : Continue :  EndIf
							
							If Token()\Token = "," 
								IsMap = #False 
								IsLinkedList = #False 
								IsOptional = #False
								Continue
							EndIf
							
							If Token()\Token = "="
								IsOptional = #True
								Procedures()\Params()\IsOptional = IsOptional
								Continue
							EndIf
							
							If IsOptional
								Continue
							EndIf
							
							If Token()\Token = "List" 
								IsLinkedList = #True 
								Continue
							ElseIf Token()\Token = "Map" 
								IsMap	= #True 
								Continue
							EndIf
							
							AddElement(Procedures()\Params())
							
							Procedures()\Params()\Name         = Token()\Token
							Procedures()\Params()\IsMap        = IsMap	
							Procedures()\Params()\IsLinkedList = IsLinkedList
							
						ForEver
						
				EndSelect
		EndSelect
	Next
	
; 	ForEach Procedures()
; 		Debug "Procedure: " + Procedures()\Name
; 		Debug "|"
; 		ForEach Procedures()\Params()
; 			Debug "|-Param (Name): "         + Procedures()\Params()\Name
; 			Debug "|-Param (IsMap): "        + Procedures()\Params()\IsMap
; 			Debug "|-Param (IsLinkedList): " + Procedures()\Params()\IsLinkedList
; 			Debug "|-Param (IsOptional): "   + Procedures()\Params()\IsOptional
; 			Debug "|"
; 		Next
; 	Next
	
EndProcedure

Procedure ProcessLinkedLists()
	
  AddLog("Processing: LinkedLists...")

  NewMap LL_Name.s()
  NewMap LL_Type.s()
  
  Protected LL.s
  Protected isLinkedList
  Protected *Old_Element
  
  ; LinkedLists ermitteln und in eine Map schreiben
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
        		
        		; TODO: Liste aller Procedures erstellen und ermitteln, an welcher Stelle List bergeben werden (wg. Data())

          Case "newlist" ; , "list"
            
            NextElement(Token()) ; LinkedList-Name
            
            LL = Token()\Token
            LL_Name(LL)=LL
            
            Repeat
              
              NextElement(Token())
              
              If Token()\Token = "."
              	
                NextElement(Token())
                LL_Type(LL) = Token()\Token
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                NextElement(Token())
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                NextElement(Token())
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                Break
                
              ElseIf Token()\Token = "("
              	
                LL_Type(LL) = "nostruct"
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                NextElement(Token())
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                Break
                
              EndIf
              
              If Token()\Token = "("
                Break
              EndIf
              
              If Token()\Type = #PBSC_Newline : Break : EndIf
              If Token()\Type = #PBSC_Comment : Break : EndIf
              
            ForEver
            
        EndSelect
    EndSelect
  Next
  
;   ForEach LL_Name()
;   	Debug "LL_Name(): " + LL_Name()
;   Next
  
  
  Protected inIf
	Protected LastCommand.s 
    
  ForEach Token()
  	
  	Select LCase(Token()\Token)
  		Case "if", "elseif"
  			inIf = #True
  		Case "endif"
  			inIf = #False	
  	EndSelect
  	
  	If Token()\Type = #pbsc_newline
 			inIf = #False	
  	EndIf
  	
  	
    Select Token()\Type
        
      Case #PBSC_Identifier
      	
      	Select LCase(Token()\Token)
      			
      		Case "cjson.insertjsonlist", 
      		     "cjson.extractjsonlist"
      			
      			LastCommand = Token()\Token
      			
      		Default
      			
;       			Protected ProcedureCall.s = ""
;       			
;       			ForEach Procedures()
;       				If Procedures()\Name = Token()\Token And Procedures()\Line <> Token()\Line
;       					ProcedureCall = Token()\Token 
;       					Break
;       				EndIf
;       			Next
;       			
;       			If ProcedureCall <> ""
;       				Debug "ProcedureCall: " + ProcedureCall
;       			EndIf
      			
    
      	EndSelect
      	
      	; ----
      	
        isLinkedList = #False
        
        If FindMapElement(LL_Name(), Token()\Token)
        	
          Protected ssss.s = LL_Type(Token()\Token)
          
          *Old_Element = @Token()
          
          NextElement(Token())
          If Token()\Token = "("
            NextElement(Token())
            If Token()\Token = ")"
              
              Repeat
                isLinkedList = #True
                NextElement(Token())
              Until	Token()\Type = #PBSC_Newline Or 
                   	Token()\Type = #PBSC_Comment
              
            EndIf        
          EndIf        
          
          ChangeCurrentElement(Token(), *Old_Element)
          
          If isLinkedList
          	
            If ssss = "nostruct"
              
              NextElement(Token())
              NextElement(Token())
              NextElement(Token())
              
              If Token()\Token = "="
              	
              	If Not inIf
              		
              		PreviousElement(Token())
              		PreviousElement(Token())
              		PreviousElement(Token())
              		
              		NextElement(Token())
              		Token()\Token = ""
              		Token()\Type = #PBSC_Other
              		
              		NextElement(Token())
              		Token()\Token = ""
              		Token()\Type = #PBSC_Other
              		
              		NextElement(Token())
              		Token()\Token = ""
              		Token()\Type = #PBSC_Other
              		
              		NextElement(Token())
              		
              		Protected LL_Value.s = ""
              		
              		Repeat
              			
              			If Token()\Type = #PBSC_Newline : Break : EndIf
              			If Token()\Type = #PBSC_Comment : Break : EndIf
              			
              			Select Token()\Token
              					
              				Case "("
              					Token()\Token = ""
              					Token()\Type = #PBSC_Other
              					
              				Case ")"
              					Token()\Token = ".GetContent()"
              					Token()\Type = #PBSC_Other
              					
              				Case "\"
              					Token()\Token = "."
              					Token()\Type = #PBSC_Other
              					
              				Default
              					
              			EndSelect
              			
              			LL_Value + Token()\Token
              			
              			Token()\Token = ""
              			
              			NextElement(Token())
              			
              		ForEver
              		
              		PreviousElement(Token())
              		Token()\Token = ".SetContent(" + LL_Value + ")"
              		
              	Else
              		
              		PreviousElement(Token())
              		Token()\Token = ""
              		PreviousElement(Token())
              		Token()\Token = ".GetContent()" 
              		
              	EndIf
              	
              Else
              	
                PreviousElement(Token())
                PreviousElement(Token())
                PreviousElement(Token())
                
                Select LastCommand 
                		
                	Case "cJson.InsertJSONList",
                	     "cJson.ExtractJSONList"
                		
                		LastCommand = ""
                		
                	Default
                		
                		Token()\Token = Token()\Token + ".GetContent()"
                		
                EndSelect
                
                
                NextElement(Token())
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                
                NextElement(Token())
                Token()\Token = ""
                Token()\Type = #PBSC_Other
                
              EndIf
              
            Else
            	
            	Select LastCommand 
            			
            		Case "cJson.InsertJSONList",
            		     "cJson.ExtractJSONList"
            			
            			LastCommand = ""
            			
            		Default
            			
            			Token()\Token = Token()\Token + ".GetContent()"
            			
            	EndSelect
            	
            	
            	NextElement(Token())
            	Token()\Token = ""
            	Token()\Type = #PBSC_Other
            	
            	NextElement(Token())
            	Token()\Token = ""
            	Token()\Type = #PBSC_Other
            	
            EndIf
            
          EndIf
          
        EndIf                
        
    EndSelect
    
  Next
  
  Protected LLToken.s
  Protected Action.s
	Protected OPOP.s

  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
      	
        Select LCase(Token()\Token)
        		
          Case "newlist"
          	
          	;(
          	
          	Token()\Token = ""
          	
          	NextElement(Token())
          	
          	If LL_Type(Token()\Token) = "nostruct"
          		Token()\Token = "var " + Token()\Token + " = new cLinkedList();"
          	Else
							Token()\Token = "var " + Token()\Token + " = new cLinkedList();"
						EndIf
          	
          	Repeat
          		
          		NextElement(Token())
          		
          		If Token()\Type = #PBSC_Newline : Break : EndIf
          		If Token()\Type = #PBSC_Comment : Break : EndIf
          		
          		Token()\Token = ""
          		
          	ForEver
          	
            ;)
            
          Case "addelement",
               "insertelement"
               
          	;(
          	
          	If Not CheckNextToken(Token(), "(")
          		Continue
          	EndIf
         	
          	OPOP = Token()\Token
          	
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; klammer auf
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; LL
            
            LLToken = Token()\Token
            LLToken = RemoveString(LLToken, ".GetContent()", #PB_String_NoCase)
            
          	If LL_Type(LLToken) = "nostruct"
  	          Token()\Token = LLToken + "." + OPOP + "()"
  	        Else
	            Token()\Token = LLToken + "." + OPOP + "( jQuery.extend(true, {}, " + LL_Type(LLToken) + ") )"
            EndIf
            
            NextElement(Token()) ; klammer auf
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; klammer zu
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; klammer zu
            Token()\Token = ""
            Token()\Type = #PBSC_Other            
            
            ;)
            
          Case "clearlist", 
               "resetlist", 
               "csort.randomizelist",
               "firstelement", 
               "lastelement",
               "listdata", 
               "listindex", 
               "listsize", 
               "nextelement",
               "previouselement",
               "pushlistposition",
               "poplistposition"
          	
          	;(
          	
          	If Not CheckNextToken(Token(), "(")
          		Continue
          	EndIf
          		
          	OPOP = Token()\Token
          	
          	Token()\Token = ""
          	Token()\Type = #PBSC_Other
          	
          	NextElement(Token()) ; klammer auf
          	Token()\Token = ""
          	Token()\Type = #PBSC_Other
          	
          	NextElement(Token()) ; LL
          	
          	If Token()\Token = "@"
          		Token()\Token = ""
          		NextElement(Token()) ; LL
          	EndIf
          	
          	LLToken = Token()\Token
          	
          	If LCase(OPOP)="csort.randomizelist"
          		LLToken = ReplaceString(LLToken, ".GetContent()", "")            
          		Token()\Token = OPOP + "(" + LLToken + ")"
          	Else
          		LLToken = ReplaceString(LLToken, ".GetContent()", "")            
          		Token()\Token = LLToken + "." + OPOP + "()"
          	EndIf
          	
          	
          	NextElement(Token()) ; klammer auf
          	If Token()\Token = "("
          		Token()\Token = ""
          		Token()\Type = #PBSC_Other
          	EndIf
          	
          	NextElement(Token()) ; klammer zu
          	If Token()\Token = ")"
          		Token()\Token = ""
          		Token()\Type = #PBSC_Other
          	EndIf
          	
          	NextElement(Token()) ; klammer zu
          	If Token()\Token = ")"
          		Token()\Token = ""
          		Token()\Type = #PBSC_Other
          	EndIf
          
            ;)
            
          Case "selectelement", 
               "deleteelement",
               "swapelements",
               "changecurrentelement"
          	
          	;(
          	
            OPOP = Token()\Token
            
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; klammer auf
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; LL
            
            LLToken = Token()\Token
            LLToken = ReplaceString(LLToken, ".GetContent()", "")            

            Token()\Token = LLToken + "." + OPOP + "("
            
            NextElement(Token()) ; klammer auf
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token()) ; klammer zu
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            
            NextElement(Token()) ; klammer zu
						If Token()\Token <> ")"
            	Token()\Token = ""
            	Token()\Type = #PBSC_Other
            EndIf
            
            ;)
            
          Case "moveelement"
          	
          	;(
          	
          	; MoveElement(List(), Location [, *RelativeElement]) -> List.MoveElement(Location [, *RelativeElement])
          	
          	OPOP = Token()\Token ; MoveElement
          	Token()\Token = ""
          	
          	NextElement(Token()) ; (
          	Token()\Token = ""
          	
          	NextElement(Token()) ; List
          	OPOP = RemoveString(Token()\Token, ".GetContent()", #PB_String_NoCase) + "." + OPOP
          	Token()\Token = ""
          	
          	NextElement(Token()) ; (
          	Token()\Token = ""
          	
          	NextElement(Token()) ; )
          	Token()\Token = ""
          	
          	NextElement(Token()) ; ,
          	Token()\Token = ""
          	
          	Token()\Token = OPOP + "("
          	
            ;)
            
          Case "copylist",
               "mergelists",
               "splitlist"
          	
          	;(
          	
          	; Result = CopyList(SourceList(), DestinationList()) -> SourceList.CopyList(DestinationList);
          	
          	Action = Token()\Token
          	
						Token()\Token = ""
						
						NextElement(Token()) ; (
						Token()\Token = ""
						
            ; ---
            
            Protected SourceList.s = ""
            
            Repeat
            	
            	NextElement(Token())
            	
            	If Token()\Token = "," 
            		Token()\Token = "" 
            		Break 
            	EndIf
            	
            	SourceList + Token()\Token
            	Token()\Token = ""
            	
            ForEver
            
            SourceList = Left(SourceList, Len(SourceList) - Len(".GetContent()"))
            
            Action = SourceList + "." + Action
            
            ; ---
            
          	Protected DestinationList.s = ""
            
            Repeat
            	
            	NextElement(Token())
            	
            	If Token()\Token = ")" Or Token()\Token = ","
            		
            		If Token()\Token = ","
            			Repeat
            				If Token()\Token = ")"
			            		Break 
            				EndIf
            				Token()\Token = ""
            				NextElement(Token())
            			ForEver
            		EndIf
            		
            		Token()\Token = "" 
            		Break 
            		
            	EndIf
            	
            	DestinationList + Token()\Token
            	Token()\Token = ""
            	
            ForEver
            
            DestinationList = Left(DestinationList, Len(DestinationList) - Len(".GetContent()"))            
            
            ; ---
            
            Token()\Token = Action + "(" + DestinationList + ")"
            
            ;)
            
          Case "foreach"
          	
          	;( Case "foreach" start
          	
          	; test if next element is a linkedlist
          	
          	NextElement(Token())
          	
          	If FindMapElement(LL_Name(), RemoveString(Token()\Token, ".GetContent()"))
          		; Debug Token()\Token + " as LL specified"
          		PreviousElement(Token())
          	Else
          		; Debug Token()\Token + " not as LL specified"
          		Continue
          	EndIf
            
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token())
            
            If Token()\Token = "("
              Token()\Token = ""
              NextElement(Token())
            EndIf
            
            
            LLToken = Token()\Token
            LLToken = ReplaceString(LLToken, ".GetContent()", "")            
            
            Token()\Token = LLToken + ".ForEach_Start();" + #LF$ + "while (" + LLToken + ".ForEach()) {" + #LF$
            Token()\Type = #PBSC_Other
            
            NextElement(Token())
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token())
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token())
            If Token()\Token = ")"
              Token()\Token = ""
            EndIf
            
            ; find the corresponding 'Next'
            
            Protected LLIndex = ListIndex(Token())
            
            Protected ForCounter = 0
            
            Repeat
            	
            	NextElement(Token())
            	
            	If LCase(Token()\Token) = "for" Or LCase(Token()\Token) = "foreach"
            		ForCounter + 1
            	EndIf
            	
            	If LCase(Token()\Token) = "next"
            		ForCounter - 1
            	EndIf
            	
            	If ForCounter = -1
            		
            		Repeat
            			
            			NextElement(Token())
            			
            			If Token()\Type = #PBSC_NewLine : Break : EndIf	
            			
            		ForEver
            		
            		Token()\Token + LLToken + ".ForEach_End();" + #LF$
            		
            		Break	
            	EndIf
            	
            ForEver
            
            SelectElement(Token(), LLIndex)
            
            ;) Case "foreach" end
            
        EndSelect
        
    EndSelect
    
  Next
  
  RemoveEmptyToken(Token())
  
EndProcedure

Procedure ProcessMaps()
	
	AddLog("Processing: Maps...")
	
	NewMap Map_Name.s()
	NewMap Map_Type.s()
	
	Protected Map.s
	
	; Maps ermitteln und in eine Map schreiben
	
	;(
	
	ForEach Token()
		
		Select Token()\Type
				
			Case #PBSC_Identifier
				
				Select LCase(Token()\Token)
						
					Case "newmap"
						
						NextElement(Token()) ; Map-Name
						
						Map = Token()\Token
						Map_Name(Map) = Map
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Token = "."
								NextElement(Token())
								Map_Type(Map) = Token()\Token
								Token()\Token = ""
								Token()\Type = #PBSC_Other
								NextElement(Token())
								Token()\Token = ""
								Token()\Type = #PBSC_Other
								NextElement(Token())
								Token()\Token = ""
								Token()\Type = #PBSC_Other
								Break
							ElseIf Token()\Token = "("
								Map_Type(Map) = "nostruct"
								Token()\Token = ""
								Token()\Type = #PBSC_Other
								NextElement(Token())
								Token()\Token = ""
								Token()\Type = #PBSC_Other
								Break
							EndIf
							
							If Token()\Token = "(" : Break : EndIf
							
							If Token()\Type = #PBSC_Newline : Break : EndIf
							If Token()\Type = #PBSC_Comment : Break : EndIf
							
						ForEver
						
				EndSelect
		EndSelect
	Next
	
	;   ForEach Map_Name()
	;   	Debug "Map-Name: " + Map_Name()
	;   	Debug "Map-Type: " + Map_Type(Map_Name())
	;   Next
	
	;)
	
	Protected isMap
	Protected *Old_Element
	Protected Assignment
	Protected withKey 
	Protected MapKey.s
	Protected inIf
	
	ForEach Token()
		
		If LCase(Token()\Token)="if"
			inIf = #True
		ElseIf LCase(Token()\Token)="endif"	
			inIf = #False
		EndIf
		
		If Token()\Type = #pbsc_newline
			inIf = #False	
		EndIf  	
		
		Select Token()\Type
				
			Case #PBSC_Identifier
				
				isMap = #False
				Assignment = #False
				withKey = #False
				MapKey = ""
				
				If FindMapElement(Map_Name(), Token()\Token)
					
					Protected MapType.s = Map_Type(Token()\Token)
					
					*Old_Element = @Token()
					
					NextElement(Token())
					
					If Token()\Token = "("
						
						isMap = #True
						
						NextElement(Token())
						
						If Token()\Token = ")" ; myMap() oder myMap()="hallo"
							
							withKey = #False
							
							NextElement(Token())
							
							If Token()\Token = "=" And Not inIf
								Assignment = #True
							EndIf
							
						Else ; myMap(myKey) oder myMap(myKey)="huhu" 
							
							withKey = #True
							
							Define KlammerAuf = 1
							
							Repeat
								
								If Token()\Token ="("
									KlammerAuf + 1
								EndIf
								
								If Token()\Token =")"
									KlammerAuf - 1
								EndIf
								
								If Token()\Token ="\"
									Token()\Token = "."
								EndIf
								
								If KlammerAuf = 0
									Token()\Token = ""
									Break
								EndIf 
								
								If Token()\Type = #PBSC_Newline : Break : EndIf
								If Token()\Type = #PBSC_Comment : Break : EndIf
								
								MapKey + Token()\Token
								Token()\Token = ""
								
								NextElement(Token())
								
							ForEver              
							
							NextElement(Token())
							
							If Token()\Token = "=" And Not inIf
								Assignment = #True
							EndIf
							
						EndIf
						
					EndIf        
					
					ChangeCurrentElement(Token(), *Old_Element)
					
					If isMap
						
						If withKey
							
							If Assignment
								
								NextElement(Token())
								Token()\Token=""
								
								NextElement(Token())
								Token()\Token=""
								
								NextElement(Token())
								Token()\Token=""
								
								NextElement(Token())
								Token()\Token=""
								
								NextElement(Token())
								
								If MapType = "nostruct"
									
									If Token()\Token
										
										Protected TI = ListIndex(Token())
										Protected TT.s = Token()\Token
										
										Repeat
											
											NextElement(Token())
											
											If Token()\Type = #PBSC_Newline : Break : EndIf
											If Token()\Type = #PBSC_Comment : Break : EndIf                			
											
											TT + Token()\Token
											Token()\Token = ""
											
										ForEver
										
										SelectElement(Token(), TI)
										
										Token()\Token = ".setData(" + TT + ", " + MapKey + ")"
										
									Else
										Token()\Token = ".setData(" + MapKey + ")"
									EndIf
									
								Else
									
									Token()\Token = ".getData(" + MapKey + ")." + MapType + " = " + Token()\Token
									
								EndIf
								
							Else
								
								NextElement(Token())
								Token()\Token = ".getData(" + MapKey + ")"
								
							EndIf
							
						Else
							
							If Assignment
								
								Protected TempToken.s
								
								TempToken = Token()\Token + ".setData("
								
								Token()\Token = ""
								
								NextElement(Token())
								Token()\Token=""
								NextElement(Token())
								Token()\Token=""
								NextElement(Token())
								Token()\Token=""
								
								NextElement(Token())
								TempToken + Token()\Token + ")"
								
								Token()\Token = TempToken
								
							Else
								
								*Old_Element = @Token()
								
								Protected LastCommand.s = ""
								
								Repeat
									
									If token()\Type = #pbsc_newline
										NextElement(Token())
										LastCommand = token()\Token
										Break
									EndIf
									
									PreviousElement(Token())
									
								ForEver
								
								ChangeCurrentElement(Token(), *Old_Element)
								
								Select LastCommand 
										
									Case "cJson.InsertJSONMap",
									     "cJson.ExtractJSONMap"
										
										LastCommand = ""
										
									Default
										
										Token()\Token = Token()\Token + ".getData()"
										
								EndSelect
								
								NextElement(Token())
								Token()\Token=""
								NextElement(Token())
								Token()\Token=""
								
							EndIf
							
						EndIf               
						
					EndIf
					
				EndIf                
				
		EndSelect
		
	Next
	
	Protected MapToken.s
	Protected Action.s 
	
	; + NewMap
	; + AddMapElement()
	; + ClearMap()
	; + CopyMap()
	; + DeleteMapElement()
	; + FindMapElement()
	; + FreeMap()
	; + MapKey()
	; + MapSize()
	; + NextMapElement()
	; + PopMapPosition()
	; + PushMapPosition()
	; + ResetMap()
	
	ForEach Token()
		
		Select Token()\Type
				
			Case #PBSC_Identifier
				
				Select LCase(Token()\Token)
						
					Case "newmap"
						
						;(
						
						; NewMap name.<type>([Slots]) -> name = new cMap(type)
						
						; NewMap Cars.Car() -> Cars = new cMap(Car)
						
						Token()\Token = ""
						
						NextElement(Token())
						
						If Map_Type(Token()\Token) = "nostruct"
							Token()\Token = "var " + Token()\Token + " = new cMap(); "
						Else
							Token()\Token = "var " + Token()\Token + " = new cMap(" + Map_Type(Token()\Token) + ")"
						EndIf
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Type = #PBSC_Newline : Break : EndIf
							If Token()\Type = #PBSC_Comment : Break : EndIf
							
							Token()\Token = ""
							
						ForEver
						
						;)
						
					Case "findmapelement", 
					     "addmapelement",
					     "deletemapelement"
						
						;(
						
						; Result = AddMapElement   (Map(),   Key$ [, Flags]) -> Map.AddMapElement(Key$ [, Flags])
						; Result = FindMapElement  (Map(),   Key$)           -> Map.FindMapElement(Key$)
						; Result = DeleteMapElement(Map() [, Key$])          -> Map.DeleteMapElement([Key$])
						
						Action = Token()\Token
						Token()\Token = ""
						
						NextElement(Token()) ; (
						Token()\Token = ""
						
						NextElement(Token())
						Action = RemoveString(Token()\Token, ".getData()") + "." + Action + "("
						Token()\Token = ""
						
						Define isComma = #False
						
						Repeat
							
							If Token()\Type = #PBSC_Newline : Break : EndIf
							If Token()\Type = #PBSC_Comment : Break : EndIf
							
							If Token()\Token = "," 
								isComma = #True
								Token()\Token = "" 
							EndIf
							
							If isComma
								Action + Token()\Token
							EndIf
							
							Token()\Token = ""
							
							NextElement(Token())
							
						ForEver
						
						PreviousElement(Token())
						
						Token()\Token = Action
						
						;)
						
					Case "copymap"
						
						;(
						
						; Result = CopyMap(SourceMap(), DestinationMap()) -> CopyMap(SourceMap , DestinationMap);
						
						Action = "cMap." + Token()\Token
						Token()\Token = ""
						
						NextElement(Token()) ; (
						Token()\Token = ""
						
						; ---
						
						Protected SourceMap.s = ""
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Token = "," 
								Token()\Token = "" 
								Break 
							EndIf
							
							SourceMap + Token()\Token
							Token()\Token = ""
							
						ForEver
						
						SourceMap = Left(SourceMap, Len(SourceMap) - Len(".getData()"))
						
						; ---
						
						Protected DestinationMap.s = ""
						
						Repeat
							
							NextElement(Token())
							
							If Token()\Token = ")" 
								Token()\Token = "" 
								Break 
							EndIf
							
							DestinationMap + Token()\Token
							Token()\Token = ""
							
						ForEver
						
						DestinationMap = Left(DestinationMap, Len(DestinationMap) - Len(".getData()"))            
						
						; ---
						
						Token()\Token = Action + "(" + SourceMap + ", " + DestinationMap + ")"
						
						;)
						
					Case "mapkey",
					     "mapsize",
					     "clearmap",
					     "resetmap",
					     "freemap",
					     "nextmapelement",
					     "popmapposition",
					     "pushmapposition"
						
						;(
						
						; Key$   = MapKey(Map())          -> Map.MapKey()
						; Result = MapSize(Map())         -> Map.MapSize()
						;          ClearMap(Map())        -> Map.ClearMap()
						;          ResetMap(Map())        -> Map.ResetMap()
						;          FreeMap(Map())         -> Map.FreeMap()
						; Result = NextMapElement(Map())  -> Map.NextMapElement()
						;          PopMapPosition(Map())  -> Map.PopMapPosition()
						;          PushMapPosition(Map()) -> Map.PushMapPosition()
						
						Action = Token()\Token
						Token()\Token = ""
						
						NextElement(Token()) ; (
						Token()\Token = ""
						
						Map = ""
						
						Repeat
							
							If Token()\Token = ")" 
								Token()\Token = "" 
								Break 
							EndIf
							
							Map + Token()\Token
							Token()\Token = ""
							
							NextElement(Token())
							
						ForEver
						
						Map = Left(Map, Len(Map) - Len(".getData()"))
						
						Token()\Token = Map + "." + Action + "()" 
						
						;)
						
					Case "foreach"
						
						;(
						
						; test if next element is a map
						
						NextElement(Token())
						
						If FindMapElement(Map_Name(), RemoveString(Token()\Token, ".getData()"))
							; Debug Token()\Token + " as Map specified"
							PreviousElement(Token())
						Else
							; Debug Token()\Token + " not as Map specified"
							Continue
						EndIf
						
						Token()\Token = ""
						Token()\Type = #PBSC_Other
						
						NextElement(Token())
						
						MapToken = Token()\Token
						MapToken = ReplaceString(MapToken, ".getData()", "")
						
						Token()\Token = MapToken + ".ForEach_Init();" + #LF$ + "while (" + MapToken + ".ForEach()) {" + #LF$
						Token()\Type = #PBSC_Other
						
						NextElement(Token())
						Token()\Token = ""
						Token()\Type = #PBSC_Other
						
						NextElement(Token())
						Token()\Token = ""
						Token()\Type = #PBSC_Other
						
						;)
						
				EndSelect
				
		EndSelect
		
	Next
	
	RemoveEmptyToken(Token())
	
EndProcedure

Procedure ProcessArrays(List TT.Token())
  
  AddLog("Processing: Arrays...")

  NewMap ArrayName.s()
  
  Protected IsMultiDim
  Protected CurrentArrayName.s
  Protected CurrentStructure.s
  
  ForEach TT()
    
    Select TT()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(TT()\Token)
        		
        	Case "redim"
        		
        		TT()\Token = "cArray.ReDim("
        		
            Repeat
              
              If TT()\Type = #PBSC_Newline : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
            	Select TT()\Token
              	Case "("
 	              	TT()\Token = " [ "
	              Case ")"
 	              	TT()\Token = " ]) "
	              	Break	
	            EndSelect
              
              NextElement(TT())
              
            ForEver
            
          Case "dim"
            
            ; Dim myArray(100)            -> myArray = cArray.Dim(null, [100]);
            ; Dim myArray(100, 100)       -> myArray = cArray.Dim(null, [100, 100]);
          	; Dim myArray.sStructure(100) -> myArray = cArray.Dim(sStructure, [100]);
            
	          TT()\Token = "var "
            
            TT()\Type = #PBSC_Other
            
            ; nchste element: variable
            NextElement(TT())
            
            ArrayName(TT()\Token) = TT()\Token
            
            CurrentArrayName = TT()\Token
            
            Protected CI = ListIndex(TT())
            
            CurrentStructure = ""
            
            If CheckNextToken(TT(), ".")
            	NextElement(TT())
            	TT()\Token = ""
            	NextElement(TT())
            	CurrentStructure = TT()\Token
            	TT()\Token = ""
            EndIf
            
            Repeat
              
            	Select TT()\Token
              	Case "("
 	              	TT()\Token = " ["
	              Case ")"
 	              	TT()\Token = " ])"
	              	Break	
	            EndSelect
	            
              NextElement(TT())
              
            ForEver
            
            TT()\Token + #LF$
            
            SelectElement(TT(), CI)
            
            If CurrentStructure = ""
            	TT()\Token = CurrentArrayName + " = cArray.Dim(null, "
            Else
 	            TT()\Token = CurrentArrayName + " = cArray.Dim(" + CurrentStructure + ", "
 	          EndIf	
 	          
 	        Case "carray.arraysize"
 	        	
 	        	NextElement(TT()) ; (
 	        	NextElement(TT()) ; [Array]
 	        	NextElement(TT())	; (
 	        	TT()\Token = ""
 	        	NextElement(TT())	; )
 	        	TT()\Token = ""
 	        	
            
        EndSelect
        
    EndSelect
    
  Next
  
    ; Debug "#################"
    
    ForEach ArrayName()
      Debug ArrayName()
    Next
    
    ; Debug "#################"
  
  Protected KlammerAuf
  
  ForEach TT()
    
    Select TT()\Type
        
  		Case #PBSC_Identifier
  			
  			If FindMapElement(ArrayName(), TT()\Token)
  			  
  			  ; position merken
  			  Protected selectedToken = ListIndex(TT())
  			  
          If CheckNextToken(TT(), "(")
            
            NextElement(TT())
            
            If CheckNextToken(TT(), ")")
              
              ; hier geht die Klammer sofort wieder zu, wie z.b.
              ; RandomizeArray(myArray())
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
              NextElement(TT())
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
            Else
              
              TT()\Token = "["
              
              KlammerAuf = 1
              
              IsMultiDim = #False
              
              Repeat
                
                If TT()\Token ="("
                  KlammerAuf + 1
                EndIf
                
                If TT()\Token =")"
                  KlammerAuf - 1
                EndIf
                
                If TT()\Token ="," And KlammerAuf = 1 ; ???
                	IsMultiDim = #True
                	TT()\Token = "]["
                EndIf
                
                If TT()\Token =")" And KlammerAuf = 0
                  TT()\Token = "]"
                EndIf
                
                If KlammerAuf = 0
                  Break
                EndIf 
                
                If TT()\Token = "="
                  Break
                EndIf
                
                If TT()\Type = #PBSC_Newline : Break : EndIf
                If TT()\Type = #PBSC_Comment : Break : EndIf
                
                NextElement(TT())
                
              ForEver
              
            EndIf
            
          EndIf
          
          ; position zurcksetzen
          SelectElement(TT(), selectedToken)
          
        EndIf
        
    EndSelect
    
  Next
  
  FreeMap(ArrayName())
  
;   ; Result = CopyArray(SourceArray(), DestinationArray())
; 	; CopyArray()
;   
;   Protected SourceArray.s
;   Protected DestinationArray.s
;   Protected IsIf
;   
;   ForEach TT()
;   	
;   	IsIf = #False
;   	
;   	If FindString(LCase(TT()\Token), "carray.copyarray")
;   		
;   		; If CopyArray
;   		; CopyArray
;   		
;   		CI = ListIndex(TT())
;   		
;   		PreviousElement(TT())
;   		If LCase(TT()\Token) = "if"
;   			CI = ListIndex(TT())
;   			IsIf = #True
; 	  	EndIf
;   		NextElement(TT())
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; (
;   		
;   		If IsIf
;   			TT()\Token = " (true) {"
;   		Else
;   			TT()\Token = ""
;   		EndIf
;   		
;   		
;   		NextElement(TT()) ; SourceArray
;   		SourceArray = TT()\Token
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; (
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; )
;   		TT()\Token = ""
;   		
;   		; ---
;   		
;   		NextElement(TT()) ; ,
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; DestinationArray
;   		DestinationArray =  TT()\Token
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; (
;   		TT()\Token = ""
;   		
;   		NextElement(TT()) ; )
;   		TT()\Token = ""
;   		
;   		Repeat
;   			
;   			If TT()\Type = #PBSC_Newline : Break : EndIf
;   			If TT()\Type = #PBSC_Comment : Break : EndIf
;   			
;   			TT()\Token = ""
;   			NextElement(TT())
;   			
;   		ForEver  		
;   		
;   		SelectElement(TT(), CI)
;   		
;   		TT()\Token = DestinationArray + " = cArray.CopyArray(" + SourceArray + ")" + #LF$ + TT()\Token
;   		
;   	EndIf
;   	
;   Next
  
  
EndProcedure

Procedure ProcessProcedure(List TT.Token())
  
  AddLog("Processing: Procedure - EndProcedure...")

  ; ----
  ; Procedure - ProcedureReturn - EndProcedure
  ; ----
  
  Protected inProcedure
  Protected inParams
  ; Protected ProcName.s
  
  NewList OptionalParameters.sOptionalParams()
  
  ForEach TT()
  	
  	Select TT()\Type
  			
  		Case #PBSC_Identifier
  			
  			Select LCase(TT()\Token)
  					
  				Case "procedure", "proceduredll", "procedurecdll"
  					
  					TT()\Token = "function"
  					
  					inProcedure = #True
  					
  					;Protected merker = ListIndex(TT())
						;Repeat
						;	NextElement(TT())
						;	If TT()\Token <> ""
						;		ProcName = TT()\Token
						;		Break
						;	EndIf
						;ForEver
						;SelectElement(TT(), merker)
  					
  				Case "procedurereturn"
  					TT()\Token = "return"
  					
  				Case "endprocedure"
  					TT()\Token = "}" ; + #CRLF$ + "window." + ProcName + " = " + ProcName + ";"
																; ProcName = ""
  					
  				Case "list", "map", "array"
  					
  					TT()\Token = ""
  					TT()\Type = #PBSC_Other
  					
  					NextElement(TT()) ; LinkedList-Var
  					
  					Repeat
  						
  						NextElement(TT()) 
  						
  						If TT()\Token = ")"
  							TT()\Token = ""
  							TT()\Type = #PBSC_Other
  							Break	
  						EndIf
  						
  						TT()\Token = ""
  						TT()\Type = #PBSC_Other
  						
  					ForEver
  					
  				Default
  					
  					If inParams
  						
  						AddElement(OptionalParameters())
  						OptionalParameters()\v1 = TT()\Token
  						
  					EndIf
  					
  			EndSelect
  			
  		Default
  			
  			Select LCase(TT()\Token)
  					
  				Case "=" ; optional parameter
  					
  					If inParams
  						
  						OptionalParameters()\v2 = TT()\Token
  						TT()\Token = ""
  						
  						Repeat
  							
  							If TT()\Token = ")"  : Break : EndIf
  							If TT()\Token = ","  : Break : EndIf
  							
  							OptionalParameters()\v3 + TT()\Token
  							TT()\Token = ""
  							
  							NextElement(TT())
  							
  						ForEver
  						
  						PreviousElement(TT())
  						
  					EndIf
  					
  				Case "("
  					
  					If inProcedure
  						inParams = #True
  					EndIf
  					
  				Case ")"
  					
  					If inProcedure
  						
  						TT()\Token + " {" + #CRLF$
  						
  						inProcedure = #False
  						inParams = #False
  						
  						ForEach OptionalParameters()
  							
  							If OptionalParameters()\v1 <> "" And 
  							   OptionalParameters()\v2 <> "" And
  							   OptionalParameters()\v3 <> ""
  								
  								TT()\Token + "if (typeof " + OptionalParameters()\v1 + " === " + Chr(34) + "undefined" + Chr(34) +
  								                " || " + OptionalParameters()\v1 + " === null) " + OptionalParameters()\v1 + " = " + OptionalParameters()\v3 + ";" + #CRLF$
  								
  							EndIf
  							
  						Next
  						
  						ClearList(OptionalParameters())
  						
  					EndIf
  					
  			EndSelect
  			
  	EndSelect
  	
  Next
    
EndProcedure

Procedure ProcessProcedurePHP(List TT.Token())
  
  AddLog("Processing: Procedure - EndProcedure...")

  ; ----
  ; Procedure - ProcedureReturn - EndProcedure
  ; ----
  
  Protected inProcedure
  Protected inParams
  ; Protected ProcName.s
  
  ForEach TT()
  	
  	Select TT()\Type
  			
  		Case #PBSC_Identifier
  			
  			Select LCase(TT()\Token)
  					
  				Case "procedure", "proceduredll", "procedurecdll"
  					
  					TT()\Token = "function"
  					
  					inProcedure = #True
  					
  				Case "procedurereturn"
  					TT()\Token = "return"
  					
  				Case "endprocedure"
  					TT()\Token = "}" ; + #CRLF$ + "window." + ProcName + " = " + ProcName + ";"
																; ProcName = ""
  					
  				Case "list", "map"
  					
  					TT()\Token = ""
  					TT()\Type = #PBSC_Other
  					
  					NextElement(TT()) ; LinkedList-Var
  					
  					Repeat
  						
  						NextElement(TT()) 
  						
  						If TT()\Token = ")"
  							TT()\Token = ""
  							TT()\Type = #PBSC_Other
  							Break	
  						EndIf
  						
  						TT()\Token = ""
  						TT()\Type = #PBSC_Other
  						
  					ForEver
  					
  				Default
  					
  					If inParams
  						TT()\Token = "$" + TT()\Token
  					EndIf
  					
  			EndSelect
  			
  		Default
  			
  			Select LCase(TT()\Token)
  					
  				Case "("
  					
  					If inProcedure
  						inParams = #True
  					EndIf
  					
  				Case ")"
  					
  					If inProcedure
  						
  						TT()\Token + " {" + #CRLF$
  						
  						inProcedure = #False
  						inParams = #False
  						
  					EndIf
  					
  			EndSelect
  			
  	EndSelect
  	
  Next
    
EndProcedure

Procedure ProcessCallbacks(List TT.Token())
  
  ForEach TT()
  	
    If TT()\Token = "@"
    	
    	TT()\Token = ""
    	
    	NextElement(TT())
    	
    	TT()\Token = ReplaceString(TT()\Token, "getcontent", "GetCurrentElement", #PB_String_NoCase)
    	
      NextElement(TT())
      
      If TT()\Token = "("
        
        NextElement(TT())
        
        If TT()\Token = ")"  
          
          TT()\Token = ""
          
          PreviousElement(TT())
          TT()\Token = ""
          
        EndIf
        
      EndIf
      
    EndIf
    
  Next  
  
EndProcedure

Procedure ProcessStatics()
	
	Protected StaticToken.s
	Protected *Old_Element
	
	ForEach Token()
		Select Token()\Type
			Case #PBSC_Identifier
				Select LCase(Token()\Token)
					Case "static"
						
						Token()\Token = ""
						
						NextElement(Token())
						
						StaticToken = Token()\Token
						
						Token()\Token = "arguments.callee." + Token()\Token
						
						*Old_Element = @Token()
						
						Repeat
							
							;Debug Token()\Token
							
							If LCase(Token()\Token) = "endprocedure"
								Break
							EndIf
							
							NextElement(Token())
							
							If Token()\Token = StaticToken
								Token()\Token = "arguments.callee." + Token()\Token
							EndIf
							
						ForEver
						
						ChangeCurrentElement(Token(), *Old_Element)
						
				EndSelect
		EndSelect
	Next  
	
EndProcedure

Procedure ProcessDebug()
  
  ; Debug "huhu" -> Debug("huhu")
  
  Protected inDebug
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
          Case "debug"
          	
          	inDebug = #True
          	
          	Token()\Token = "cDebugger." + Token()\Token
          	
          	NextElement(Token())
          	
            InsertElement(Token())
            Token()\Token = "("

        EndSelect
        
      Case #PBSC_NewLine, #PBSC_Comment
        
      	If inDebug
      		
          inDebug = #False
          
          InsertElement(Token())
          Token()\Token = ")"
          
        EndIf
        
    EndSelect
    
  Next  
  
EndProcedure

Procedure ProcessEnumerationsAndConstants()
  
  AddLog("Processing: Enumerations and Constants...")
  
  Protected inEnumeration
  Protected EnumCounter = 0
  Protected NamedEnumeration.s
  
  NewMap NamedEnumerationMap()
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier 
        
        Select LCase(Token()\Token)
            
          Case "enumeration"
            
            inEnumeration = #True
            
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
            NextElement(Token())
            
            If Token()\Token = #LF$
              
              EnumCounter = 0
              NamedEnumeration = ""
              
            Else
            	
            	If Token()\Token <> "#" 
            		
            		If Token()\Type = #PBSC_Number
            			
            			EnumCounter = Val(Token()\Token)
            			
            			Token()\Token = ""
            			Token()\Type = #PBSC_Other
            			
            		Else
            			
            			; test if pb_compiler_enumerationvalue
            			
            			Protected NE.s = Token()\Token ; remember current token
            			
            			NextElement(Token())
            			NextElement(Token())
            			
            			If LCase(Token()\Token) = "pb_compiler_enumerationvalue"
            				
            				If NamedEnumeration <> ""
            					NamedEnumerationMap(NE) = NamedEnumerationMap(NamedEnumeration)
            				Else
            					NamedEnumerationMap(NE) = EnumCounter
            				EndIf
            				
            			EndIf
            			
            			PreviousElement(Token())
            			PreviousElement(Token())
            			
            			; 
            			
            			NamedEnumeration = Token()\Token
            			
            			Token()\Token = ""
            			Token()\Type = #PBSC_Other
            			
            			; check if there is a start-value
            			
            			NextElement(Token())
            			
            			If Token()\Token <> #LF$
            				
            				EnumCounter = Val(Token()\Token)
										NamedEnumerationMap(NE) = EnumCounter
            				
	            			Token()\Token = ""
	            			Token()\Type = #PBSC_Other
            			
            			EndIf
            			
            		EndIf
            		
            	EndIf
            	
            EndIf
            
            PreviousElement(Token())
            
          Case "endenumeration"
            
          	inEnumeration = #False
          	
          	If NamedEnumeration <> ""
            	Token()\Token = "cCommon.PB_Compiler_EnumerationValue = " + Str(NamedEnumerationMap(NamedEnumeration)) + ";"
          	Else
            	Token()\Token = "cCommon.PB_Compiler_EnumerationValue = " + Str(EnumCounter) + ";"
          	EndIf
          	
            Token()\Type = #PBSC_Other
            
          Case "pb_compiler_enumerationvalue"
          	
          	PreviousElement(Token())
          	Token()\Token = ""
            Token()\Type = #PBSC_Other
          	
          	NextElement(Token())
          	Token()\Token = ""
            Token()\Type = #PBSC_Other
            
        EndSelect
        
      Case #PBSC_Constant
      	
      	If Left(Token()\Token, 1) = "#"
      		Token()\Token = Mid(Token()\Token, 2)
      	EndIf
      	
      	If inEnumeration
      		
      		If CheckNextToken(Token(), "=")
      			
      			NextElement(Token()) ; =
      			NextElement(Token())
      			
      			EnumCounter = Val(Token()\Token)
      			
      			PreviousElement(Token())
      			PreviousElement(Token())
      			
      			Token()\Token = "var " + Token()\Token
      			
      		Else
      			
      			If NamedEnumeration <> ""
      				Token()\Token = "var " + Token()\Token + " = " + Str(NamedEnumerationMap(NamedEnumeration))
      			Else
      				Token()\Token = "var " + Token()\Token + " = " + Str(EnumCounter)
      			EndIf
      			
      		EndIf
      		
      		EnumCounter + 1
      		
      		NamedEnumerationMap(NamedEnumeration) + 1
      		
      	EndIf
      	
    EndSelect
    
  Next
  
  FreeMap(NamedEnumerationMap())
  
EndProcedure

Procedure ProcessSelectCase(List TT.Token())
	
; ToDo:	
; var x = this.dealer;
; switch (true) {
;     Case (x < 5):
;         alert("less than five");
;         Break;
;     Case (x > 4 && x < 9):
;         alert("between 5 and 8");
;         Break;
;     Case (x > 8 && x < 12):
;         alert("between 9 and 11");
;         Break;
;     Default:
;         alert("none");
;         Break;
; }	
	
	Dim SelectValue.s(100)
	
  AddLog("Processing: Select - Case - Default - EndSelect...")

  ; ----
  ; Select - Case - Default - EndSelect
  ; ----
  
  Protected ReturnValue = #True
  
  Protected FirstIf = #False
  
  Protected inSelect, inCase
  
  Protected CaseValue.s
  
  Protected Level = -1
  
  ForEach TT()
    
    Select TT()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(TT()\Token)
            
          Case "select"
          	
          	Level + 1
          	
            FirstIf = #True
            
            TT()\Token = ""
            TT()\Type = #PBSC_Other
            NextElement(TT())
            
            SelectValue(Level) = ""
            
            Repeat
              
              If TT()\Type = #PBSC_NewLine : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
              If TT()\Token = "\"
              	TT()\Token = "."
              EndIf
              
              SelectValue(Level) + TT()\Token
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
              NextElement(TT())
              
            ForEver
            
          Case "case"
          	
          	CaseValue = ""
          	
            If Not FirstIf
              TT()\Token = "}"
            Else
              TT()\Token = ""
            EndIf
            
            TT()\Type = #PBSC_Other
            
            NextElement(TT())
            
            Repeat
              
              If TT()\Type = #PBSC_NewLine : Break : EndIf
              If TT()\Type = #PBSC_Comment : Break : EndIf
              
              If LCase(TT()\Token) = "to"
                
                TT()\Token = ""
                TT()\Type = #PBSC_Other
                
                NextElement(TT())
                
                CaseValue + "[&&]" + TT()\Token
                
              ElseIf TT()\Token = ","
                
                TT()\Token = ""
                TT()\Type = #PBSC_Other
                
                NextElement(TT())
                
                CaseValue + "[||]" + TT()\Token
                
              Else
                
              	CaseValue + TT()\Token
              	
              EndIf
              
              TT()\Token = ""
              TT()\Type = #PBSC_Other
              
              NextElement(TT())
              
            ForEver
            
            Debug CaseValue
            
            PreviousElement(TT())
            
            Protected Out.s
            
            If FindString(CaseValue, "[||]") = 0 And FindString(CaseValue, "[&&]") = 0
              
              Out = " ( " + SelectValue(Level) + " == " + CaseValue + " ) "
              
            ElseIf FindString(CaseValue, "[||]") > 0 And FindString(CaseValue, "[&&]") = 0
              
              Out = " ( " + SelectValue(Level) + " == " + ReplaceString(CaseValue, "[||]",  " || " + SelectValue(Level) + " == ") + " ) "
              
            ElseIf FindString(CaseValue, "[||]") = 0 And FindString(CaseValue, "[&&]") > 0
              
              Out = " ( " + SelectValue(Level) + " >= " + ReplaceString(CaseValue, "[&&]",  " && " + SelectValue(Level) + " <= ") + " ) "
              
            ElseIf FindString(CaseValue, "[||]") > 0 And FindString(CaseValue, "[&&]") > 0
              
              ; a - > 1[||]2[||]4[||]50[&&]51[||]52[||]53[||]70[&&]90
              
              ; } else if (v_a==1 || v_a==2 || v_a==4 || (v_a>=50 && v_a<=51) || v_a==52 || v_a==53 || (v_a>=70 && v_a<=90)) {
              
              Protected CC
              Protected CV.s
              
              CV = CaseValue
              
              For CC = 1 To CountString(CaseValue, "[&&]")
                
                Protected T1.s = StringField(CaseValue, CC + 0, "[&&]")
                Protected T2.s = StringField(CaseValue, CC + 1, "[&&]")
                
                T1 = StringField(T1, CountString(T1, "[||]") + 1, "[||]")
                
                If FindString(T2, "[||]")
                  T2 = StringField(T2, CountString(T2, "[||]") - 2, "[||]")
                EndIf
                
                CV = ReplaceString(CV, T1  + "[&&]" + T2, "(" + SelectValue(Level) + " >= " + T1 + " && " + SelectValue(Level) + " <= " + T2 + ")")
                
              Next
              
              CaseValue = ReplaceString(CV, "[||](", " || (")
              
              Out = " ( " + SelectValue(Level) + " == " + ReplaceString(CaseValue, "[||]",  " || " + SelectValue(Level) + " == ") + " ) "
              
            Else
              
              Out = "???"
              
            EndIf
            
            CaseValue = ""
            
            If FirstIf
              TT()\Token = " if " + Out + " { "
            Else
              TT()\Token = " else if " + Out + " {"
            EndIf
            
            FirstIf = #False
            
          Case "default"
            
            TT()\Token = " } else {"
            TT()\Type = #PBSC_Other
            FirstIf = #False
            
          Case "endselect"
            
            TT()\Token = "}"
            TT()\Type = #PBSC_Other
            
            Level - 1
            
            If Level = -1
            	Dim SelectValue.s(100)
            EndIf
            
        EndSelect
        
    EndSelect
    
  Next
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessSelectCase_old()
  
  AddLog("Processing: Select - Case - Default - EndSelect...")

  ; ----
  ; Select - Case - Default - EndSelect
  ; ----
  
  Protected ReturnValue = #True
  
  Protected inSelect, inCase
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case "select"
            
            inSelect = #True
            
          Case "case", "default", "endselect"
            
            If Not inSelect
              
              InsertElement(Token())
              Token()\Type = #PBSC_Identifier
              Token()\Token = "break;"
              NextElement(Token())
              
              InsertElement(Token())
              Token()\Type = #PBSC_NewLine
              Token()\Token = #LF$
              NextElement(Token())
              
            EndIf
            
            inSelect = #False
            
        EndSelect
        
    EndSelect
    
  Next  
  
  ; //
  
  inSelect = #False
  
  ; //
  
  Define CaseFrom 
  Define CaseTo
  Define CaseCounter
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case "select"
            inSelect = #True
            Token()\Token = "switch ("
            
          Case "case"
            Token()\Token = "case"
            inCase = #True
            
          Case "default"
            Token()\Token = "default"
            inCase = #True
            
          Case "endselect"
            Token()\Token = "}"
            
          Case "to"
          	
          	If inSelect
          		
          		PreviousElement(Token())
          		
          		If Token()\Token <> Str(Val(Token()\Token))
          			AddLog("Cannot solve 'Case [var] To [var]'. Sorry!", #True)
          			ReturnValue = #False
          			Break
          		EndIf
          		
          		CaseFrom = Val(Token()\Token)
          		Token()\Token = ""
          		NextElement(Token())
          		Token()\Token = ""
          		NextElement(Token())
          		
          		If Token()\Token <> Str(Val(Token()\Token))
          			AddLog("Cannot solve 'Case [var] To [var]'. Sorry!", #True)
          			ReturnValue = #False
          			Break
          		EndIf
          		
          		CaseTo = Val(Token()\Token)
          		Token()\Token = ""
          		
          		For CaseCounter = CaseFrom To CaseTo
          			If Token()\Token = ""
          				Token()\Token + Str(CaseCounter)
          			Else
          				Token()\Token + " : case " + Str(CaseCounter)
          			EndIf
          		Next
          		
          	EndIf
          	
        EndSelect
        
      Case #PBSC_NewLine, #PBSC_Comment
        
        If inSelect = #True
          
          PreviousElement(Token())
          Token()\Token + ") {"
          NextElement(token())
          inSelect = #False
          
        EndIf
        
        If inCase = #True
          
          PreviousElement(Token())
          Token()\Token + ":"
          NextElement(token())
          inCase = #False
          
        EndIf
        
      Case #PBSC_Other
      	
      	If Token()\Token = ","
      		If inCase = #True
      			Token()\Token = " : case "
      		EndIf
      	EndIf
        
    EndSelect
    
  Next  
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessDeclarationsJS(List TT.Token())
  
  AddLog("Processing: Declarations...")
  
  ; "global", "protected", "define" -> "var"
  
  Protected ReturnValue = #True
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
      	Select LCase(TT()\Token)
      	  Case "global", "protected", "define"
        	  
        	  If CheckNextToken(TT(), ".")
        	    
        	    Protected v1.s=TT()\Token
        	    
        	    NextElement(TT())
        	    NextElement(TT())
        	    
        	    Protected v2.s=TT()\Token
        	    
        	    AddLog("")
        	    AddLog("Line " + Str(TT()\Line) + ": Typed declaration keywords are not allowed (yet)")
        	    AddLog("Not allowed: " + v1 + "." + v2 + " myVar")
        	    AddLog("Better use:  " + v1 + " myVar." + v2)
        	    AddLog("", #True)
        	    
        	    ReturnValue = #False
        	    
        	    Break 
        	    
        	  EndIf
        	  
        	  TT()\Token = "var"
        	  
        EndSelect
        
    EndSelect
  Next  
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessDeclarationsPHP(List TT.Token())
  
  AddLog("Processing: Declarations...")
  
  ; "global", "protected", "define" -> "var"
  
  Protected ReturnValue = #True
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
      	Select LCase(TT()\Token)
      			
      	  Case "global", "protected", "define"
        	  
        	  If CheckNextToken(TT(), ".")
        	    
        	    Protected v1.s=TT()\Token
        	    
        	    NextElement(TT())
        	    NextElement(TT())
        	    
        	    Protected v2.s=TT()\Token
        	    
        	    AddLog("")
        	    AddLog("Line " + Str(TT()\Line) + ": Typed declaration keywords are not allowed (yet)")
        	    AddLog("Not allowed: " + v1 + "." + v2 + " myVar")
        	    AddLog("Better use:  " + v1 + " myVar." + v2)
        	    AddLog("", #True)
        	    
        	    ReturnValue = #False
        	    
        	    Break 
        	    
        	  EndIf
        	  
        	  TT()\Token = "$"
        	  
        EndSelect
        
    EndSelect
  Next  
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure ProcessWithEndWidth()
	
	Protected inWith
	Protected WithElement.s
	
	ForEach Token()
		Select Token()\Type
			Case #PBSC_Identifier
				Select LCase(Token()\Token)
					Case "with"
						
						inWith = #True
						
						WithElement = ""
						
						Token()\Token = ""
						
						NextElement(Token())
						
						Repeat
						  
						  If Token()\Type = #PBSC_Newline : Break : EndIf
						  If Token()\Type = #PBSC_Comment : Break : EndIf
						  
						  WithElement + Token()\Token
						  
						  Token()\Token = ""
						  
						  NextElement(Token())
						  
						ForEver
						
					Case "endwith"
						
						inWith = #False
						Token()\Token = ""
						
				EndSelect
				
			Case #PBSC_Other
				
				If Token()\Token = "\"
					
					PreviousElement(Token())
					If Token()\Type = #PBSC_NewLine
						NextElement(Token())
						InsertElement(Token())
						Token()\Token = WithElement
					EndIf
					NextElement(Token())
					
				EndIf
				
		EndSelect
	Next
	
EndProcedure

Procedure Process_OperatorsShorthands_New()
	
 ; Operators shorthands
 
 	Protected isEqual
  Protected isShort
  Protected Marker
  Protected Operator.s
  
  NewList ShortHandVar.Token()
  
  ForEach Token()
  	
  	Select Token()\Type
  			
  		Case #PBSC_Other
  			
  			Select Token()\Token 
  					
  				Case "=" 
  					isEqual = #True
  					
  				Case "+", "-", "*", "/" ; , "!", "|", "&", "~"
  					
  					If Not (isEqual)
  						
  						Operator = Token()\Token
  						
  						Marker = ListIndex(Token())
  						
  						; Zum Zeilenanfang springen
  						
  						Repeat
  							If Token()\Type = #PBSC_NewLine
  								Break
  							EndIf
  							PreviousElement(Token())
  						ForEver
  						
  						NextElement(Token())
  						
  						; Variable-Daten sammeln
  						Repeat
  							
  							If ListIndex(Token()) = Marker
  								Break
  							EndIf
  							
  							AddElement(ShortHandVar())
  							
  							ShortHandVar()\Line  = Token()\Line
  							ShortHandVar()\Token = Token()\Token
  							ShortHandVar()\Type  = Token()\Type
  							ShortHandVar()\TypeX = Token()\TypeX
  							
  							NextElement(Token())
  							
  						ForEver
  						
  						Token()\Token = "="
  						
  						NextElement(Token())
  						
  						InsertElement(Token())
  						
  						Token()\Line  = ShortHandVar()\Line
  						Token()\Token = "("
  						Token()\Type  = ShortHandVar()\Type
  						Token()\TypeX = ShortHandVar()\TypeX
  						
  						NextElement(Token())
  						
  						ForEach ShortHandVar()
  							
  							InsertElement(Token())
  							
  							Token()\Line  = ShortHandVar()\Line
  							Token()\Token = ShortHandVar()\Token
  							Token()\Type  = ShortHandVar()\Type
  							Token()\TypeX = ShortHandVar()\TypeX
  						
  							NextElement(Token())
  							
  						Next
  						
							InsertElement(Token())
							Token()\Line  = ShortHandVar()\Line
							Token()\Token = Operator
							Token()\Type  = #PBSC_Other
							Token()\TypeX = #PBSC_Operator
  							
							NextElement(Token())
							NextElement(Token())
							
							InsertElement(Token())
							Token()\Line  = ShortHandVar()\Line
							Token()\Token = ")"
							Token()\Type  = #PBSC_Other
							Token()\TypeX = #PBSC_Operator
							
  						ClearList(ShortHandVar())
  						
  					EndIf	
  					
        EndSelect
        
      Case #PBSC_NewLine
      	
      	isEqual = #False
      	
    EndSelect
    
  Next
  
EndProcedure

Procedure Process_OperatorsShorthands()
	
 ; Operators shorthands
  
  Protected isEqual
  Protected isShort
  Protected Marker
  
  ForEach Token()
  	
  	Select Token()\Type
  			
      Case #PBSC_NewLine
      	isEqual = #False
      	
      Default
      	
  			Select Token()\Token 
  					
          Case "=", ">", "<", "<>", "><"
          	isEqual = #True

          Case "+", "-", "*", "/", "!", "|", "&", "~"
          	
          	isShort = #False
          	
          	If isEqual = #False
          		
          		Marker = ListIndex(Token())
          		
          		isShort=#True
          		
          		Repeat
          			
          			If Token()\Type = #PBSC_Newline : Break : EndIf
          			If Token()\Type = #PBSC_Comment : Break : EndIf
          			
          			Select Token()\Token 
          				Case ",", "(", ")", "."
          					isShort = #False
          					Break
          			EndSelect
          			
          			If FindString(Token()\Token, Chr(34))
          				isShort = #False
          				Break
          			EndIf
          			
          			If FindString(Token()\Token, "=")
          				isShort = #False
          				Break
          			EndIf
          			
          			If LCase(Token()\Token) = "case"
          				isShort = #False
          				Break
          			EndIf
          			
          			If LCase(Token()\Token) = "procedurereturn"
          				isShort = #False
          				Break
          			EndIf
          			
          			If PreviousElement(Token()) = 0
          				Break
          			EndIf
          			
          		ForEver
          		
          		SelectElement(Token(), Marker)
          		
          		If isShort								
          			Token()\Token + "="
          		EndIf
          		
          	EndIf
          	
        EndSelect
        
    EndSelect
    
  Next
  
EndProcedure

; ****************************************************

Procedure.s Convert(FileContent.s)
  
  AddLog("Converting...")
  
  ; ---
  
  ; get a list of all files
  
  ClearList(FileToCopy())
  ListFiles(SourceFolder)
  
 	; ---
  
  
  
  ClearList(Token())
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  FileContent = ReplaceString(FileContent, #CRLF$, #LF$) 
  
  Lexer\SetFileString(FileContent)
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line  = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected Output.s
  
  ; #PBSC_NewLine = #LF$
  
  ; ----
  
  ForEach Token()
    If Token()\Type = #PBSC_Other And Token()\Token = ":"
      Token()\Token = #LF$
      Token()\Type = #PBSC_NewLine
    EndIf
  Next
  
  Define isLineContinuation
  
  ForEach Token()
    If Token()\Type = #PBSC_NewLine
      
      PreviousElement(Token())
      
      ; A split line has to end with one of the following operator: 
      ; plus (+), comma (,), or (|), And, Or, Xor.
      
      Select LCase(Token()\Token)
        Case ",", "+", "|", "and", "or", "xor"
          isLineContinuation = #True
        Default
          isLineContinuation = #False
      EndSelect
      
      NextElement(Token())
      
      If isLineContinuation
        DeleteElement(Token())
      EndIf
      
    EndIf
  Next
  
  ForEach Token()
  	Select Token()\Type
  		Case #PBSC_Identifier
  			If Left(Token()\Token, 1) = "#"
  				Token()\Type = #PBSC_Constant
  			EndIf
  	EndSelect
  Next
  
  ;(
  ; #LF, #CRLF$, #False, #True, #Nul, #Null
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Constant
        Select LCase(Token()\Token)
        		
        	Case "#ack$"         : Token()\Token = "cString.Chr(6)"
        	Case "#bel$"         : Token()\Token = "cString.Chr(7)"
        	Case "#bs$"          : Token()\Token = "cString.Chr(8)"
        	Case "#can$"         : Token()\Token = "cString.Chr(24)"
        	Case "#cr$"          : Token()\Token = "cString.Chr(13)"
        	Case "#crlf$"        : Token()\Token = "cString.Chr(13) + cString.Chr(10)"
        	Case "#dc1$"         : Token()\Token = "cString.Chr(17)"
        	Case "#dc2$"         : Token()\Token = "cString.Chr(18)"
        	Case "#dc3$"         : Token()\Token = "cString.Chr(19)"
        	Case "#dc4$"         : Token()\Token = "cString.Chr(20)"
        	Case "#del$"         : Token()\Token = "cString.Chr(127)"
        	Case "#dle$"         : Token()\Token = "cString.Chr(16)"
        	Case "#doublequote$" : Token()\Token = "cString.Chr(34)"
        	Case "#dquote$"      : Token()\Token = "cString.Chr(34)"
        	Case "#em$"          : Token()\Token = "cString.Chr(25)"
        	Case "#enq$"         : Token()\Token = "cString.Chr(5)"
        	Case "#eot$"         : Token()\Token = "cString.Chr(4)"
        	Case "#esc$"         : Token()\Token = "cString.Chr(27)"
        	Case "#etb$"         : Token()\Token = "cString.Chr(23)"
        	Case "#etx$"         : Token()\Token = "cString.Chr(3)"
        	Case "#false"        : Token()\Token = "false"
        	Case "#ff$"          : Token()\Token = "cString.Chr(12)"
        	Case "#fs$"          : Token()\Token = "cString.Chr(28)"
        	Case "#gs$"          : Token()\Token = "cString.Chr(29)"
        	Case "#ht$"          : Token()\Token = "cString.Chr(9)"
        	Case "#lf$"          : Token()\Token = "cString.Chr(10)"
        	Case "#lfcr$"        : Token()\Token = "cString.Chr(10) + cString.Chr(13)"
        	Case "#nak$"         : Token()\Token = "cString.Chr(21)"
        	Case "#nul", "#null" : Token()\Token = "null"
        	Case "#nul$"         : Token()\Token = "null"
        	Case "#null$"        : Token()\Token = "null"
        	Case "#rs$"          : Token()\Token = "cString.Chr(30)"
        	Case "#si$"          : Token()\Token = "cString.Chr(15)"
        	Case "#so$"          : Token()\Token = "cString.Chr(14)"
        	Case "#soh$"         : Token()\Token = "cString.Chr(1)"
        	Case "#stx$"         : Token()\Token = "cString.Chr(2)"
        	Case "#sub$"         : Token()\Token = "cString.Chr(26)"
        	Case "#tab"          : Token()\Token = "9"
        	Case "#tab$"         : Token()\Token = "cString.Chr(9)"
        	Case "#true"         : Token()\Token = "true"
        	Case "#us$"          : Token()\Token = "cString.Chr(31)"
        	Case "#vt$"          : Token()\Token = "cString.Chr(11)"
        	Case "#pi"           : Token()\Token = "Math.PI"
            
        EndSelect
    EndSelect
  Next
  
  ;)
  
  ; Remove Pointer-Asterisk und String-'$'
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        If Left(Token()\Token, 1) = "*"
          Token()\Token = Mid(Token()\Token, 2)
        EndIf
        If Right(Token()\Token, 1) = "$"
          Token()\Token = Left(Token()\Token, Len(Token()\Token) - 1)
        EndIf
    EndSelect
  Next
  
  ; ---------------
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        If LCase(Token()\Token) = "calldebugger"
          Token()\Token = "debugger"
        EndIf
    EndSelect
  Next
  
  ;(
  ; ---------------
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
      	Select LCase(Token()\Token) 
      		Case "usecordova"
      			IsInUse\Cordova = #True
      			Token()\Token = ""
      			NextElement(Token()) ; (
      			Token()\Token = ""
      			NextElement(Token()) ; )
      			Token()\Token = ""
      		Case "usemobile"
      			IsInUse\TouchSwipe = #True
      			IsInUse\Mobile = #True
      			Token()\Token = ""
      			NextElement(Token()) ; (
      			Token()\Token = ""
      			NextElement(Token()) ; )
      			Token()\Token = ""
      		Case "usefontawesome"
      			IsInUse\FontAwesome = #True
      			Token()\Token = ""
      			NextElement(Token()) ; (
      			Token()\Token = ""
      			NextElement(Token()) ; )
      			Token()\Token = ""
      		Case "usebootstrap"
      			IsInUse\Bootstrap = #True
      			Token()\Token = ""
      			NextElement(Token()) ; (
      			Token()\Token = ""
      			NextElement(Token()) ; )
      			Token()\Token = ""
      		Case "usepdf"
          	IsInUse\PDF = #True
      			Token()\Token = ""
      			NextElement(Token()) ; (
      			Token()\Token = ""
      			NextElement(Token()) ; )
      			Token()\Token = ""
      		Case "newpdf"
      			IsInUse\PDF = #True
      		Case "p2w_savetext", "p2w_savecsv", "p2w_savecanvas", "savexml"
      			IsInUse\FileSaver = #True
      		Case "saveimage"
      			IsInUse\SaveImage = #True
          Case "listicongadget", "gridgadget"
          	IsInUse\w2ui = #True
          Case "dategadget"
          	IsInUse\DateTimePicker = #True
          Case "treegadget"
            IsInUse\TreeGadget = #True
          Case "trackbargadget"
            IsInUse\TrackBarGadget = #True
          Case "tinymcegadget"
            IsInUse\TinyMce = #True
          Case "googlemapsgeocoder", "googlemapsgeocoder2", "googlemapgadget"
            IsInUse\GoogleMap = #True
          Case "googlechartgadget"
          	IsInUse\GoogleChart = #True
          Case "p2w_blockwindow", "p2w_blockgadget"
          	IsInUse\BlockUI = #True
					Case "p2w_setcookie", "p2w_getcookie", "p2w_removecookie"
          	IsInUse\JQueryCookie = #True
					Case "splittergadget"
						IsInUse\Splitter = #True
					Case "colorrequester"
						IsInUse\ColorRequester = #True
					Case "addkeyboardshortcut", "removekeyboardshortcut", "bindkeyboardshortcut"
						IsInUse\KeyboardShortcut = #True
					Case "openwebsocket"
						IsInUse\Websocket = #True
					Default
						
						If Left( LCase(Token()\Token), Len("p2w_localstorage")) = "p2w_localstorage"
							IsInUse\Preference = #True
						EndIf

				EndSelect
				
			Case #PBSC_Constant ; [!!!]
				
				Select LCase(Token()\Token)
						
					Case "#pb_eventtype_swipe"
						IsInUse\TouchSwipe = #True
						
					Case "#pb_checkbox_threestate"
						IsInUse\TriState = #True
						
				EndSelect
				
		EndSelect
		
	Next
  ;)
  ; ---------------
	
	
  ; needed userlibraries
  
  Protected References.s
  
  ForEach FileToCopy()
		If FindString(FileToCopy(), "system/references.txt", 1, #PB_String_NoCase)
			References = LoadTextfile(FileToCopy())
			FileToCopy() = ""
			Break
		EndIf
 	Next
 	
 	Protected LineCounter, CountLines
 	Protected ReferenceLine.s
 	Protected Keyword.s, LibPath.s
 	Protected LibPathIsNeeded
 	
 	If References <> ""
 		
 		CountLines = CountString(References, #CRLF$)
 		
 		For LineCounter = 1 To CountLines 
 			
 			ReferenceLine = StringField(References, LineCounter, #CRLF$)
 			
 			If ReferenceLine <> ""
 				
 				Keyword       = StringField(ReferenceLine, 1, ";")
 				LibPath       = StringField(ReferenceLine, 2, ";")
 				
 				LibPathIsNeeded = #False
 				
 				ForEach Token()
 					Select Token()\Type
 						Case #PBSC_Identifier
 							Select LCase(Token()\Token) 
 								Case LCase(Keyword)
 									LibPathIsNeeded = #True
 									Break
 							EndSelect
 					EndSelect
 				Next
 				
 				If LibPathIsNeeded 
 					AddLog("Needed: " + LibPath)
 				Else
 					AddLog("Unneeded: " + LibPath)
 					ForEach FileToCopy()
 						If FindString(FileToCopy(), LibPath, 1, #PB_String_NoCase)
 							FileToCopy() = ""
 						EndIf
 					Next
 				EndIf
 				
 			EndIf
 			
 		Next
 	EndIf
 	

	
  
  ; remove Declare 
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
        	Case "declare"
        		
        		Token()\Token = ""
        		Token()\Type = #PBSC_Other
        		
        		Repeat
        		  
        		  NextElement(Token())
        		  
        		  If Token()\Type = #PBSC_Newline : Break : EndIf
        		  If Token()\Type = #PBSC_Comment : Break : EndIf
        		  
        		  Token()\Token = ""
        		  Token()\Type = #PBSC_Other
        		  
        		ForEver
        		
        EndSelect
    EndSelect
  Next
  
  ; remove Bool() 
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
        	Case "bool"
        		Token()\Token = ""
        		Token()\Type = #PBSC_Other
        EndSelect
    EndSelect
  Next
  
  Process_OperatorsShorthands()
  
  ; escape Backspace 
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_String
        Token()\Token = ReplaceString(Token()\Token, "\", "\\")
    EndSelect
  Next
  
  ; Enable-/DisableDebugger
  
  ForEach Token()
  	Select Token()\Type
  		Case #PBSC_Identifier
  			Select LCase(Token()\Token)
  				Case "enabledebugger", "disabledebugger"
  					Token()\Token = "cDebugger." + Token()\Token + "()"
  			EndSelect
  	EndSelect
  Next
  
  ; convert $[number] 
  
  ForEach Token()
  	Select Token()\Type
  		Case #PBSC_Number
  		  If Left(Token()\Token, 1) = "$"
  		    Token()\Token = "cString.Val(" + Chr(34) + Token()\Token + Chr(34) + ")"
        EndIf 
  	EndSelect
  Next
  
  RemoveTypeDeclarations(Token())
  
  ProcessVariables()
  
  ProcessIdentifier2Classnames(Token())
  
  If TargetSource = #Target_PHP
  	If ProcessDeclarationsPHP(Token()) = #False
  		ProcedureReturn ""
  	EndIf
  Else
  	If ProcessDeclarationsJS(Token()) = #False
  		ProcedureReturn ""
  	EndIf
  EndIf

  ProcessStatics()
  
  ProcessEnumerationsAndConstants()
  
  ; ProcessWithEndWidth()
  
  If ProcessStructures(Token()) = #False : ProcedureReturn "" : EndIf
 
  GetProcedureInformations()
  
  ProcessArrays(Token())
  
  ProcessLinkedLists()
  
  ProcessMaps()
  
  If ProcessSelectCase(Token()) = #False : ProcedureReturn "" : EndIf
  
  Protected inParenthesis
  Protected inDot
  Protected isEqualSign
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_NewLine
        inDot = #False
        inParenthesis = #False
        
      Case #PBSC_Other
        
        Select Token()\Token
          Case "."
            inDot = #True
          Case "("
            inParenthesis = #True
          Case ")"
            inParenthesis = #False
        EndSelect
        
      Case #PBSC_Identifier 
        
        Select LCase(Token()\Token)
            
        	Case "w"
;         		PreviousElement(Token())
;         		Token()\Token = ""
;         		NextElement(Token())
;         		Token()\Token = ""
        		
          Case "b", "a", "c", "u", "l", "i", "f", "q", "d", "s"
            
          Default
            
            If inDot
              
              PreviousElement(Token())
              Token()\Token = ""
              Token()\Type = #PBSC_Other
              NextElement(Token())
              
              If inParenthesis
                Token()\Token = ""
              Else
              	
              	NextElement(Token())
								isEqualSign = Bool(Token()\Token = "=")
              	PreviousElement(Token())
              	
              	If Not isEqualSign
              		
              		If FindString(Token()\Token,"firstelement",    1, #PB_String_NoCase) Or
              		   FindString(Token()\Token,"lastelement",     1, #PB_String_NoCase) Or
              		   FindString(Token()\Token,"nextelement",     1, #PB_String_NoCase) Or
              		   FindString(Token()\Token,"previouselement", 1, #PB_String_NoCase)
              			
              			Token()\Token = " = " + Token()\Token
              			
              		Else
              			
              			Token()\Token = " = jQuery.extend(true, {}, " + Token()\Token + ")"
              			
              		EndIf
              		
              	Else
              		
              		Token()\Token = ""
              		
                EndIf
                
              EndIf
              
              inDot = #False
              
            EndIf
            
        EndSelect
        
    EndSelect
    
  Next	
  
  
  ;ProcessProcedure_TS()
  ProcessProcedure(Token())
  
  ProcessDebug()
  ProcessCallbacks(Token())
  ProcessComments(Token())
  
  ; ----  
  
  ForEach Token()
  	
    Select Token()\Type
    		
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case "true"     : Token()\Token = "true"
          Case "false"    : Token()\Token = "false"
          Case "or"       : Token()\Token = "||"
          Case "and"      : Token()\Token = "&&"
          Case "not"      : Token()\Token = "!"
          Case "break"    : Token()\Token = "break"
          Case "continue" : Token()\Token = "continue"
            
          Default
            
        EndSelect
        
      Case #PBSC_Other
        
        Select LCase(Token()\Token)
        		
        	Case "&" : Token()\Token = "&&"
        		
					Case "\" 
						Token()\Token = "."
						
        EndSelect
        
      Default
       
    EndSelect
    
  Next
  
  ; //
  
  ForEach Token()
  	Select Token()\Type
  			
  		Case #PBSC_Identifier
  			
  			If Left(LCase(Token()\Token), Len("ccipher.")) = "ccipher."
  				IsInUse\CryptoJs = #True
  				
  			ElseIf Left(LCase(Token()\Token), Len("cpreference.")) = "cpreference."
  				IsInUse\Preference = #True
  				
  			ElseIf Left(LCase(Token()\Token), Len("csprite.")) = "csprite." Or 
  			       Left(LCase(Token()\Token), Len("c2ddrawing.")) = "c2ddrawing."
  				IsInUse\Pixi = #True
  				
  			ElseIf Left(LCase(Token()\Token), Len("cmenu.")) = "cmenu."
  				IsInUse\Menu = #True
  				
  			ElseIf Left(LCase(Token()\Token), Len("cdatabase.")) = "cdatabase."
  				IsInUse\Database   = #True
  				IsInUse\Preference = #True
  				
  			ElseIf Left(LCase(Token()\Token), Len("cxml.")) = "cxml."
  				IsInUse\XML = #True
  			EndIf
  			
  	EndSelect
  Next  
  
  ProcessForNext(Token())
  ProcessWhileWend(Token())
  ProcessRepeatUntil(Token())
  ProcessIf(Token())

  Output = ""
  ForEach Token()
    Output + Token()\Token + " "
  Next
  
  ProcedureReturn Output
  
EndProcedure

Procedure.s PreProcess_IncludeFile(FileContent.s)
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  FileContent = ReplaceString(FileContent, #CRLF$, #LF$)
  
  Lexer\SetFileString(FileContent)
  
  ClearList(Token())
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line  = Lexer\GetCurrentLineNb() 
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Structure sIncludes
    Code.s
    Line.i
  EndStructure
  
  NewList Include.sIncludes()
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
          Case "xincludefile", "includefile"
            
            If CheckNextToken(Token(), "(")
              NextElement(Token()) ; Klammer auf?
            EndIf
            
            NextElement(Token()) ; Dateiname
            
            AddElement(Include())
            
            Include()\Code = RemoveString(Token()\Token, Chr(34))
            
            If FileSize(Include()\Code) > 0
	            Include()\Code = LoadTextFile(Include()\Code)
            Else
  	          Include()\Code = LoadTextFile(GetPathPart(SourceFile) + Include()\Code)
            EndIf
            
            Include()\Code = ReplaceString(Include()\Code, #CRLF$, #LF$)
            
            Include()\Line = Token()\Line
            
        EndSelect
    EndSelect
  Next
  
  Protected Output.s
  Protected CountLines, LineCounter
  
  NewList CodeLine.s()
  
  CountLines = CountString(FileContent, #LF$)
  For LineCounter = 1 To CountLines
    AddElement(CodeLine())
    CodeLine() = StringField(FileContent, LineCounter, #LF$)
  Next
  
  ForEach Include()
    SelectElement(CodeLine(), Include()\Line - 1)
    CodeLine() = Include()\Code
  Next
  
  FreeList(Include())
  
  ForEach CodeLine()
    If CodeLine() <> ""
      Output + CodeLine() + #LF$
    EndIf
  Next
  
  FreeList(CodeLine())
  
  ProcedureReturn Output
  
EndProcedure

Procedure.s PreProcess_IgnoreParts(FileContent.s)
	
  Protected CountLines, LineCounter
  Protected Line.s 
  Protected Code.s
  Protected BeginComment
  
  CountLines = CountString(FileContent, #LF$)
  
  For LineCounter = 1 To CountLines
    
    Line = Trim(StringField(FileContent, LineCounter, #LF$))
    
    If StartsWith(TrimChars(LCase(Line)), LCase("P2W_Converter_Begin_Comment"))
      Line = ""
      BeginComment = #True
      
    ElseIf StartsWith(TrimChars(LCase(Line)), LCase("P2W_Converter_End_Comment"))
      Line = ""
      BeginComment = #False
      
    EndIf
    
    If Line <> ""
      If Not BeginComment
        Code + Line + #LF$
      EndIf
    EndIf
    
  Next  
  
  ProcedureReturn Code
  
EndProcedure

Procedure.s PreProcess_RemoveMacros(FileContent.s)
	
  Protected CountLines, LineCounter
  Protected Line.s 
  Protected Code.s
  Protected BeginComment
  
  BeginComment = #False
  
  CountLines = CountString(FileContent, #LF$)
  
  For LineCounter = 1 To CountLines
    
    Line = Trim(StringField(FileContent, LineCounter, #LF$))
    
    If StartsWith(TrimChars(LCase(Line)), LCase("Macro"))
      Line = ""
      BeginComment = #True
      
    ElseIf StartsWith(TrimChars(LCase(Line)), LCase("EndMacro"))
      Line = ""
      BeginComment = #False
      
    EndIf
    
    If Line <> ""
      If Not BeginComment
        Code + Line + #LF$
      EndIf
    EndIf
    
  Next
	
  ProcedureReturn Code
  
EndProcedure

Procedure.s PreProcess_InlineJS(FileContent.s)
	
	Protected Line.s
	Protected Code.s
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows 
  	
  	Protected Seps.s = #LF$
  	Protected Token  = strtok(@FileContent, @Seps) 
  	
  	While Token
  		
  		Line = LTrimChars(PeekS(Token))
  		
  		If StartsWith(Line, "!")
  			Code + "EnableJS" + #LF$ + ";" + Mid(Line, 2) + #LF$ + "DisableJS" + #LF$
  		Else
  			Code + Line + #LF$
  		EndIf			
  		
  		Token = strtok(#Null, @Seps)
  		
  	Wend  
  	
  CompilerElse
  	
	  Protected LineCounter
  	Protected CountLines = CountString(FileContent, #LF$)
  	
  	For LineCounter = 1 To CountLines
  		Line = LTrimChars(StringField(FileContent, LineCounter, #LF$))
  		If StartsWith(Line, "!")
  			Code + "EnableJS" + #LF$ + ";" + Mid(Line, 2) + #LF$ + "DisableJS" + #LF$
  		Else
  			Code + Line + #LF$
  		EndIf
  	Next
  	
  CompilerEndIf
	
  ProcedureReturn Code
  
EndProcedure

Procedure.s PreProcess_DataSection(FileContent.s)
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Lexer\SetFileString(ReplaceString(FileContent, #CRLF$, #LF$) )
  
  ClearList(Token())
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line  = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected GlobalLabelVar.s
  Protected LabelVars.s
  Protected DataArray.s
  Protected DataCounter
  Protected Output.s
  
  ForEach Token()
  	
  	If Token()\Token = ":" ; Label?
  		
  		PreviousElement(Token())
  		
  		LabelVars + "var P2W_Label_" + Token()\Token + " = " + Str(DataCounter) + #CRLF$
  		
  		If GlobalLabelVar = ""
  			GlobalLabelVar = "cCommonP2W_Global_Label = P2W_Label_" + Token()\Token + #CRLF$
  		EndIf
  		
  		NextElement(Token())
  		
  	EndIf
  	
  	If Token()\Token = "Data"
  		
  		If DataArray = ""
  			DataArray = "cCommonP2W_DataArray=["
  		EndIf
  		
  		Repeat
  			
  			NextElement(Token())
  			
  			If Token()\Type = #PBSC_NewLine
  				DataCounter + 1
	  			DataArray + ","
  				Break
  			EndIf
  			
  			If Token()\Type = #PBSC_Other And Token()\Token = ","
  				DataCounter + 1
  			EndIf
  			
  			If Token()\Type = #PBSC_Other And Token()\Token = "."
	  			NextElement(Token())
	  			NextElement(Token())
  			EndIf
  			
  			DataArray + Token()\Token
  			
  		ForEver
  		
  	EndIf
  	
    If Token()\Token <> ""
      Output + Token()\Token + " "
    EndIf  	
  	
  Next
  
  If DataArray <> ""
  	
  	If Right(DataArray, 1) = ","
  		DataArray = Left(DataArray, Len(DataArray) - 1)
  	EndIf
  	
  	DataArray + "]"
  	
  	Output = LabelVars + #CRLF$ + GlobalLabelVar + #CRLF$ + DataArray + #CRLF$ + Output
  
  EndIf
  
  ProcedureReturn Output
  
EndProcedure

Procedure.s PreProcess_IncludeBinary(FileContent.s)
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Lexer\SetFileString(ReplaceString(FileContent, #CRLF$, #LF$) )
  
  ClearList(Token())
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  ForEach Token()
  	If LCase(Token()\Token) = "includebinary"
    	NextElement(Token())
    	If Token()\Type = #PBSC_String
      	AddElement(IncludeBinaryFile())
      	IncludeBinaryFile() = ReplaceString( Token()\Token , Chr(34) , "")
    	EndIf
    EndIf
  Next
  ClearList(Token())
	
  Protected CountLines, LineCounter
  Protected Line.s 
  Protected Code.s
  Protected BeginDataSection
  
  CountLines = CountString(FileContent, #LF$)
  
  For LineCounter = 1 To CountLines
    
    Line = Trim(StringField(FileContent, LineCounter, #LF$))
    
    If StartsWith(TrimChars(LCase(Line)), "datasection")
      
      Line = ""
      BeginDataSection = #True
      
    ElseIf StartsWith(TrimChars(LCase(Line)), "enddatasection")
      
      Line = ""
      BeginDataSection = #False
      
    ElseIf StartsWith(TrimChars(LCase(Line)), "includebinary")
      
      Line = ""
      
    EndIf
    
    If Line <> ""
      
      If Not BeginDataSection
        Code + Line + #LF$
      EndIf
      
    EndIf
    
  Next
  
  ForEach IncludeBinaryFile()
  	Debug "IncludeBinaryFile: " + IncludeBinaryFile()
  Next
  
  ProcedureReturn Code  
  
EndProcedure

Procedure.s PreProcess_MoveGlobalsOutOfProcedures(FileContent.s)
	
  Protected CountLines
  Protected LineCounter
  Protected Output.s
  NewList CodeLine.s()
  
  FileContent = ReplaceString(FileContent, #CRLF$, #LF$)
  
  CountLines = CountString(FileContent, #LF$)
  
  For LineCounter = 1 To CountLines
  	AddElement(CodeLine())
    CodeLine() = Trim(StringField(FileContent, LineCounter, #LF$))
  Next
  
  ; #########
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Lexer\SetFileString(FileContent)
  
  ClearList(Token())
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected GlobalBlock.s
  
  Protected isGlobal
  Protected isStructure
  
  ForEach Token()
  	
  	If Token()\Type = #PBSC_Identifier 
  		
  		Select LCase(Token()\Token)
  			Case "global"
  				isGlobal = #True
  			Case "structure"
  				isStructure = #True
  		EndSelect
  		
  		If isGlobal Or isStructure
  			
  			SelectElement(CodeLine(), Token()\Line - 1)
  			GlobalBlock + CodeLine() + #LF$
  			CodeLine() = ""
  			
  		EndIf
  		
  		isGlobal = #False	
  		
  		Select LCase(Token()\Token)
  			Case "endstructure"
  				isStructure = #False
  		EndSelect
  		
  	EndIf
  	
  Next
  
  ClearList(Token())
  
  ; #############
  
  Output + GlobalBlock
  
  ForEach CodeLine()
  	If Trim(CodeLine()) <> ""
  		Output + CodeLine() + #LF$
  	EndIf
  Next
  
  ProcedureReturn Output	
	
EndProcedure

; ------------------

Procedure.s GetRequestSelect(ServerCode.s)
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  ServerCode = ReplaceString(ServerCode, #CRLF$, #LF$)
  
  Lexer\SetFileString(ServerCode)
  
  ClearList(Token())
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected *Old_Element
  Protected isExport
  
  Protected ProcName.s
  
  Protected isString
  
  ForEach Token()
  	
  	
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
          Case "proceduredll", "procedurecdll"
              
          	isString = #False
          	
            NextElement(Token())
            
            If Token()\Token = "."
            	
            	NextElement(Token())
            	
            	If LCase(Token()\Token) = "s"
            		isString = #True
            	EndIf
            	
            	NextElement(Token())
            	
            EndIf
            
            ProcName = Token()\Token
            
            RequestSelect + #TAB$ + "Case " + Chr(34) + LCase(ProcName) + Chr(34) + #CRLF$
            
            If isString
            	RequestSelect + #TAB$ + #TAB$ + "ReturnValue = " + ProcName + "("
            Else
            	RequestSelect + #TAB$ + #TAB$ + "ReturnValue = Str(" + ProcName + "("
            EndIf
            
            NextElement(Token()) ; Klammer auf
            NextElement(Token()) ; Klammer zu?
            
            While Token()\Token <> ")"
              
              RequestSelect + " GetValue(" + Chr(34) + Token()\Token + Chr(34) + "), "
              
              If CheckNextToken(Token(), ".")
                NextElement(Token())
                NextElement(Token())
              EndIf
              
              If CheckNextToken(Token(), ",")
                NextElement(Token())
              EndIf
              
              NextElement(Token())
              
            Wend
            
            If EndsWith(Trim(RequestSelect), ",")
              RequestSelect = Left(Trim(RequestSelect), Len(Trim(RequestSelect)) - 1)
            EndIf
            
            If isString
            	RequestSelect + ")" + #CRLF$
            Else
            	RequestSelect + "))" + #CRLF$
            EndIf
            
        EndSelect
    EndSelect
  Next
  
  If RequestSelect <> ""
    
    RequestSelect = " Select Request " + #CRLF$ +
                    RequestSelect + #CRLF$ +
                    " Default " + #CRLF$ +
                    "   ReturnValue = " + Chr(34) + "unknown request: '" + Chr(34) + " + Request + " + Chr(34) + "'" + Chr(34) + #CRLF$ +
                    " EndSelect"  
    
  EndIf
  
  ; Debug RequestSelect
  
  ProcedureReturn RequestSelect
  
EndProcedure
Procedure.s GetRequestSelect4ASP(ServerCode.s)
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  ServerCode = ReplaceString(ServerCode, #CRLF$, #LF$)
  
  Lexer\SetFileString(ServerCode)
  
  ClearList(Token())
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected *Old_Element
  Protected isExport
  
  Protected ProcName.s
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
          Case "proceduredll", "procedurecdll"
          	
            NextElement(Token())
            
            If Token()\Token = "."
            	
            	NextElement(Token())
            	NextElement(Token())
            	
            EndIf
            
            ProcName = Token()\Token
            
            RequestSelect + #TAB$ + "Case " + Chr(34) + LCase(ProcName) + Chr(34) + #CRLF$
            
           	RequestSelect + #TAB$ + #TAB$ + "Response.Write " + ProcName + " ( "
            
            NextElement(Token()) ; Klammer auf
            NextElement(Token()) ; Klammer zu?
            
            While Token()\Token <> ")"
              
              RequestSelect + " GetValue(" + Chr(34) + Token()\Token + Chr(34) + "), "
              
              If CheckNextToken(Token(), ".")
                NextElement(Token())
                NextElement(Token())
              EndIf
              
              If CheckNextToken(Token(), ",")
                NextElement(Token())
              EndIf
              
              NextElement(Token())
              
            Wend
            
            If EndsWith(Trim(RequestSelect), ",")
              RequestSelect = Left(Trim(RequestSelect), Len(Trim(RequestSelect)) - 1)
            EndIf
            
           	RequestSelect + " ) " + #CRLF$
            
        EndSelect
    EndSelect
  Next
  
  If RequestSelect <> ""
    
  	RequestSelect = "Dim myRequest : myRequest = LCase(Request.QueryString(" + Chr(34) + "request" + Chr(34) + "))" + #CRLF$ +
  	                "" + #CRLF$ +
  	                "Select Case myRequest" + #CRLF$ +
                    RequestSelect + #CRLF$ +
                    "Case Else" + #CRLF$ +
                    " Response.Write " + Chr(34) + "unknown request: '" + Chr(34) + " & myRequest & " + Chr(34) + "'" + Chr(34) + #CRLF$ +
                    "End Select"  
    
  EndIf
  
  ;Debug RequestSelect
  
  ProcedureReturn RequestSelect
  
EndProcedure
Procedure.s GetRequestSelect4ASPX(ServerCode.s)
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  ServerCode = ReplaceString(ServerCode, #CRLF$, #LF$)
  
  Lexer\SetFileString(ServerCode)
  
  ClearList(Token())
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected *Old_Element
  Protected isExport
  
  Protected ProcName.s
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
          Case "proceduredll", "procedurecdll"
          	
            NextElement(Token())
            
            If Token()\Token = "."
            	
            	NextElement(Token())
            	NextElement(Token())
            	
            EndIf
            
            ProcName = Token()\Token
            
            RequestSelect + #TAB$ + "Case " + Chr(34) + LCase(ProcName) + Chr(34) + #CRLF$
            
           	RequestSelect + #TAB$ + #TAB$ + "Response.Write(" + ProcName + "("
            
            NextElement(Token()) ; Klammer auf
            NextElement(Token()) ; Klammer zu?
            
            While Token()\Token <> ")"
              
              RequestSelect + "GetValue(" + Chr(34) + Token()\Token + Chr(34) + "), "
              
              If CheckNextToken(Token(), ".")
                NextElement(Token())
                NextElement(Token())
              EndIf
              
              If CheckNextToken(Token(), ",")
                NextElement(Token())
              EndIf
              
              NextElement(Token())
              
            Wend
            
            If EndsWith(Trim(RequestSelect), ",")
              RequestSelect = Left(Trim(RequestSelect), Len(Trim(RequestSelect)) - 1)
            EndIf
            
           	RequestSelect + "))" + #CRLF$
            
        EndSelect
    EndSelect
  Next
  
  If RequestSelect <> ""
    
  	RequestSelect = "Dim myRequest As String : myRequest = LCase(Request.QueryString(" + Chr(34) + "request" + Chr(34) + "))" + #CRLF$ +
  	                "" + #CRLF$ +
  	                "Select Case myRequest" + #CRLF$ +
                    RequestSelect + #CRLF$ +
                    "Case Else" + #CRLF$ +
                    " Response.Write(" + Chr(34) + "unknown request: '" + Chr(34) + " & myRequest & " + Chr(34) + "'" + Chr(34) + ")" + #CRLF$ +
                    "End Select"  
    
  EndIf
  
  ;Debug RequestSelect
  
  ProcedureReturn RequestSelect
  
EndProcedure

Procedure GetParamsFromServerProcedure(ProcLine.s, *ProcName.String, *Params.String)
	
 Protected Lexer.iPBSC = New_PBSC() 	
  
  Lexer\SetFileString(ProcLine)
  
  ClearList(Token())
  
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  ForEach Token()
  	Select Token()\Type
  		Case #PBSC_Identifier
  			Select LCase(Token()\Token)
  					
  				Case "procedure", "proceduredll", "procedurecdll"
  					
  					If CheckNextToken(Token(), ".")
  						NextElement(Token()) ; Punkt berspringen
  						NextElement(Token()) ; Prozedurtyp berspringen
  					EndIf
  					
   					NextElement(Token())
   					*ProcName\s = Token()\Token
   					
   					NextElement(Token()) ; -> Klammer auf
   					
   					Repeat
   						
   						NextElement(Token())
   						
   						If Token()\Token = "."
	   						NextElement(Token()) ; Punkt berspringen
	   						NextElement(Token()) ; Variablentyp berspringen
   						EndIf
   						
   						If Token()\Token = ","
	   						NextElement(Token()) ; Komma berspringen
   						EndIf
   						
   						If Token()\Token = ")" ; Klammer zu, Affe tot
   							Break
   						EndIf
   						
   						*Params\s + "'" + Token()\Token + "': " + Token()\Token + ", "
   						
   					ForEver
   					
  			EndSelect
  	EndSelect
  Next
  
  If *Params\s <> ""
  	*Params\s = Left(*Params\s, Len(*Params\s) - 2)
  EndIf
  
EndProcedure
Procedure.s PreProcess_ServerCode_CGI(FileContent.s)
	
	TargetSource = #Target_CGI	
	
  Protected ClientCode.s, ServerCode.s
  
  Protected ProcName.String 
  Protected Params.String
  
  NewList CgiBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  Protected CallbackProcedure.s
  
  Protected regex_CGI, regex_PDLL, regex_PCDLL
  
  ; EnableCGI / DisableCGI
  
  regex_CGI = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnableCGI([\s\S]*?)^[\s]*DisableCGI", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_CGI = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureCDLL()
  
	regex_PCDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureCDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PCDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ExamineRegularExpression(regex_CGI, FileContent)
  
  While NextRegularExpressionMatch(regex_CGI)
  	
  	AddElement(CgiBlock())
  	
  	CgiBlock() = RegularExpressionMatchString(regex_CGI)
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		ExamineRegularExpression(regex_PDLL, CgiBlock())
		
		Protected dataType.s = ""
		Protected processData.s = ""		
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			Select Left(RemoveString(ServerProcedure(), "ProcedureDLL"), 2)
				Case ".b"
					dataType = "binary"
					processData = "false"
				Default
					dataType = "text"
					processData = "true"
			EndSelect			
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			ClientProcedure() + "Protected ReturnValue.s" + #LF$
			
			ClientProcedure() + "EnableJs" + #LF$
			
			Params\s = "'p2wrequest':'" + ProcName\s + "'," + Params\s
			
			ClientProcedure() + "; var params = jQuery.param( { " + Params\s + " } ); " + #LF$
			
			ClientProcedure() + "; $.ajax({ " + #LF$
			
			If dataType = "binary"
				ClientProcedure() + "; 	url: '" + CompilerInfo\CgiAddress + "?' + params, " + #LF$
			Else
				ClientProcedure() + "; 	url: '" + CompilerInfo\CgiAddress + "', " + #LF$
			EndIf
			
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If dataType <> "binary"
				If Params\s <> ""
					ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
				EndIf
			EndIf
			
			ClientProcedure() + "; 	dataType: '" + dataType + "', " + #LF$
			ClientProcedure() + "; 	processData: " + processData + ", " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { ReturnValue = result }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { ReturnValue = b + '/' + c } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "ProcedureReturn ReturnValue" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
  			
		FreeRegularExpression(regex_PDLL)
	
		; ----  	
		
		ExamineRegularExpression(regex_PCDLL, CgiBlock())
		
		While NextRegularExpressionMatch(regex_PCDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PCDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			
			Select Left(RemoveString(ClientProcedure(), "ProcedureCDLL"), 2)
				Case ".b"
					dataType = "binary"
					processData = "false"
				Default
					dataType = "text"
					processData = "true"
			EndSelect		
			
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			CallbackProcedure = ProcName\s + "Callback"
			
			ClientProcedure() + "EnableJs" + #LF$
			
			Params\s = "'p2wrequest':'" + ProcName\s + "'," + Params\s
			
			ClientProcedure() + "; var params = jQuery.param( { " + Params\s + " } ); " + #LF$
			
			ClientProcedure() + "; $.ajax({ " + #LF$
			
			If dataType = "binary"
				ClientProcedure() + "; 	url: '" + CompilerInfo\CgiAddress + "?' + params, " + #LF$
			Else
				ClientProcedure() + "; 	url: '" + CompilerInfo\CgiAddress + "', " + #LF$
			EndIf
			
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If dataType <> "binary"
				If Params\s <> ""
					ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
				EndIf
			EndIf
			
			ClientProcedure() + "; 	dataType: '" + dataType + "', " + #LF$
			ClientProcedure() + "; 	processData: " + processData + ", " + #LF$
			ClientProcedure() + "; 	async: true, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			
			If dataType = "binary"
				
				ClientProcedure() + "; success: function(result) { "  + #LF$ + 
				                    ";  if (typeof " + CallbackProcedure + " == 'function') { " + #LF$ +
				                    ";   " + CallbackProcedure + "(true, result, {" + Params\s + "}); " + #LF$ + 
				                    ";  } " + #LF$ +
				                    "; }, " + #LF$
				
			Else
				ClientProcedure() + "; 	success: function(result)  { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(true, result, {" + Params\s + "}) }, " + #LF$
			EndIf			
			
			ClientProcedure() + "; 	error:   function (a, b, c) { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(false, b + '/' + c) } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
  			
		FreeRegularExpression(regex_PCDLL)
		
		; ----  	
		
		ClientProcedures = ""
  	ForEach ClientProcedure()
  		ClientProcedures + ClientProcedure() + #LF$
  	Next
  	
  	ClientCode = ReplaceString(ClientCode, CgiBlock(), ClientProcedures)
  	
		; ----  	
  	
 		ServerCode + RegularExpressionGroup(regex_CGI, 1) + #LF$
  	
  Wend
      
	FreeRegularExpression(regex_CGI)
	
	; ----------------------------------------------------
	
  Protected CgiTempFilename.s
  Protected RequestSelect.s
  
  Protected TemplateCode.s
 	Protected TemplateFilename.s 

  If ServerCode <> ""
  	
  	If CompilerInfo\CgiFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'CgiFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\CgiAddress = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'CgiAddress' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetExePath() + "system\usertemplates\cgi.pb") > 0
  		TemplateFilename = GetExePath() + "system\usertemplates\cgi.pb"
  	ElseIf FileSize(GetExePath() + "system\templates\cgi.pb") > 0
  		TemplateFilename = GetExePath() + "system\templates\cgi.pb"
  	EndIf
  	
  	If TemplateFilename <> ""
  		
  		TemplateCode = LoadTextFile(TemplateFilename)
  		
  		TemplateCode = ReplaceString(TemplateCode, "; ### PB2Web ServerCode ###", ServerCode)
  		
  		RequestSelect = GetRequestSelect(ServerCode)
  		
  		TemplateCode = ReplaceString(TemplateCode, "; ### PB2Web RequestSelect ###", RequestSelect)
  		
  		CgiTempFilename = GetPathPart(SourceFile) + "tempcgi.pb"
  		
  		SaveTextFile(TemplateCode, CgiTempFilename)
  		
  		Protected FileDeleteCounter = 0
  		
  		Repeat
  			
  			If FileSize(CompilerInfo\CgiFilename) = -1
  				Break
  			EndIf
  			
  			Addlog("Trying to delete '" + CompilerInfo\CgiFilename + "'. " + Str(FileDeleteCounter + 1) + " try...")
  			
  			DeleteFile(CompilerInfo\CgiFilename)
  			
  			FileDeleteCounter + 1
  			
  			If FileDeleteCounter > 9
  				Break
  			EndIf
  			
  			Delay(500)
  			
  		ForEver
  		
  		Protected Compiler = RunProgram(Chr(34) + PbCompiler               + Chr(34) + " " + 
  		                                Chr(34) + CgiTempFilename          + Chr(34) + " /CONSOLE /EXE " +
  		                                Chr(34) + CompilerInfo\CgiFilename + Chr(34),
  		                                "",
  		                                "",
  		                                #PB_Program_Open | #PB_Program_Read | #PB_Program_Hide)
  		
  		Protected ExitCode = 1
  		
  		If Compiler
  			
  			While ProgramRunning(Compiler)
  				If AvailableProgramOutput(Compiler)
  					AddLog(ReadProgramString(Compiler))
  				EndIf
  			Wend
  			
  			ExitCode = ProgramExitCode(Compiler)
  			
  			CloseProgram(Compiler) ; Close the connection to the program
  			
  			If ExitCode <> 0
  				
  				Addlog("")
  				Addlog("Press return to continue", #True)
  				Addlog("")            
  				
  			EndIf
  			
  			
  		Else
  			
  			Addlog("")
  			Addlog("Something went wrong :-(", #True)
  			Addlog("")
  			
  		EndIf			  
  		
  		; DeleteFile(CgiTempFilename)
  		
  	Else
  		
  		AddLog("Found no template-file", #True)
  		
  	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

Procedure.s ConvertToPHP(Code.s)
	
	Code = #LF$ + Code
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  Code = ReplaceString(Code, #CRLF$, #LF$)
  
  Lexer\SetFileString(Code)
  
  NewList TT.Token()
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(TT())
    TT()\Token = Lexer\GetNextToken() 
    TT()\Type  = Lexer\GetCurrentType() 
    TT()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected inProcedure
  
  ForEach TT()
  	Select TT()\Type
  			
  		Case #PBSC_Identifier
  			
        Select LCase(TT()\Token)
        	Case "proceduredll", "procedurecdll"
        		TT()\Token = "function"
        		inProcedure = #True
          Case "endprocedure"
          	TT()\Token = "}"
        EndSelect
        
      Case #PBSC_NewLine
      	
      	If inProcedure
      		TT()\Token = "{" + TT()\Token
      	EndIf
      	
      	inProcedure = #False
      	
      	If CheckNextToken(TT(), "!")
      		NextElement(TT())
       		TT()\Token = ""
				EndIf
      	
      Case #PBSC_Comment
      	
      	If inProcedure
      		TT()\Token = "{ //" + Mid(TT()\Token, 2)
      	EndIf
      	
      	inProcedure = #False
      	
      Default
      	If inProcedure
      		Select TT()\Token
      			Case "."
      				TT()\Token = ""
      				NextElement(TT())
      				TT()\Token = ""
      			Case "("
      				NextElement(TT())
      				If TT()\Token <> ")"
      					TT()\Token = "$" + TT()\Token
      				EndIf
      			Case ","
      				NextElement(TT())
      				TT()\Token = "$" + TT()\Token
      		EndSelect
      	EndIf
    EndSelect
  Next
  
  RemoveEmptyToken(TT())
  
	Code = ""
  
	ForEach TT()
		If CheckNextToken(TT(), ".")
			Code + TT()\Token
		ElseIf CheckNextToken(TT(), ":")
			Code + TT()\Token
		ElseIf CheckNextToken(TT(), "/")
			Code + TT()\Token
		ElseIf TT()\Token = "."
			Code + TT()\Token
		ElseIf TT()\Token = "/"
			Code + TT()\Token
		ElseIf TT()\Token = "!"
			Code + TT()\Token
		ElseIf TT()\Token = ":"
			Code + TT()\Token
		ElseIf TT()\Token = "$"
			Code + TT()\Token
		ElseIf TT()\Token = "-" And CheckNextToken(TT(), ">")
			Code + TT()\Token
		ElseIf TT()\Token = "=" And CheckNextToken(TT(), ">")
			Code + TT()\Token
		ElseIf TT()\Token = "=" And CheckNextToken(TT(), "=")
			Code + TT()\Token
		ElseIf TT()\Token = "&" And CheckNextToken(TT(), "&")
			Code + TT()\Token
		Else
			If TT()\Type = #PBSC_Number
				Code + TT()\Token
			Else
				Code + TT()\Token + " "
			EndIf
		EndIf
	Next
  
	Debug "ServerCode:>>>" + #LF$ + Code + #LF$ + "<<<"
	
	ProcedureReturn Code
	
EndProcedure
Procedure.s PreProcess_ServerCode_PHP(FileContent.s)
	
	TargetSource = #Target_PHP
	
  Protected ClientCode.s, ServerCode.s
 
  NewList PhpBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  
  Protected regex_PHP, regex_PDLL, regex_PCDLL
  
  ; EnablePHP / DisablePHP
  
  regex_PHP = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnablePHP([\s\S]*?)^[\s]*DisablePHP", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_PHP = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureCDLL()
  
	regex_PCDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureCDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PCDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ServerCode = ""
  
  ExamineRegularExpression(regex_PHP, FileContent)
  
  While NextRegularExpressionMatch(regex_PHP)
  	
  	AddElement(PhpBlock())
  	
  	PhpBlock() = RegularExpressionMatchString(regex_PHP)
  	
  	ServerCode + StringField(PhpBlock(), 2, "EnablePHP") + #LF$
  	ServerCode = StringField(ServerCode, 1, "DisablePHP") + #LF$
  	
		ExamineRegularExpression(regex_PDLL, PhpBlock())
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		Protected dataType.s = ""
		Protected processData.s = ""
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			Select Left(RemoveString(ServerProcedure(), "ProcedureDLL"), 2)
				Case ".b"
					dataType = "binary"
					processData = "false"
				Default
					dataType = "text"
					processData = "true"
			EndSelect
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; var returnValue; " + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\PhpAddress + "?request=' + arguments.callee.name, " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			ClientProcedure() + "; 	data: arguments, " + #LF$
			ClientProcedure() + "; 	dataType: '" + dataType + "', " + #LF$
			ClientProcedure() + "; 	processData: " + processData + ", " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function(result) {" + #LF$
			ClientProcedure() + "; 		returnValue = result;" + #LF$
			ClientProcedure() + "; 	}," + #LF$
			ClientProcedure() + "; 	error: function(a, b, c) {" + #LF$
			ClientProcedure() + "; 	  returnValue = 'error: ' + b + '/' + c;" + #LF$
			ClientProcedure() + "; 	}" + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "; return returnValue; " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
		
		FreeRegularExpression(regex_PDLL)
	
		ExamineRegularExpression(regex_PCDLL, PhpBlock())
		
		While NextRegularExpressionMatch(regex_PCDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PCDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			Select Left(RemoveString(ClientProcedure(), "ProcedureCDLL"), 2)
				Case ".b"
					dataType = "binary"
					processData = "false"
				Default
					dataType = "text"
					processData = "true"
			EndSelect
			
			Protected CallbackProcedure.s = StringField(ClientProcedure(), 1, "(")
			CallbackProcedure = Trim(StringField(CallbackProcedure, CountString(CallbackProcedure, " ") + 1, " "))
			CallbackProcedure + "Callback"
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\PhpAddress + "?request=' + arguments.callee.name, " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			ClientProcedure() + "; 	data: arguments, " + #LF$
			ClientProcedure() + "; 	dataType: '" + dataType + "', " + #LF$
			ClientProcedure() + "; 	processData: " + processData + ", " + #LF$
			ClientProcedure() + "; 	async: true, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			
			If dataType = "binary"
				
				ClientProcedure() + "; success: function(result) { "  + #LF$ + 
				                    ";  if (typeof " + CallbackProcedure + " == 'function') { " + #LF$ +
				                    ";   // var reader = new window.FileReader(); " + #LF$ +
				                    ";   // reader.readAsDataURL(result); " + #LF$ +
				                    ";   // reader.onloadend = function() { " + #LF$ +
				                    ";   // " + CallbackProcedure + "(true, reader.result); " + #LF$ + 
				                    ";   " + CallbackProcedure + "(true, result); " + #LF$ + 
				                    ";   // } " + #LF$ + 
				                    ";  } " + #LF$ +
				                    "; }, " + #LF$
				
			Else
				ClientProcedure() + "; 	success: function(result)  { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(true, result) }, " + #LF$
			EndIf
			
			ClientProcedure() + "; 	error: function(a, b, c) { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(false, b + '/' + c) } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend		
		
		FreeRegularExpression(regex_PCDLL)
		
		;  ForEach ServerProcedure()
		;   ServerCode + ServerProcedure() + #LF$
		;  Next
		
		ClientProcedures = ""
		ForEach ClientProcedure()
			ClientProcedures + ClientProcedure() + #LF$
		Next
		
		ClientCode = ReplaceString(ClientCode, PhpBlock(), ClientProcedures)
		
	Wend
	
	FreeRegularExpression(regex_PHP)
	
	; ----------------------------------------------------
	
  Protected TemplateCode.s
 	Protected TemplateFilename.s

  If ServerCode <> ""
  	
  	ServerCode = ConvertToPHP(ServerCode)
  	
  	If CompilerInfo\PhpFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'PhpFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\PhpAddress = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'PhpAddress' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetExePath() + "system\usertemplates\php.php") > 0
  		TemplateFilename = GetExePath() + "system\usertemplates\php.php"
  	ElseIf FileSize(GetExePath() + "system\templates\php.php") > 0
  		TemplateFilename = GetExePath() + "system\templates\php.php"
  	EndIf
  	
   	If TemplateFilename <> ""
   		TemplateCode = LoadTextFile(TemplateFilename)
   		TemplateCode = ReplaceString(TemplateCode, "// ### PB2Web ServerCode ###", ServerCode)
   		SaveTextFile(TemplateCode, CompilerInfo\PhpFilename)
   	Else
   		AddLog("no template found", #True)
   	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

Procedure.s ConvertToNodeJs(Code.s)
	
	Code = #LF$ + Code
	
	NewList ProcedureName.s()
	
	Dim TempArray.s(0)
	Split(TempArray(), Code, #LF$)
	Protected Counter
	For Counter = 0 To ArraySize(TempArray())
		TempArray(Counter) = TrimChars(TempArray(Counter))
		If StartsWith(LCase(TempArray(Counter)), "proceduredll")
			
			AddElement(ProcedureName())
			ProcedureName() = ReplaceString(TempArray(Counter), "ProcedureDLL.s ", "", #PB_String_NoCase)
			ProcedureName() = Left(ProcedureName(), FindString(ProcedureName(), "(") - 1)
			
			TempArray(Counter) = ""
			
		EndIf
		If StartsWith(LCase(TempArray(Counter)), "endprocedure")
			TempArray(Counter) = ""
		EndIf
		If StartsWith(TempArray(Counter), "!")
			TempArray(Counter) = Mid(TempArray(Counter), 2)
		EndIf
	Next
	
	Code = " var PB2NodeJsFunctions = { }; "
	Code + Join(TempArray(), #LF$)
	ForEach ProcedureName()
		Code + "PB2NodeJsFunctions." + ProcedureName() + " = " + ProcedureName() + ";" + #LF$
	Next
	
	; Debug "ServerCode:>>>" + #LF$ + Code + #LF$ + "<<<"
	
	ProcedureReturn Code
	
EndProcedure
Procedure.s PreProcess_ServerCode_NodeJs(FileContent.s)
	
	TargetSource = #Target_NodeJS
	
  Protected ClientCode.s, ServerCode.s
  
  NewList NodeJsBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  
  Protected regex_NodeJs, regex_PDLL
  
  ; EnableNodeJs / DisableNodeJs
  
  regex_NodeJs = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnableNodeJs([\s\S]*?)^[\s]*DisableNodeJs", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_NodeJs = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ServerCode = ""
  
  ExamineRegularExpression(regex_NodeJs, FileContent)
  
  While NextRegularExpressionMatch(regex_NodeJs)
  	
  	AddElement(NodeJsBlock())
  	
  	NodeJsBlock() = RegularExpressionMatchString(regex_NodeJs)
  	
		ExamineRegularExpression(regex_PDLL, NodeJsBlock())
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
		
			ClientProcedure() + "Protected ReturnValue.s" + #LF$
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\NodeJsAddress + "?request=' + arguments.callee.name, " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			ClientProcedure() + "; 	data: arguments, " + #LF$
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { ReturnValue = result }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { ReturnValue = b + '/' + c } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "ProcedureReturn ReturnValue" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
		
		ForEach ServerProcedure()
			ServerCode + ServerProcedure() + #LF$
		Next
		
		ClientProcedures = ""
		ForEach ClientProcedure()
			ClientProcedures + ClientProcedure() + #LF$
		Next
		
		ClientCode = ReplaceString(ClientCode, NodeJsBlock(), ClientProcedures)
		
	Wend
	
	FreeRegularExpression(regex_PDLL)
	FreeRegularExpression(regex_NodeJs)
	
	; ----------------------------------------------------
	
  Protected TemplateFilename.s
  Protected TemplateCode.s

  If ServerCode <> ""
  	
  	ServerCode = ConvertToNodeJs(ServerCode)
  	
  	If CompilerInfo\NodeJsFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'NodeJsFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\NodeJsAddress = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'NodeJsAddress' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	
  	If FileSize(GetExePath() + "system\usertemplates\nodejs.js") > 0
  		TemplateFilename = GetExePath() + "system\usertemplates\nodejs.js"
  	ElseIf FileSize(GetExePath() + "system\templates\nodejs.js") > 0
  		TemplateFilename = GetExePath() + "system\templates\nodejs.js"
  	EndIf  	  	
  	
   	If TemplateFilename <> ""
   		
   		Protected ServerPort.s 
   		
   		ServerPort = GetURLPart(CompilerInfo\NodeJsAddress, #PB_URL_Port)
   		If ServerPort = ""
   			ServerPort = "80"
   			If LCase(GetURLPart(CompilerInfo\NodeJsAddress, #PB_URL_Protocol))="https"
   				ServerPort = "443"
				EndIf
   		EndIf
   		
   		TemplateCode = LoadTextFile(TemplateFilename)
   		TemplateCode = ReplaceString(TemplateCode, "// ### PB2Web ServerPort ###", "var port = " + ServerPort + ";")
   		TemplateCode = ReplaceString(TemplateCode, "// ### PB2Web ServerCode ###", ServerCode)
   		
   		SaveTextFile(TemplateCode, CompilerInfo\NodeJsFilename)
   		
   		; http://nodemon.io/
   		; "nodemon " + CompilerInfo\NodeJsFilename + " -e node.js --exitcrash"
   		
   	Else
   		
   		AddLog("no template found", #True)
   		
   	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

Procedure.s ConvertToPython(Code.s)
		
	Code = #LF$ + Code
	
	Protected Lexer.iPBSC = New_PBSC() 	
	
	Protected RequestSelect.s
	
	Code = ReplaceString(Code, #CRLF$, #LF$)
	
	; Whitespaces are important!
	Code = ReplaceString(Code, " ", Chr(1))
	Code = ReplaceString(Code, #TAB$, Chr(2))
	
	Lexer\SetFileString(Code)
	
	NewList TT.Token()
	
	; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(TT())
    TT()\Token = Lexer\GetNextToken() 
    TT()\Type  = Lexer\GetCurrentType() 
    TT()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected inProcedure
  NewList vars.s()
  Protected ProcName.s
  ForEach TT()
  	Select TT()\Type
  			
  		Case #PBSC_Identifier
  			
  			Select LCase(TT()\Token)
  					
        	Case "proceduredll", "procedurecdll"
        		
        		TT()\Token = "def "
        		
        		Define LI = ListIndex(TT())
        		
        		Repeat
        			NextElement(TT())
        			If TT()\Token = "("
        				Break
        			EndIf
        		ForEver
        		Repeat
        			PreviousElement(TT())
        			If TT()\Token <> Chr(1) And TT()\Token <> Chr(2)
        				Break
        			EndIf
        		ForEver
        		
        		ProcName = TT()\Token
        		
        		SelectElement(TT(), LI)
        		
        		TT()\Token = "@app.route('/" + ProcName + "', methods=['POST'])" + #LF$ + TT()\Token

        		inProcedure = #True
        		
          Case "endprocedure"
          	TT()\Token = ""
	      		ClearList(vars())
        EndSelect
        
      Case #PBSC_NewLine
      	
      	If inProcedure
      		
      		TT()\Token = ":" + TT()\Token
      		
      		ForEach vars()
      			TT()\Token + " " + vars() + " = request.form.get('" + Str(ListIndex(vars())) + "')" + #LF$
      		Next
      		
      		ClearList(vars())
      		
      	EndIf
      	
      	inProcedure = #False
      	
      Case #PBSC_Comment
      	
      	If inProcedure
      		
      		TT()\Token = ": #" + Mid(TT()\Token, 2)
      		
      		ForEach vars()
      			TT()\Token + " " + vars() + " = request.form.get('" + Str(ListIndex(vars())) + "')" + #LF$
      		Next
      		
      		ClearList(vars())
      		
      	EndIf
      	
      	inProcedure = #False
      	
      Default
      	If inProcedure
      		Select TT()\Token
      			Case "."
      				TT()\Token = ""
      				NextElement(TT())
      				TT()\Token = ""
      			Case "("
      				NextElement(TT())
      				If TT()\Token <> ")"
      					AddElement(vars())
      					vars() = TT()\Token
      					TT()\Token = ""
      				EndIf
      			Case ","
      				TT()\Token = ""
      				NextElement(TT())
    					AddElement(vars())
    					vars() = TT()\Token
      				TT()\Token = ""
      		EndSelect
      	EndIf
      	If TT()\Token = "!"
      		TT()\Token = ""
      	EndIf
    EndSelect
  Next
  
  RemoveEmptyToken(TT())
  
	Code = ""
  
	ForEach TT()
		Code + TT()\Token
	Next
  
  Code = ReplaceString(Code, Chr(1), " ")
  Code = ReplaceString(Code, Chr(2), #TAB$)
  
  Debug "ServerCode:>>>" + #LF$ + Code + #LF$ + "<<<"
	
	ProcedureReturn Code
	
EndProcedure
Procedure.s PreProcess_ServerCode_Python(FileContent.s)
	
	TargetSource = #Target_Python
	
  Protected ClientCode.s, ServerCode.s
 
  NewList PythonBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  
  Protected regex_Python, regex_PDLL, regex_PCDLL
  
  ; EnablePython / DisablePython
  
  regex_Python = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnablePython([\s\S]*?)^[\s]*DisablePython", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_Python = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureCDLL()
  
	regex_PCDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureCDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PCDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ServerCode = ""
  
  ExamineRegularExpression(regex_Python, FileContent)
  
  While NextRegularExpressionMatch(regex_Python)
  	
  	AddElement(PythonBlock())
  	
  	PythonBlock() = RegularExpressionMatchString(regex_Python)
  	
		ExamineRegularExpression(regex_PDLL, PythonBlock())
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			ClientProcedure() + "Protected ReturnValue.s" + #LF$
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			If CompilerInfo\UseFlask
				ClientProcedure() + "; 	url: '/' + arguments.callee.name, " + #LF$
			Else  	
				ClientProcedure() + "; 	url: '" + CompilerInfo\PythonAddress + "/' + arguments.callee.name, " + #LF$
  		EndIf
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			ClientProcedure() + "; 	data: arguments, " + #LF$
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { ReturnValue = result }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { ReturnValue = b + '/' + c } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "ProcedureReturn ReturnValue" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
		
		FreeRegularExpression(regex_PDLL)
	
		ExamineRegularExpression(regex_PCDLL, PythonBlock())
		
		While NextRegularExpressionMatch(regex_PCDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PCDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			Define CallbackProcedure.s = StringField(ClientProcedure(), 1, "(")
			CallbackProcedure = Trim(StringField(CallbackProcedure, CountString(CallbackProcedure, " ") + 1, " "))
			CallbackProcedure + "Callback"
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			If CompilerInfo\UseFlask
				ClientProcedure() + "; 	url: '/' + arguments.callee.name, " + #LF$
			Else  	
				ClientProcedure() + "; 	url: '" + CompilerInfo\PythonAddress + "/' + arguments.callee.name, " + #LF$
  		EndIf
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			ClientProcedure() + "; 	data: arguments, " + #LF$
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	async: true, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(true, result) }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(false, b + '/' + c) } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend		
		
		FreeRegularExpression(regex_PCDLL)
		
		ForEach ServerProcedure()
			ServerCode + ServerProcedure() + #LF$
		Next
		
		ClientProcedures = ""
		ForEach ClientProcedure()
			ClientProcedures + ClientProcedure() + #LF$
		Next
		
		ClientCode = ReplaceString(ClientCode, PythonBlock(), ClientProcedures)
		
	Wend
	
	FreeRegularExpression(regex_Python)
	
	; ----------------------------------------------------
	
  Protected TemplateCode.s
  Protected TemplateFilename.s
  
  If ServerCode <> ""
  	
  	ServerCode = ConvertToPython(ServerCode)
  	
  	If CompilerInfo\PythonFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'PythonFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\UseFlask <> "1"
  		If CompilerInfo\PythonAddress = ""
  			AddLog("")
  			AddLog("Found ServerCode but 'PythonAddress' is empty", #True)
  			AddLog("")
  			ProcedureReturn ""
  		EndIf
  	EndIf
  	
  	If CompilerInfo\UseFlask = "1"
  		If FileSize(GetExePath() + "system\usertemplates\flask.py") > 0
  			TemplateFilename = GetExePath() + "system\usertemplates\flask.py"
  		ElseIf FileSize(GetExePath() + "system\templates\flask.py") > 0
  			TemplateFilename = GetExePath() + "system\templates\flask.py"
  		EndIf
  	Else
  		If FileSize(GetExePath() + "system\usertemplates\python.py") > 0
  			TemplateFilename = GetExePath() + "system\usertemplates\python.py"
  		ElseIf FileSize(GetExePath() + "system\templates\python.py") > 0
  			TemplateFilename = GetExePath() + "system\templates\python.py"
  		EndIf
  	EndIf
  	
   	If TemplateFilename <> ""
  	
   		TemplateCode = LoadTextFile(TemplateFilename)
   		
   		TemplateCode = ReplaceString(TemplateCode, "# ### PB2Web ServerCode ###", ServerCode)
   		TemplateCode = ReplaceString(TemplateCode, "#FLASKPORT#", CompilerInfo\FlaskPort)
   		
   		SaveTextFile(TemplateCode, CompilerInfo\PythonFilename)
   		
   	Else
   		
   		AddLog("no template found", #True)
   		
   	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

Procedure.s ConvertToASP(Code.s)
	
	Code = #LF$ + Code
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  Code = ReplaceString(Code, #CRLF$, #LF$)
  
  Lexer\SetFileString(Code)
  
  NewList TT.Token()
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(TT())
    TT()\Token = Lexer\GetNextToken() 
    TT()\Type  = Lexer\GetCurrentType() 
    TT()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected inProcedure
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
        	Case "proceduredll", "procedurecdll"
        		TT()\Token = "Function"
        		inProcedure = #True
          Case "endprocedure"
          	TT()\Token = "End Function"
        EndSelect
    	Case #PBSC_NewLine, #PBSC_Comment
    		inProcedure = #False
      Default
      	If inProcedure
      		If TT()\Token = "."
      			TT()\Token = ""
      			NextElement(TT())
      			TT()\Token = ""
      		EndIf
      	EndIf
      	If TT()\Token = "!"
      		TT()\Token = ""
      	EndIf
    EndSelect
  Next
  
  RemoveEmptyToken(TT())
  
	Code = ""
  
	ForEach TT()
		If CheckNextToken(TT(), ".")
			Code + TT()\Token
		ElseIf TT()\Token = "."
			Code + TT()\Token
		Else
			Code + TT()\Token + " "
		EndIf
	Next
  
	Debug "ServerCode:>>>" + #LF$ + Code + #LF$ + "<<<"
	
	ProcedureReturn Code
	
EndProcedure
Procedure.s PreProcess_ServerCode_ASP(FileContent.s)
	
	TargetSource = #Target_ASP
	
  Protected ClientCode.s, ServerCode.s
 
  NewList ASPBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  
  Protected ProcName.String 
  Protected Params.String
  Protected CallbackProcedure.s
  
  Protected regex_ASP, regex_PDLL, regex_PCDLL
  
  ; EnableASP / DisableASP
  
  regex_ASP = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnableASP([\s\S]*?)^[\s]*DisableASP", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_ASP = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureCDLL()
  
	regex_PCDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureCDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PCDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ServerCode = ""
  
  ExamineRegularExpression(regex_ASP, FileContent)
  
  While NextRegularExpressionMatch(regex_ASP)
  	
  	AddElement(ASPBlock())
  	
  	ASPBlock() = RegularExpressionMatchString(regex_ASP)
  	
		ExamineRegularExpression(regex_PDLL, ASPBlock())
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			; -----------------
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			; -----------------
			
			ClientProcedure() + "Protected ReturnValue.s" + #LF$
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\ASPAddress + "?request=" + ProcName\s + "', " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If Params\s <> ""
				ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
			EndIf
			
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { ReturnValue = result }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { ReturnValue = b + '/' + c } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "ProcedureReturn ReturnValue" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
		
		FreeRegularExpression(regex_PDLL)
	
		ExamineRegularExpression(regex_PCDLL, ASPBlock())
		
		While NextRegularExpressionMatch(regex_PCDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PCDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			; -----------------
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			; -----------------
			
			CallbackProcedure = ProcName\s + "Callback"
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\ASPAddress + "?request=" + ProcName\s + "', " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If Params\s <> ""
				ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
			EndIf
			
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	async: true, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(true, result) }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(false, b + '/' + c) } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend		
		
		FreeRegularExpression(regex_PCDLL)
		
		ForEach ServerProcedure()
			ServerCode + ServerProcedure() + #LF$
		Next
		
		ClientProcedures = ""
		ForEach ClientProcedure()
			ClientProcedures + ClientProcedure() + #LF$
		Next
		
		ClientCode = ReplaceString(ClientCode, ASPBlock(), ClientProcedures)
		
	Wend
	
	FreeRegularExpression(regex_ASP)
	
	; ----------------------------------------------------
	
  Protected TemplateFilename.s
  Protected TemplateCode.s
  Protected RequestSelect.s
  
  If ServerCode <> ""
  	
		RequestSelect = GetRequestSelect4ASP(ServerCode)
   		
   	Debug RequestSelect
   		
  	ServerCode = ConvertToASP(ServerCode)
  	
  	If CompilerInfo\ASPFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'ASPFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetPathPart(CompilerInfo\ASPFilename)) <> -2
  		AddLog("")
  		AddLog("Folder '" + GetPathPart(CompilerInfo\ASPFilename) + " doesn't exist", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\ASPAddress = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'ASPAddress' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetExePath() + "system\usertemplates\asp.asp") > 0
  		TemplateFilename = GetExePath() + "system\usertemplates\asp.asp"
  	ElseIf FileSize(GetExePath() + "system\templates\asp.asp") > 0
  		TemplateFilename = GetExePath() + "system\templates\asp.asp"
  	EndIf  	
  	
   	If TemplateFilename <> ""
   		
   		TemplateCode = LoadTextFile(TemplateFilename)
   		TemplateCode = ReplaceString(TemplateCode, "' ### PB2Web ServerCode ###", ServerCode)
   		TemplateCode = ReplaceString(TemplateCode, "' ### PB2Web RequestSelect ###", RequestSelect)
   		
   		SaveTextFile(TemplateCode, CompilerInfo\ASPFilename)
   		
   	Else
   		
   		AddLog("no template found", #True)
   		
   	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

Procedure.s ConvertToASPX(Code.s)
	
	Code = #LF$ + Code
	
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected RequestSelect.s
  
  Code = ReplaceString(Code, #CRLF$, #LF$)
  
  Lexer\SetFileString(Code)
  
  NewList TT.Token()
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(TT())
    TT()\Token = Lexer\GetNextToken() 
    TT()\Type  = Lexer\GetCurrentType() 
    TT()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Protected inProcedure
  
  ForEach TT()
    Select TT()\Type
      Case #PBSC_Identifier
        Select LCase(TT()\Token)
        	Case "proceduredll", "procedurecdll"
        		TT()\Token = "Function"
        		inProcedure = #True
          Case "endprocedure"
          	TT()\Token = "End Function"
        EndSelect
    	Case #PBSC_NewLine, #PBSC_Comment
    		inProcedure = #False
      Default
      	If inProcedure
      		If TT()\Token = "."
      			TT()\Token = ""
      			NextElement(TT())
      			TT()\Token = ""
      		EndIf
      	EndIf
      	If TT()\Token = "!"
      		TT()\Token = ""
      	EndIf
    EndSelect
  Next
  
  RemoveEmptyToken(TT())
  
	Code = ""
  
	ForEach TT()
		If CheckNextToken(TT(), ".")
			Code + TT()\Token
		ElseIf TT()\Token = "."
			Code + TT()\Token
		Else
			Code + TT()\Token + " "
		EndIf
	Next
  
	Debug "ServerCode:>>>" + #LF$ + Code + #LF$ + "<<<"
	
	ProcedureReturn Code
	
EndProcedure
Procedure.s PreProcess_ServerCode_ASPX(FileContent.s)
	
	TargetSource = #Target_ASPX
	
  Protected ClientCode.s, ServerCode.s
 
  NewList AspxBlock.s()
  NewList ServerProcedure.s()
  
  NewList   ClientProcedure.s()
  Protected ClientProcedures.s
  
  Protected ProcName.String 
  Protected Params.String
  Protected CallbackProcedure.s
  
  Protected regex_ASPX, regex_PDLL, regex_PCDLL
  
  ; EnableASPX / DisableASPX
  
  regex_ASPX = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*EnableASPX([\s\S]*?)^[\s]*DisableASPX", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
  
  If regex_ASPX = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureDLL()
  
	regex_PDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
 	; ProcedureCDLL()
  
	regex_PCDLL = CreateRegularExpression(#PB_Any, "^[\t]*[\ ]*ProcedureCDLL([\s\S]*?)\(([\s\S]*?)^[\s]*EndProcedure", #PB_RegularExpression_MultiLine | #PB_RegularExpression_NoCase)	
	
	If regex_PCDLL = 0
  	Debug RegularExpressionError()
  	ProcedureReturn ""
  EndIf
  
  ; ----
  
  ClientCode = FileContent
  
  ServerCode = ""
  
  ExamineRegularExpression(regex_ASPX, FileContent)
  
  While NextRegularExpressionMatch(regex_ASPX)
  	
  	AddElement(AspxBlock())
  	
  	AspxBlock() = RegularExpressionMatchString(regex_ASPX)
  	
		ExamineRegularExpression(regex_PDLL, AspxBlock())
		
		ClearList(ServerProcedure())
		ClearList(ClientProcedure())
		
		While NextRegularExpressionMatch(regex_PDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			; -----------------
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			; -----------------
			
			ClientProcedure() + "Protected ReturnValue.s" + #LF$
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\AspxAddress + "?request=" + ProcName\s + "', " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If Params\s <> ""
				ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
			EndIf
			
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	async: false, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { ReturnValue = result }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { ReturnValue = b + '/' + c } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "ProcedureReturn ReturnValue" + #LF$
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend
		
		FreeRegularExpression(regex_PDLL)
	
		ExamineRegularExpression(regex_PCDLL, AspxBlock())
		
		While NextRegularExpressionMatch(regex_PCDLL)
			
			AddElement(ServerProcedure())
			ServerProcedure() = RegularExpressionMatchString(regex_PCDLL)
			
			AddElement(ClientProcedure())
			ClientProcedure() = StringField(ServerProcedure(), 1, #LF$) + #LF$
			
			; -----------------
			
			ProcName\s = ""
			Params\s = ""
			
			GetParamsFromServerProcedure(ClientProcedure(), ProcName, Params)
			
			; -----------------
			
			CallbackProcedure = ProcName\s + "Callback"
			
			ClientProcedure() + "EnableJs" + #LF$
			ClientProcedure() + "; $.ajax({ " + #LF$
			ClientProcedure() + "; 	url: '" + CompilerInfo\AspxAddress + "?request=" + ProcName\s + "', " + #LF$
			ClientProcedure() + "; 	type: 'POST', " + #LF$
			
			If Params\s <> ""
				ClientProcedure() + "; 	data: { " + Params\s + " }, " + #LF$
			EndIf
			
			ClientProcedure() + "; 	dataType: 'text', " + #LF$
			ClientProcedure() + "; 	async: true, " + #LF$
			ClientProcedure() + "; 	cache: false, " + #LF$
			ClientProcedure() + "; 	success: function (result)  { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(true, result) }, " + #LF$
			ClientProcedure() + "; 	error:   function (a, b, c) { if (typeof " + CallbackProcedure + " == 'function') " + CallbackProcedure + "(false, b + '/' + c) } " + #LF$
			ClientProcedure() + "; }); " + #LF$
			ClientProcedure() + "DisableJs" + #LF$
			
			ClientProcedure() + "EndProcedure" + #LF$  				
			
		Wend		
		
		FreeRegularExpression(regex_PCDLL)
		
		ForEach ServerProcedure()
			ServerCode + ServerProcedure() + #LF$
		Next
		
		ClientProcedures = ""
		ForEach ClientProcedure()
			ClientProcedures + ClientProcedure() + #LF$
		Next
		
		ClientCode = ReplaceString(ClientCode, AspxBlock(), ClientProcedures)
		
	Wend
	
	FreeRegularExpression(regex_ASPX)
	
	; ----------------------------------------------------
	
  Protected TemplateFilename.s
  Protected TemplateCode.s
  Protected RequestSelect.s
  
  If ServerCode <> ""
  	
		RequestSelect = GetRequestSelect4ASPX(ServerCode)
   		
   	Debug RequestSelect
   		
  	ServerCode = ConvertToASPX(ServerCode)
  	
  	If CompilerInfo\AspxFilename = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'AspxFilename' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetPathPart(CompilerInfo\AspxFilename)) <> -2
  		AddLog("")
  		AddLog("Folder '" + GetPathPart(CompilerInfo\AspxFilename) + " doesn't exist", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If CompilerInfo\AspxAddress = ""
  		AddLog("")
  		AddLog("Found ServerCode but 'AspxAddress' is empty", #True)
  		AddLog("")
  		ProcedureReturn ""
  	EndIf
  	
  	If FileSize(GetExePath() + "system\usertemplates\aspx.aspx") > 0
  		TemplateFilename = GetExePath() + "system\usertemplates\aspx.aspx"
  	ElseIf FileSize(GetExePath() + "system\templates\aspx.aspx") > 0
  		TemplateFilename = GetExePath() + "system\templates\aspx.aspx"
  	EndIf  	
  	
   	If TemplateFilename <> ""
   		
   		TemplateCode = LoadTextFile(TemplateFilename)
   		TemplateCode = ReplaceString(TemplateCode, "' ### PB2Web ServerCode ###", ServerCode)
   		TemplateCode = ReplaceString(TemplateCode, "' ### PB2Web RequestSelect ###", RequestSelect)
   		
   		SaveTextFile(TemplateCode, CompilerInfo\AspxFilename)
   		
   	Else
   		
   		AddLog("no template found", #True)
   		
   	EndIf
  	
  EndIf
  
  ProcedureReturn ClientCode
  
EndProcedure

; ------------------

Procedure PreProcess_GetCompilerInfos(FileContent.s)
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  Protected is_P2W_Converter_SetProfile
  
  FileContent = ReplaceString(FileContent, #CRLF$, #LF$)
  
  Lexer\SetFileString(FileContent)
  
  ClearList(Token())
  
  ; red Token 
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)
  
  Structure sParam
  	Name.s
  	Value.s
  EndStructure
  	
  NewList Params.sParam()
  
  ForEach Token()
    
    Select Token()\Type
        
      Case #PBSC_Identifier
        
        Select LCase(Token()\Token)
            
          Case LCase("P2W_Converter_SetProfile")
            is_P2W_Converter_SetProfile = #True
            
          Case LCase("P2W_Converter_DoNotCopy")
            CompilerInfo\DoNotCopy = #True
            
          Case LCase("P2W_SetConfig")
          	
          	AddElement(Params())
          	
          	NextElement(Token()) ; (
          	
          	NextElement(Token()) ; ParamName
          	Params()\Name = RemoveString(Token()\Token, Chr(34))
            
          	NextElement(Token()) ; ,

          	NextElement(Token()) ; ParamValue
          	Params()\Value = RemoveString(Token()\Token, Chr(34))
          	
          	Select LCase(Params()\Value)
          		Case "#true", "true"
          			Params()\Value = "1"
          	EndSelect
            
        EndSelect
        
      Case #PBSC_String
        
        If is_P2W_Converter_SetProfile = #True
          is_P2W_Converter_SetProfile = #False  
          CompilerInfo\ProfileName = RemoveString(Token()\Token, Chr(34))
        EndIf
        
    EndSelect
    
  Next
  
  ForEach Params()
  	
  	Select LCase(Params()\Name)
  			
  		Case LCase("#P2W_Config_ApplicationName") : CompilerInfo\ApplicationNameUser = Params()\Value

  		Case LCase("#P2W_Config_ServerAddress")   : CompilerInfo\ServerAddressUser   = Params()\Value
  		Case LCase("#P2W_Config_TargetFolder")    : CompilerInfo\TargetFolderUser    = Params()\Value
  			
  		Case LCase("#P2W_Config_ServerAddressJS") : CompilerInfo\ServerAddressJS     = Params()\Value
  		Case LCase("#P2W_Config_TargetFolderJS")  : CompilerInfo\TargetFolderJS      = Params()\Value
  			
  		Case LCase("#P2W_Config_AspFilename")     : CompilerInfo\AspFilenameUser     = Params()\Value
  		Case LCase("#P2W_Config_AspAddress")      : CompilerInfo\AspAddressUser	     = Params()\Value
  			
  		Case LCase("#P2W_Config_AspxFilename")    : CompilerInfo\AspxFilenameUser    = Params()\Value
  		Case LCase("#P2W_Config_AspxAddress")     : CompilerInfo\AspxAddressUser	   = Params()\Value
  			
  		Case LCase("#P2W_Config_CgiFilename")     : CompilerInfo\CgiFilenameUser	   = Params()\Value
  		Case LCase("#P2W_Config_CgiAddress")      : CompilerInfo\CgiAddressUser      = Params()\Value
  			
  		Case LCase("#P2W_Config_PhpFilename")     : CompilerInfo\PhpFilenameUser     = Params()\Value
  		Case LCase("#P2W_Config_PhpAddress")      : CompilerInfo\PhpAddressUser	     = Params()\Value
  			
  		Case LCase("#P2W_Config_NodeJsFilename")  : CompilerInfo\NodeJsFilenameUser  = Params()\Value
  		Case LCase("#P2W_Config_NodeJsAddress")   : CompilerInfo\NodeJsAddressUser   = Params()\Value
  			
  		Case LCase("#P2W_Config_PythonFilename")  : CompilerInfo\PythonFilenameUser  = Params()\Value
  		Case LCase("#P2W_Config_PythonAddress")   : CompilerInfo\PythonAddressUser   = Params()\Value
  			
  		Case LCase("#P2W_Config_UseFlask")        : CompilerInfo\UseFlaskUser        = Params()\Value
  		Case LCase("#P2W_Config_FlaskPort")       : CompilerInfo\FlaskPortUser       = Params()\Value
  			
  		Default
  			
  			AddLog("")
  			AddLog("Unknown Config-Parameter found: '" + Params()\Name + "'", #True)
  			AddLog("")
  			
  	EndSelect
  	
  Next
  
EndProcedure

Procedure.s PostProcess_Raw(FileContent.s)
  
  Protected beginRaw
  Protected CountLines
  Protected LineCounter
  Protected Line.s 
  Protected Output.s
  
  CountLines = CountString(FileContent, #LF$)
  
  For LineCounter = 1 To CountLines
    
    Line = Trim(StringField(FileContent, LineCounter, #LF$))
    
    If FindString(LCase(Line), LCase("P2W_Converter_Begin_Raw")) Or 
       FindString(LCase(Line), LCase("P2W_EnableJS")) Or
       FindString(LCase(Line), LCase("EnableJS"))
    	
    	beginRaw=#True
    	
    ElseIf FindString(LCase(Line), LCase("P2W_Converter_End_Raw")) Or 
           FindString(LCase(Line), LCase("P2W_DisableJS")) Or
           FindString(LCase(Line), LCase("DisableJS")) 
    	
    	beginRaw=#False
    	
    EndIf
    
    If beginRaw
      If Left(Line, 1)=";"
        Line = Mid(Line, 2)
      EndIf
    EndIf
    
    Output + Line + #LF$
    
  Next
  
  ProcedureReturn Output	
  
EndProcedure

Procedure PostProcess_RemoveCompilerDirectives(FileContent.s)

  Protected isDirective
  
  Protected Lexer.iPBSC = New_PBSC() 	
  
  FileContent = ReplaceString(FileContent, #CRLF$, #LF$)
  
  Lexer\SetFileString(FileContent)
  
  ClearList(Token())
  
  ; Token einlesen
  While Lexer\IsNextToken() 
    AddElement(Token())
    Token()\Token = Lexer\GetNextToken() 
    Token()\Type  = Lexer\GetCurrentType() 
    Token()\Line = Lexer\GetCurrentLineNb()
  Wend
  
  PBSC_CloseFile(Lexer)  
  
  ForEach Token()
    Select Token()\Type
      Case #PBSC_Identifier
        Select LCase(Token()\Token)
            
          Case LCase("P2W_Converter_SetProfile")
            isDirective = #True
            Token()\Token = "// P2W_Converter_SetProfile"
            Token()\Type = #PBSC_Comment
            
          Case LCase("P2W_Converter_Begin_Raw")
            isDirective = #True
            Token()\Token = "// P2W_Converter_Begin_Raw"
            Token()\Type = #PBSC_Comment
            
          Case LCase("P2W_EnableJS")
            isDirective = #True
            Token()\Token = "// P2W_EnableJS"
            Token()\Type = #PBSC_Comment
            
          Case LCase("EnableJS")
            isDirective = #True
            Token()\Token = "// EnableJS"
            Token()\Type = #PBSC_Comment
            
          Case LCase("P2W_Converter_End_Raw")
            isDirective = #True
            Token()\Token = "// P2W_Converter_End_Raw"
            Token()\Type = #PBSC_Comment
            
          Case LCase("P2W_DisableJS")
            isDirective = #True
            Token()\Token = "// P2W_DisableJS"
            Token()\Type = #PBSC_Comment
            
          Case LCase("DisableJS")
            isDirective = #True
            Token()\Token = "// DisableJS"
            Token()\Type = #PBSC_Comment
            
          Case LCase("P2W_Converter_DoNotCopy")
            isDirective = #True
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
          Case LCase("P2W_SetConfig")
            isDirective = #True
            Token()\Token = ""
            Token()\Type = #PBSC_Other
            
        EndSelect
        
      Case #PBSC_NewLine
        isDirective = #False
        
    EndSelect
    
    If isDirective And Token()\Type <> #PBSC_Comment
      Token()\Token = ""
      Token()\Type = #PBSC_Other
    EndIf
    
  Next
  
EndProcedure

Procedure.s PostProcess_Finish(List TT.Token())
  
  AddLog("PostProcess: Finishing...")

  Protected Output.s
  Protected previousToken.s = ""
  Protected isMatch
  Protected beginRaw
  Protected TestIfNumber.s
  
  ForEach TT()
  	
  		If LCase(TT()\Token) = "match"
  			isMatch=#True
  		EndIf
  		
  	If TT()\Type = #PBSC_NewLine Or TT()\Type = #PBSC_Comment
  		
  		Output + TT()\Token 
  		
  		isMatch=#False
  		
  	Else
  		
  		If TT()\Token = "-" Or TT()\Token = "+"
  			
  			NextElement(TT())
    		
    		TestIfNumber = TT()\Token
    		
    		PreviousElement(TT())
    		
    		If IsNumeric(TestIfNumber)
    			Output + TT()\Token 
    		Else
    			If isMatch
    				Output + TT()\Token
    			Else
    				Output + TT()\Token + " "
    			EndIf
    		EndIf
    		
    	Else
    		
    		If CheckNextToken(TT(), ".") Or TT()\Token = "." Or 
    		   CheckNextToken(TT(), "(") Or TT()\Token  = "(" Or
    		   CheckNextToken(TT(), "[") Or TT()\Token  = "[" Or
    		   CheckNextToken(TT(), "]") Or TT()\Token  = "]" Or
    		   CheckNextToken(TT(), ")") 
    			
    			Output + TT()\Token 
    			
    		Else
    			
    			If TT()\Type = #PBSC_Number
    				Output + TT()\Token 
    			Else
    				If isMatch
    					Output + TT()\Token
    				Else
    					Output + TT()\Token + " "
    				EndIf
    			EndIf
    			
    		EndIf
    		
    	EndIf
    	
    EndIf
    
  Next
  
  ; var-var-Problem
  
  While FindString(Output, #LF$ + #LF$)
    Output = ReplaceString(Output, #LF$ + #LF$, #LF$)
  Wend
  
  Protected CountLines
  Protected LineCounter
  Protected Line.s 
  Protected NewOutput.s
  
  CountLines = CountString(Output, #LF$)
  
  For LineCounter = 1 To CountLines
    
    Line = Trim(StringField(Output, LineCounter, #LF$))
    
    If FindString(LCase(Line), LCase("P2W_Converter_Begin_Raw")) Or 
       FindString(LCase(Line), LCase("P2W_EnableJS")) Or
       FindString(LCase(Line), LCase("EnableJS"))
      beginRaw=#True
      
    ElseIf FindString(LCase(Line), LCase("P2W_Converter_End_Raw")) Or 
           FindString(LCase(Line), LCase("P2W_DisableJS")) Or
           FindString(LCase(Line), LCase("DisableJS")) 
      beginRaw=#False
    EndIf
    
    
    If Not EndsWith(Line, ";") And 
       Not EndsWith(Line, "{") And 
       Not EndsWith(Line, ":") And 
       Not EndsWith(Line, "(") And
       Not beginRaw And Line <> ""
      
      Line + ";" + #LF$
      
    EndIf
    
    If EndsWith(Line, ";") Or 
       EndsWith(Line, "{") Or
       EndsWith(Line, ":") Or
       EndsWith(Line, "(") Or
       beginRaw Or Line = ""
      
      Line + #LF$
      
    EndIf
    
    NewOutput + Line
    
  Next
  
  ; dirrrrty!
  
  NewOutput = ReplaceString(NewOutput, "/ /", "//")
  NewOutput = ReplaceString(NewOutput, "= = =", "===")
  NewOutput = ReplaceString(NewOutput, "! =", "!=")
  NewOutput = ReplaceString(NewOutput, "+ =", "+=")
  NewOutput = ReplaceString(NewOutput, "/ =", "/=")
  NewOutput = ReplaceString(NewOutput, "* =", "*=")
  NewOutput = ReplaceString(NewOutput, "- =", "-=")
  NewOutput = ReplaceString(NewOutput, "= =", "==")
  NewOutput = ReplaceString(NewOutput, "| =", "|=")
  NewOutput = ReplaceString(NewOutput, "& =", "&=")
  NewOutput = ReplaceString(NewOutput, "| |", "||")
  NewOutput = ReplaceString(NewOutput, "& &", "&&")
  NewOutput = ReplaceString(NewOutput, "||(", "|| (")
  NewOutput = ReplaceString(NewOutput, "if(", "if (")
  NewOutput = ReplaceString(NewOutput, "$ ", "$")
  NewOutput = ReplaceString(NewOutput, "// ;", "; //")
  
  
;   NewOutput = ReplaceString(NewOutput, ",;" + #LF$ + "} ;", #LF$ + "} ;")
;   NewOutput = ReplaceString(NewOutput, ",;" + #LF$, "," + #LF$)
  
  NewOutput = ReplaceString(NewOutput, "cCommonP2W_", "cCommon.P2W_")
  
  NewOutput = ReplaceString(NewOutput, "var var", "var")
  
  ; ///////////////////
  
  Output = NewOutput
  
  NewOutput = ""
  
  CountLines = CountString(Output, #LF$)
  
  For LineCounter = 1 To CountLines
    Line = Trim(StringField(Output, LineCounter, #LF$))
    If StartsWith(Line, "//") Or StartsWith(Line, "; //") 
    Else
      NewOutput + Line + #LF$
    EndIf
  Next
  
  ; ///////////////////
 
  ProcedureReturn NewOutput
  
EndProcedure

; //

If Not(#PB_Compiler_IsIncludeFile)
	
	OpenConsole()
	
	If FileSize("C:\Users\Pitta\Dropbox\PB2Web\PB2WebPB\testsource.pb") > 0
		SourceFile = "C:\Users\Pitta\Dropbox\PB2Web\PB2WebPB\testsource.pb"
	Else
		SourceFile = "C:\Users\tuebben\Dropbox\PB2Web\PB2WebPB\testsource.pb"
	EndIf
	
	
	
	Define FileContent.s
	
	FileContent = LoadTextFile(SourceFile)
	
	FileContent = PreProcess_IgnoreParts(FileContent)
	
	FileContent = PreProcess_IncludeFile(FileContent)
	
	FileContent = PreProcess_IgnoreParts(FileContent)
		
	PreProcess_GetCompilerInfos(FileContent)
	
	PbCompiler = "C:\Program Files (x86)\PureBasic\Compilers\pbcompiler.exe"
	
	CompilerInfo\CgiFilename = "d:\xampp\cgi-bin\pb2webtest.exe"
	CompilerInfo\CgiAddress = "http://localhost/cgi-bin/pb2webtest.exe"
	
	; ---
	; ServerCode
	; ---
	
	FileContent = PreProcess_ServerCode_CGI(FileContent)
	FileContent = PreProcess_ServerCode_PHP(FileContent)
	FileContent = PreProcess_ServerCode_Python(FileContent)
	FileContent = PreProcess_ServerCode_NodeJs(FileContent)
	FileContent = PreProcess_ServerCode_ASPX(FileContent)
	FileContent = PreProcess_ServerCode_ASP(FileContent)
	
	; ---
	
	FileContent = PreProcess_RemoveMacros(FileContent)
	
  FileContent = PreProcess_InlineJS(FileContent)
  
	FileContent = PreProcess_DataSection(FileContent)
	
	FileContent = PreProcess_IncludeBinary(FileContent)
	
	FileContent = PreProcess_MoveGlobalsOutOfProcedures(FileContent)
	
	If FileContent <> ""
		
		FileContent = Convert(FileContent)
		
		FileContent = PostProcess_Raw(FileContent)
		
		PostProcess_RemoveCompilerDirectives(FileContent)
		
		FileContent = PostProcess_Finish(Token())
		
	Else
		
		Debug "!!!"
		
	EndIf
	
	Debug "ClientCode: >>>" + #CRLF$ + FileContent + #CRLF$ + "<<<"
	
	; MessageRequester("!", FileContent)
	
	CloseConsole()
	
EndIf
; IDE Options = PureBasic 5.41 LTS Beta 1 (Windows - x86)
; ExecutableFormat = Console
; CursorPosition = 8102
; FirstLine = 5753
; Folding = MYBvxagfQ3P+--4-
; EnableXP